@@ -23,6 +23,12 @@
 
 #include "types/translatablestring.h"
 
+#include "muse_framework_config.h"
+
+#ifdef MUSE_MODULE_WORKSPACE
+#include "workspace/view/workspacesmenumodel.h"
+#endif
+
 #include "log.h"
 
 using namespace muse;
@@ -45,6 +51,9 @@ static QString makeId(const ActionCode& actionCode, int itemIndex)
 AppMenuModel::AppMenuModel(QObject* parent)
     : AbstractMenuModel(parent)
 {
+#ifdef MUSE_MODULE_WORKSPACE
+    m_workspacesMenuModel = std::make_shared<WorkspacesMenuModel>(this);
+#endif
 }
 
 void AppMenuModel::load()
@@ -53,6 +62,10 @@ void AppMenuModel::load()
 
     AbstractMenuModel::load();
 
+#ifdef MUSE_MODULE_WORKSPACE
+    m_workspacesMenuModel->load();
+#endif
+
     MenuItemList items {
         makeFileMenu(),
         makeEditMenu(),
@@ -60,7 +73,7 @@ void AppMenuModel::load()
         makeAddMenu(),
         makeFormatMenu(),
         makeToolsMenu(),
-        makePluginsMenu(),
+        makePluginsMenu()
     };
 
     if (globalConfiguration()->devModeEnabled()) {
@@ -103,15 +116,12 @@ void AppMenuModel::setupConnections()
         recentScoreListItem.setSubitems(recentScoresList);
     });
 
-    workspacesManager()->currentWorkspaceChanged().onNotify(this, [this]() {
+#ifdef MUSE_MODULE_WORKSPACE
+    connect(m_workspacesMenuModel.get(), &WorkspacesMenuModel::itemsChanged, this, [this]() {
         MenuItem& workspacesItem = findMenu("menu-workspaces");
-        workspacesItem.setSubitems(makeWorkspacesItems());
-    });
-
-    workspacesManager()->workspacesListChanged().onNotify(this, [this]() {
-        MenuItem& workspacesItem = findMenu("menu-workspaces");
-        workspacesItem.setSubitems(makeWorkspacesItems());
+        workspacesItem.setSubitems(m_workspacesMenuModel->items());
     });
+#endif
 
     extensionsProvider()->manifestListChanged().onNotify(this, [this]() {
         MenuItem& pluginsMenu = findMenu("menu-plugins");
@@ -149,6 +159,27 @@ void AppMenuModel::onActionsStateChanges(const muse::actions::ActionCodeList& co
     }
 }
 
+bool AppMenuModel::isMuseSamplerModuleAdded() const
+{
+#ifdef MUSE_MODULE_MUSESAMPLER
+    return museSamplerInfo() != nullptr;
+#else
+    return false;
+#endif
+}
+
+MenuItemList AppMenuModel::makeChordAndFretboardDiagramsItems()
+{
+    MenuItemList items {
+        makeMenuItem("chord-text"),
+        makeMenuItem("add-fretboard-diagram"),
+        makeSeparator(),
+        makeMenuItem("insert-fretframe", TranslatableString("appshell/menu/add/chordandfret", "Fretboard diagram legend"))
+    };
+
+    return items;
+}
+
 MenuItem* AppMenuModel::makeMenuItem(const ActionCode& actionCode, MenuItemRole menuRole)
 {
     MenuItem* item = makeMenuItem(actionCode);
@@ -169,20 +200,19 @@ MenuItem* AppMenuModel::makeFileMenu()
         makeMenuItem("file-new"),
         makeMenuItem("file-open"),
         makeMenu(TranslatableString("appshell/menu/file", "Open &recent"), recentScoresList, "menu-file-open", openRecentEnabled),
-        makeSeparator(),
         makeMenuItem("file-close"),
+        makeSeparator(),
         makeMenuItem("file-save"),
         makeMenuItem("file-save-as"),
-        makeMenuItem("file-save-a-copy"),
-        makeMenuItem("file-save-selection"),
         makeMenuItem("file-save-to-cloud"),
-        makeMenuItem("file-publish"),
+        makeMenu(TranslatableString("appshell/menu/file", "Save o&ther"), makeSaveOtherSubItems()),
+        makeMenu(TranslatableString("appshell/menu/file", "Pu&blish online"), makePublishOnlineSubItems()),
         makeSeparator(),
         makeMenuItem("file-import-pdf"),
         makeMenuItem("file-export"),
-        makeMenuItem("file-share-audio"),
         makeSeparator(),
         makeMenuItem("project-properties"),
+        makeMenuItem("parts", TranslatableString("action", "Parts…")),
         makeSeparator(),
         makeMenuItem("print"),
         makeSeparator(),
@@ -192,6 +222,24 @@ MenuItem* AppMenuModel::makeFileMenu()
     return makeMenu(TranslatableString("appshell/menu/file", "&File"), fileItems, "menu-file");
 }
 
+MenuItemList AppMenuModel::makeSaveOtherSubItems()
+{
+    MenuItemList subItems {
+        makeMenuItem("file-save-a-copy"),
+        makeMenuItem("file-save-selection"),
+    };
+    return subItems;
+}
+
+MenuItemList AppMenuModel::makePublishOnlineSubItems()
+{
+    MenuItemList subItems {
+        makeMenuItem("file-publish"),
+        makeMenuItem("file-share-audio"),
+    };
+    return subItems;
+}
+
 MenuItem* AppMenuModel::makeEditMenu()
 {
     MenuItemList editItems {
@@ -264,14 +312,18 @@ MenuItem* AppMenuModel::makeViewMenu()
         makeMenuItem("playback-setup"),
         //makeMenuItem("toggle-scorecmp-tool"), // not implemented
         makeSeparator(),
-        makeMenu(TranslatableString("appshell/menu/view", "&Toolbars"), makeToolbarsItems(), "menu-toolbars"),
-        makeMenu(TranslatableString("appshell/menu/view", "W&orkspaces"), makeWorkspacesItems(), "menu-workspaces"),
-        makeSeparator(),
-        makeMenu(TranslatableString("appshell/menu/view", "&Show"), makeShowItems(), "menu-show"),
-        makeSeparator(),
-        makeMenuItem("dock-restore-default-layout")
+        makeMenu(TranslatableString("appshell/menu/view", "&Toolbars"), makeToolbarsItems(), "menu-toolbars")
     };
 
+#ifdef MUSE_MODULE_WORKSPACE
+    viewItems << makeMenu(TranslatableString("appshell/menu/view", "W&orkspaces"), m_workspacesMenuModel->items(), "menu-workspaces"),
+#endif
+
+    viewItems << makeSeparator()
+              << makeMenu(TranslatableString("appshell/menu/view", "&Show"), makeShowItems(), "menu-show")
+              << makeSeparator()
+              << makeMenuItem("dock-restore-default-layout");
+
     return makeMenu(TranslatableString("appshell/menu/view", "&View"), viewItems, "menu-view");
 }
 
@@ -284,8 +336,10 @@ MenuItem* AppMenuModel::makeAddMenu()
         makeSeparator(),
         makeMenu(TranslatableString("appshell/menu/add", "&Measures"), makeMeasuresItems(), "menu-measures"),
         makeMenu(TranslatableString("appshell/menu/add", "&Frames"), makeFramesItems(), "menu-frames"),
-        makeMenu(TranslatableString("appshell/menu/add", "&Text"), makeTextItems(), "menu-notes"),
+        makeMenu(TranslatableString("appshell/menu/add", "&Text"), makeTextItems(), "menu-text"),
         makeMenu(TranslatableString("appshell/menu/add", "&Lines"), makeLinesItems(), "menu-lines"),
+        makeMenu(TranslatableString("appshell/menu/add", "&Chords and fretboard diagrams"),
+                 makeChordAndFretboardDiagramsItems(), "menu-chord-and-frets"),
     };
 
     return makeMenu(TranslatableString("appshell/menu/add", "&Add"), addItems, "menu-add");
@@ -347,6 +401,8 @@ MenuItem* AppMenuModel::makeToolsMenu()
         makeMenuItem("slash-fill"),
         makeMenuItem("slash-rhythm"),
         makeSeparator(),
+        makeMenuItem("enh-both"),
+        makeMenuItem("enh-current"),
         makeMenuItem("pitch-spell"),
         makeMenuItem("reset-groupings"),
         makeMenuItem("resequence-rehearsal-marks"),
@@ -405,8 +461,15 @@ MenuItem* AppMenuModel::makeHelpMenu(bool addDiagnosticsSubMenu)
     helpItems << makeMenuItem("about-musescore", MenuItemRole::AboutRole);
     helpItems << makeMenuItem("about-qt", MenuItemRole::AboutQtRole);
     helpItems << makeMenuItem("about-musicxml");
-
     helpItems << makeSeparator();
+
+#if defined(Q_OS_WIN) || defined(Q_OS_MACOS)
+    if (isMuseSamplerModuleAdded()) {
+        helpItems << makeMenuItem("clear-online-sounds-cache");
+        helpItems << makeSeparator();
+    }
+#endif
+
     helpItems << makeMenuItem("revert-factory");
 
     return makeMenu(TranslatableString("appshell/menu/help", "&Help"), helpItems, "menu-help");
@@ -426,9 +489,7 @@ MenuItem* AppMenuModel::makeDiagnosticsMenu()
         makeMenu(TranslatableString("appshell/menu/diagnostics", "&System"), systemItems, "menu-system")
     };
 
-#ifdef MUSE_MODULE_MUSESAMPLER
-    bool isMuseSamplerModuleAdded = museSamplerInfo() != nullptr;
-    if (isMuseSamplerModuleAdded) {
+    if (isMuseSamplerModuleAdded()) {
         MenuItemList museSamplerItems {
             makeMenuItem("musesampler-check"),
         };
@@ -439,7 +500,6 @@ MenuItem* AppMenuModel::makeDiagnosticsMenu()
 
         items << makeMenu(TranslatableString("appshell/menu/diagnostics", "&MuseSampler"), museSamplerItems, "menu-musesampler");
     }
-#endif
 
     if (globalConfiguration()->devModeEnabled()) {
         MenuItemList actionsItems {
@@ -465,6 +525,8 @@ MenuItem* AppMenuModel::makeDiagnosticsMenu()
             makeMenuItem("show-skylines"),
             makeMenuItem("show-system-bounding-rects"),
             makeMenuItem("show-element-masks"),
+            makeMenuItem("show-line-attach-points"),
+            makeMenuItem("mark-empty-staff-visibility-overrides"),
             makeMenuItem("mark-corrupted-measures"),
             makeMenuItem("check-for-score-corruptions")
         };
@@ -575,6 +637,7 @@ MenuItemList AppMenuModel::makeIntervalsItems()
         makeMenuItem("interval7"),
         makeMenuItem("interval8"),
         makeMenuItem("interval9"),
+        makeMenuItem("interval10"),
         makeSeparator(),
         makeMenuItem("interval-2"),
         makeMenuItem("interval-3"),
@@ -583,7 +646,8 @@ MenuItemList AppMenuModel::makeIntervalsItems()
         makeMenuItem("interval-6"),
         makeMenuItem("interval-7"),
         makeMenuItem("interval-8"),
-        makeMenuItem("interval-9")
+        makeMenuItem("interval-9"),
+        makeMenuItem("interval-10")
     };
 
     return items;
@@ -625,10 +689,21 @@ MenuItemList AppMenuModel::makeFramesItems()
         makeMenuItem("insert-hbox"),
         makeMenuItem("insert-vbox"),
         makeMenuItem("insert-textframe"),
+        makeMenuItem("insert-fretframe"),
         makeSeparator(),
+        makeMenu(TranslatableString("notation", "Insert at end of score"), makeFramesAppendItems())
+    };
+
+    return items;
+}
+
+MenuItemList AppMenuModel::makeFramesAppendItems()
+{
+    MenuItemList items {
         makeMenuItem("append-hbox"),
         makeMenuItem("append-vbox"),
-        makeMenuItem("append-textframe")
+        makeMenuItem("append-textframe"),
+        makeMenuItem("append-fretframe")
     };
 
     return items;
@@ -688,44 +763,6 @@ MenuItemList AppMenuModel::makeToolbarsItems()
     return items;
 }
 
-MenuItemList AppMenuModel::makeWorkspacesItems()
-{
-    MenuItemList items;
-
-    IWorkspacePtrList workspaces = workspacesManager()->workspaces();
-    IWorkspacePtr currentWorkspace = workspacesManager()->currentWorkspace();
-
-    std::sort(workspaces.begin(), workspaces.end(), [](const IWorkspacePtr& workspace1, const IWorkspacePtr& workspace2) {
-        return workspace1->name() < workspace2->name();
-    });
-
-    int index = 0;
-    for (const IWorkspacePtr& workspace : workspaces) {
-        MenuItem* item = new MenuItem(uiActionsRegister()->action("select-workspace"), this);
-        item->setId(makeId(item->action().code, index++));
-
-        UiAction action = item->action();
-        action.title = TranslatableString::untranslatable(String::fromStdString(workspace->title()));
-
-        item->setAction(action);
-        item->setArgs(ActionData::make_arg1<std::string>(workspace->name()));
-        item->setSelectable(true);
-        item->setSelected(workspace == currentWorkspace);
-
-        UiActionState state;
-        state.enabled = true;
-        state.checked = item->selected();
-        item->setState(state);
-
-        items << item;
-    }
-
-    items << makeSeparator()
-          << makeMenuItem("configure-workspaces");
-
-    return items;
-}
-
 MenuItemList AppMenuModel::makeShowItems()
 {
     MenuItemList items {
