@@ -35,11 +35,13 @@
 #include "engraving/infrastructure/mscwriter.h"
 #include "engraving/dom/excerpt.h"
 #include "engraving/rw/mscsaver.h"
+#include "engraving/types/typesconv.h"
 
 #include "internal/converterutils.h"
 
 #include "backendjsonwriter.h"
 #include "notationmeta.h"
+#include "scoreelementsscanner.h"
 
 #include "log.h"
 
@@ -67,6 +69,36 @@ static const std::string DEV_INFO_NAME = "devinfo";
 static constexpr bool ADD_SEPARATOR = true;
 static constexpr auto NO_STYLE = "";
 
+static ScoreElementScanner::Options parseScoreElementScannerOptions(const std::string& json)
+{
+    QJsonParseError parseError;
+    const QJsonDocument doc = QJsonDocument::fromJson(QByteArray::fromStdString(json), &parseError);
+    if (parseError.error != QJsonParseError::NoError) {
+        LOGE() << "JSON parse error:" << parseError.errorString();
+        return {};
+    }
+
+    const QJsonObject obj = doc.object();
+
+    ScoreElementScanner::Options options;
+    options.avoidDuplicates = obj.value("avoidDuplicates").toBool();
+
+    const QJsonArray typeArray = obj.value("types").toArray();
+    for (const auto typeObj : typeArray) {
+        if (!typeObj.isString()) {
+            continue;
+        }
+
+        const std::string typeStr = typeObj.toString().toStdString();
+        const ElementType type = TConv::fromXml(typeStr, ElementType::INVALID);
+        if (type != ElementType::INVALID) {
+            options.acceptedTypes.insert(type);
+        }
+    }
+
+    return options;
+}
+
 Ret BackendApi::exportScoreMedia(const muse::io::path_t& in, const muse::io::path_t& out, const muse::io::path_t& highlightConfigPath,
                                  const muse::io::path_t& stylePath,
                                  bool forceMode)
@@ -192,6 +224,26 @@ Ret BackendApi::exportScoreTranspose(const muse::io::path_t& in, const muse::io:
     return result ? make_ret(Ret::Code::Ok) : make_ret(Ret::Code::InternalError);
 }
 
+Ret BackendApi::exportScoreElements(const muse::io::path_t& in, const muse::io::path_t& out, const std::string& optionsJson,
+                                    const muse::io::path_t& stylePath, bool forceMode)
+{
+    TRACEFUNC;
+
+    RetVal<INotationProjectPtr> prj = openProject(in, stylePath, forceMode);
+    if (!prj.ret) {
+        return prj.ret;
+    }
+
+    INotationPtr notation = prj.val->masterNotation()->notation();
+
+    QFile outputFile;
+    openOutputFile(outputFile, out);
+
+    BackendJsonWriter jsonWriter(&outputFile);
+
+    return doExportScoreElements(notation, jsonWriter, optionsJson);
+}
+
 Ret BackendApi::openOutputFile(QFile& file, const muse::io::path_t& out)
 {
     bool ok = false;
@@ -238,16 +290,6 @@ RetVal<project::INotationProjectPtr> BackendApi::openProject(const muse::io::pat
     return RetVal<INotationProjectPtr>::make_ok(notationProject);
 }
 
-PageList BackendApi::pages(const INotationPtr notation)
-{
-    auto elements = notation->elements();
-    if (!elements) {
-        return {};
-    }
-
-    return elements->pages();
-}
-
 QVariantMap BackendApi::readBeatsColors(const muse::io::path_t& filePath)
 {
     TRACEFUNC
@@ -262,18 +304,17 @@ QVariantMap BackendApi::readBeatsColors(const muse::io::path_t& filePath)
         return QVariantMap();
     }
 
-    QJsonDocument document = QJsonDocument::fromJson(fileData.val.toQByteArrayNoCopy());
-    QJsonObject obj = document.object();
-    QJsonArray colors = obj.value("highlight").toArray();
+    const QJsonDocument document = QJsonDocument::fromJson(fileData.val.toQByteArrayNoCopy());
+    const QJsonObject obj = document.object();
+    const QJsonArray colors = obj.value("highlight").toArray();
 
     QVariantMap result;
 
-    for (const QJsonValue colorObj: colors) {
-        QJsonObject cobj = colorObj.toObject();
-        QJsonArray beatsIndexes = cobj.value("beats").toArray();
-        QColor beatsColor = QColor(cobj.value("color").toString());
+    for (const auto colorObj : colors) {
+        const QJsonArray beatsIndexes = colorObj[u"beats"].toArray();
+        const QColor beatsColor = QColor(colorObj[u"color"].toString());
 
-        for (const QJsonValue index: beatsIndexes) {
+        for (const auto index : beatsIndexes) {
             result[index.toString()] = beatsColor;
         }
     }
@@ -294,7 +335,7 @@ Ret BackendApi::exportScorePngs(const INotationPtr notation, BackendJsonWriter&
     jsonWriter.addKey("pngs");
     jsonWriter.openArray();
 
-    PageList notationPages = pages(notation);
+    const PageList& notationPages = notation->elements()->pages();
 
     bool result = true;
     for (size_t i = 0; i < notationPages.size(); ++i) {
@@ -336,7 +377,7 @@ Ret BackendApi::exportScoreSvgs(const INotationPtr notation, const muse::io::pat
     jsonWriter.addKey("svgs");
     jsonWriter.openArray();
 
-    PageList notationPages = pages(notation);
+    const PageList& notationPages = notation->elements()->pages();
     QVariantMap beatsColors = readBeatsColors(highlightConfigPath);
 
     bool result = true;
@@ -537,7 +578,7 @@ Ret BackendApi::doExportScoreParts(const IMasterNotationPtr masterNotation, QIOD
 
     ExcerptNotationList excerpts = allExcerpts(masterNotation);
 
-    for (IExcerptNotationPtr excerpt : excerpts) {
+    for (const IExcerptNotationPtr& excerpt : excerpts) {
         mu::engraving::Score* partScore = excerpt->notation()->elements()->msScore();
         std::map<String, String> partMetaTags = partScore->metaTags();
 
@@ -581,7 +622,7 @@ Ret BackendApi::doExportScorePartsPdfs(const IMasterNotationPtr masterNotation,
     QJsonObject jsonForPdfs;
     jsonForPdfs["score"] = QString::fromStdString(scoreFileName);
     QByteArray scoreBin = processWriter(PDF_WRITER_NAME, masterNotation->notation()).val;
-    jsonForPdfs["scoreBin"] = QString::fromLatin1(scoreBin);
+    jsonForPdfs["scoreBin"] = QLatin1String(scoreBin);
 
     INotationPtrList notations;
     notations.push_back(masterNotation->notation());
@@ -591,13 +632,12 @@ Ret BackendApi::doExportScorePartsPdfs(const IMasterNotationPtr masterNotation,
 
     ExcerptNotationList excerpts = allExcerpts(masterNotation);
 
-    for (IExcerptNotationPtr e : excerpts) {
+    for (const IExcerptNotationPtr& e : excerpts) {
         QJsonValue partNameVal(e->name());
         partsNamesArray.append(partNameVal);
 
         QByteArray partBin = processWriter(PDF_WRITER_NAME, e->notation()).val;
-        QJsonValue partVal(QString::fromLatin1(partBin));
-        partsArray.append(partVal);
+        partsArray.append(QLatin1String(partBin));
 
         notations.push_back(e->notation());
     }
@@ -612,7 +652,7 @@ Ret BackendApi::doExportScorePartsPdfs(const IMasterNotationPtr masterNotation,
     };
 
     QByteArray fullScoreData = processWriter(PDF_WRITER_NAME, notations, options).val;
-    jsonForPdfs["scoreFullBin"] = QString::fromLatin1(fullScoreData.toBase64());
+    jsonForPdfs["scoreFullBin"] = QLatin1String(fullScoreData);
 
     QJsonDocument jsonDoc(jsonForPdfs);
     bool ok = destinationDevice.write(QJsonDocument(jsonDoc).toJson(QJsonDocument::Compact)) != -1;
@@ -640,6 +680,76 @@ Ret BackendApi::doExportScoreTranspose(const INotationPtr notation, BackendJsonW
     return ret;
 }
 
+muse::Ret BackendApi::doExportScoreElements(const notation::INotationPtr notation, BackendJsonWriter& jsonWriter,
+                                            const std::string& optionsJson, bool addSeparator)
+{
+    mu::engraving::Score* score = notation->elements()->msScore();
+    ScoreElementScanner::Options options = parseScoreElementScannerOptions(optionsJson);
+    ScoreElementScanner::InstrumentElementMap elements = ScoreElementScanner::scanElements(score, options);
+
+    QJsonArray rootArray;
+
+    for (const auto& instrumentPair : elements) {
+        QJsonArray typeArray;
+
+        for (const auto& pair: instrumentPair.second) {
+            QJsonArray elementArray;
+
+            for (const ScoreElementScanner::ElementInfo& element : pair.second) {
+                QJsonObject obj;
+
+                if (!element.name.empty()) {
+                    obj["name"] = element.name.toQString();
+                }
+
+                if (!element.notes.empty()) {
+                    obj["notes"] = element.notes.toQString();
+                }
+
+                if (!element.text.empty()) {
+                    obj["text"] = element.text.toQString();
+                }
+
+                if (element.staffIdx != muse::nidx) {
+                    obj["staffIdx"] = static_cast<int>(element.staffIdx);
+                }
+
+                if (element.voiceIdx != muse::nidx) {
+                    obj["voiceIdx"] = static_cast<int>(element.voiceIdx);
+                }
+
+                if (element.measureIdx != muse::nidx) {
+                    obj["measureIdx"] = static_cast<int>(element.measureIdx);
+                }
+
+                if (RealIsEqualOrMore(element.beat, 0.f)) {
+                    obj["beat"] = element.beat;
+                }
+
+                if (!obj.empty()) {
+                    elementArray << obj;
+                }
+            }
+
+            QString type = mu::engraving::TConv::toXml(pair.first).ascii();
+            QJsonObject typeObj;
+            typeObj[type] = elementArray;
+            typeArray << typeObj;
+        }
+
+        QJsonObject instrumentObj;
+        instrumentObj["instrumentId"] = instrumentPair.first.instrumentId.toQString();
+        instrumentObj["partId"] = instrumentPair.first.partId.toQString();
+        instrumentObj["types"] = typeArray;
+        rootArray << instrumentObj;
+    }
+
+    jsonWriter.addKey("scoreElements");
+    jsonWriter.addValue(QJsonDocument(rootArray).toJson(QJsonDocument::Compact), addSeparator);
+
+    return make_ok();
+}
+
 RetVal<QByteArray> BackendApi::scorePartJson(mu::engraving::Score* score, const std::string& fileName)
 {
     ByteArray scoreData;
@@ -677,7 +787,7 @@ void BackendApi::switchToPageView(IMasterNotationPtr masterNotation)
 {
     //! NOTE: All operations must be done in page view mode
     masterNotation->notation()->setViewMode(ViewMode::PAGE);
-    for (IExcerptNotationPtr excerpt : masterNotation->excerpts()) {
+    for (const IExcerptNotationPtr& excerpt : masterNotation->excerpts()) {
         excerpt->notation()->setViewMode(ViewMode::PAGE);
     }
 }
@@ -686,7 +796,7 @@ void BackendApi::renderExcerptsContents(IMasterNotationPtr masterNotation)
 {
     //! NOTE: Due to optimization, only the master score is layouted
     //!       Let's layout all the scores of the excerpts
-    for (IExcerptNotationPtr excerpt : masterNotation->excerpts()) {
+    for (const IExcerptNotationPtr& excerpt : masterNotation->excerpts()) {
         Score* score = excerpt->notation()->elements()->msScore();
         if (!score->autoLayoutEnabled()) {
             score->doLayout();
