@@ -20,17 +20,21 @@
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+#include "apistructs.h"
 #include "cursor.h"
 #include "elements.h"
 #include "score.h"
 #include "engraving/dom/masterscore.h"
+#include "engraving/dom/anchors.h"
 #include "engraving/dom/chordrest.h"
 #include "engraving/dom/chord.h"
 #include "engraving/dom/rest.h"
 #include "engraving/dom/note.h"
+#include "engraving/dom/repeatlist.h"
 #include "engraving/dom/stafftext.h"
 #include "engraving/dom/measure.h"
 #include "engraving/dom/page.h"
+#include "engraving/dom/score.h"
 #include "engraving/dom/system.h"
 #include "engraving/dom/segment.h"
 #include "engraving/dom/timesig.h"
@@ -39,7 +43,7 @@
 using namespace mu::engraving::apiv1;
 
 Cursor::Cursor(mu::engraving::Score* s)
-    : QObject(0), _filter(mu::engraving::SegmentType::ChordRest)
+    : QObject(0), m_filter(mu::engraving::SegmentType::ChordRest)
 {
     setScore(s);
 }
@@ -50,7 +54,7 @@ Cursor::Cursor(mu::engraving::Score* s)
 
 Score* Cursor::score() const
 {
-    return wrap<Score>(_score, Ownership::SCORE);
+    return wrap<Score>(m_score, Ownership::SCORE);
 }
 
 //---------------------------------------------------------
@@ -59,13 +63,13 @@ Score* Cursor::score() const
 
 void Cursor::setScore(mu::engraving::Score* s)
 {
-    if (_score == s) {
+    if (m_score == s) {
         return;
     }
 
-    _score = s;
+    m_score = s;
 
-    switch (_inputStateMode) {
+    switch (m_inputStateMode) {
     case INPUT_STATE_INDEPENDENT:
         is.reset(new InputState);
         break;
@@ -81,7 +85,7 @@ void Cursor::setScore(Score* s)
 
 mu::engraving::InputState& Cursor::inputState()
 {
-    return is ? *is.get() : _score->inputState();
+    return is ? *is.get() : m_score->inputState();
 }
 
 //---------------------------------------------------------
@@ -96,7 +100,7 @@ void Cursor::setInputStateMode(InputStateMode val)
         is.reset(new InputState);
     }
 
-    _inputStateMode = val;
+    m_inputStateMode = val;
 }
 
 //---------------------------------------------------------
@@ -117,9 +121,9 @@ void Cursor::rewind(RewindMode mode)
     // rewind to start of score
     //
     if (mode == SCORE_START) {
-        mu::engraving::Measure* m = _score->firstMeasure();
+        mu::engraving::Measure* m = m_score->firstMeasure();
         if (m) {
-            setSegment(m->first(_filter));
+            setSegment(m->first(m_filter));
             nextInTrack();
         } else {
             setSegment(nullptr);
@@ -129,28 +133,29 @@ void Cursor::rewind(RewindMode mode)
     // rewind to start of selection
     //
     else if (mode == SELECTION_START) {
-        if (!_score->selection().isRange()) {
+        if (!m_score->selection().isRange()) {
             return;
         }
-        setSegment(_score->selection().startSegment());
-        setTrack(static_cast<int>(_score->selection().staffStart() * VOICES));
+        setSegment(m_score->selection().startSegment());
+        setTrack(static_cast<int>(m_score->selection().staffStart() * VOICES));
         nextInTrack();
     }
     //
     // rewind to end of selection
     //
     else if (mode == SELECTION_END) {
-        if (!_score->selection().isRange()) {
+        if (!m_score->selection().isRange()) {
             return;
         }
-        setSegment(_score->selection().endSegment());
-        setTrack(static_cast<int>((_score->selection().staffEnd() * VOICES) - 1));      // be sure _track exists
+        setSegment(m_score->selection().endSegment());
+        setTrack(static_cast<int>((m_score->selection().staffEnd() * VOICES) - 1));      // be sure m_track exists
     }
 }
 
 //---------------------------------------------------------
 //   rewindToTick
 ///   Rewind cursor to a position defined by tick.
+///   To avoid rounding errors, use \ref rewindToFraction.
 ///   \param tick Determines the position where to move
 ///   this cursor.
 ///   \see \ref mu::plugins::api::Segment::tick "Segment.tick"
@@ -162,16 +167,44 @@ void Cursor::rewindToTick(int tick)
     // integer ticks may contain numeric errors so it is
     // better to search not precisely if possible
     mu::engraving::Fraction fTick = mu::engraving::Fraction::fromTicks(tick + 1);
-    mu::engraving::Segment* seg = _score->tick2leftSegment(fTick);
-    if (!(seg->segmentType() & _filter)) {
+    mu::engraving::Segment* seg = m_score->tick2leftSegment(fTick);
+    if (!(seg->segmentType() & m_filter)) {
         // we need another segment type, search by known tick
-        seg = _score->tick2segment(seg->tick(), /* first */ true, _filter);
+        seg = m_score->tick2segment(seg->tick(), /* first */ true, m_filter);
     }
 
     setSegment(seg);
     nextInTrack();
 }
 
+void Cursor::rewindToFraction(FractionWrapper* f)
+{
+    const mu::engraving::Fraction fraction = f->fraction();
+    if (!fraction.isValid() || fraction.negative()) {
+        return;
+    }
+
+    // Since we are now dealing with precise tick values, we don't need to call tick2leftSegment.
+    // We just search for a valid segment at the given tick, and if none exist, do nothing.
+    mu::engraving::Segment* seg = m_score->tick2segment(fraction, /* first */ true, m_filter);
+
+    // Exception for TimeTick segments, since here the user may want to create a new anchor.
+    if (!seg && m_filter & mu::engraving::SegmentType::TimeTick) {
+        mu::engraving::Measure* measure = m_score->tick2measure(fraction);
+        if (measure) {
+            mu::engraving::TimeTickAnchor* anchor = mu::engraving::EditTimeTickAnchors::createTimeTickAnchor(measure,
+                                                                                                             fraction - measure->tick(),
+                                                                                                             track2staff(track()));
+            mu::engraving::EditTimeTickAnchors::updateLayout(measure);
+            seg = anchor->segment();
+        } else {
+            LOGW("Cursor::rewindToFraction: Cannot create TimeTickAnchor at provided tick.");
+        }
+    }
+
+    setSegment(seg);
+}
+
 //---------------------------------------------------------
 //   prev
 ///   Move the cursor to the previous segment.
@@ -201,7 +234,7 @@ bool Cursor::next()
     if (!segment()) {
         return false;
     }
-    setSegment(segment()->next1(_filter));
+    setSegment(segment()->next1(m_filter));
     nextInTrack();
     return segment();
 }
@@ -220,11 +253,11 @@ bool Cursor::nextMeasure()
         return false;
     }
     mu::engraving::Measure* m = segment()->measure()->nextMeasure();
-    if (m == 0) {
+    if (!m) {
         setSegment(nullptr);
         return false;
     }
-    setSegment(m->first(_filter));
+    setSegment(m->first(m_filter));
     nextInTrack();
     return segment();
 }
@@ -237,6 +270,7 @@ bool Cursor::nextMeasure()
 
 void Cursor::add(EngravingItem* wrapped)
 {
+    using mu::engraving::SegmentType;
     mu::engraving::EngravingItem* s = wrapped ? wrapped->element() : nullptr;
     if (!segment() || !s) {
         return;
@@ -248,126 +282,151 @@ void Cursor::add(EngravingItem* wrapped)
         return;            // Don't allow operation.
     }
 
-    const int _track = track();
+    const int m_track = track();
     mu::engraving::Segment* _segment = segment();
 
     wrapped->setOwnership(Ownership::SCORE);
-    s->setScore(_score);
-    s->setTrack(_track);
+    s->setScore(m_score);
+    s->setTrack(m_track);
     s->setParent(_segment);
 
     if (s->isChordRest()) {
         s->score()->undoAddCR(toChordRest(s), _segment->measure(), _segment->tick());
-    } else if (s->type() == ElementType::KEYSIG) {
+        return;
+    }
+    switch (s->type()) {
+    case ElementType::KEYSIG: {
         mu::engraving::Segment* ns = _segment->measure()->undoGetSegment(SegmentType::KeySig, _segment->tick());
         s->setParent(ns);
-        _score->undoAddElement(s);
-    } else if (s->type() == ElementType::TIMESIG) {
+        m_score->undoAddElement(s);
+        break;
+    }
+    case ElementType::TIMESIG: {
         mu::engraving::Measure* m = _segment->measure();
         Fraction tick = m->tick();
-        _score->cmdAddTimeSig(m, _track, toTimeSig(s), false);
-        m = _score->tick2measure(tick);
-        _segment = m->first(_filter);
+        m_score->cmdAddTimeSig(m, m_track, toTimeSig(s), false);
+        m = m_score->tick2measure(tick);
+        _segment = m->first(m_filter);
         nextInTrack();
-    } else {
-        switch (s->type()) {
-        // To be added at measure level
-        case ElementType::MEASURE_NUMBER:
-        case ElementType::SPACER:
-        case ElementType::JUMP:
-        case ElementType::MARKER:
-        case ElementType::HBOX:
-        case ElementType::STAFFTYPE_CHANGE:
-        case ElementType::LAYOUT_BREAK: {
-            mu::engraving::Measure* m = _segment->measure();
-            s->setParent(m);
-            _score->undoAddElement(s);
-            break;
-        }
+        break;
+    }
 
-        // To be added at chord level
-        case ElementType::NOTE:
-        case ElementType::ARPEGGIO:
-        case ElementType::TREMOLO_SINGLECHORD:
-        case ElementType::TREMOLO_TWOCHORD:
-        case ElementType::CHORDLINE:
-        case ElementType::ARTICULATION: {
-            mu::engraving::EngravingItem* curElement = currentElement();
-            if (curElement->isChord()) {
-                // call Chord::addInternal() (i.e. do the same as a call to Chord.add())
-                Chord::addInternal(toChord(curElement), s);
-            }
-            break;
-            break;
-        }
+    // To be added at measurebase level
+    case ElementType::LAYOUT_BREAK: {
+        MeasureBase::addInternal(_segment->measure(), s);
+        break;
+    }
+    // To be added at measure level
+    // todo: insert boxes before/after other boxes correctly
+    case ElementType::MEASURE_NUMBER:
+    case ElementType::SPACER:
+    case ElementType::JUMP:
+    case ElementType::MARKER:
+    case ElementType::HBOX:
+    case ElementType::STAFFTYPE_CHANGE: {
+        mu::engraving::Measure* m = _segment->measure();
+        s->setParent(m);
+        m_score->undoAddElement(s);
+        break;
+    }
 
-        // To be added at chord/rest level
-        case ElementType::LYRICS: {
-            mu::engraving::EngravingItem* curElement = currentElement();
-            if (curElement->isChordRest()) {
-                s->setParent(curElement);
-                _score->undoAddElement(s);
-            }
-            break;
+    // To be added at chord level
+    case ElementType::NOTE:
+    case ElementType::ARPEGGIO:
+    case ElementType::TREMOLO_SINGLECHORD:
+    case ElementType::TREMOLO_TWOCHORD:
+    case ElementType::CHORDLINE:
+    case ElementType::ORNAMENT:
+    case ElementType::TAPPING:
+    case ElementType::ARTICULATION: {
+        mu::engraving::EngravingItem* curElement = currentElement();
+        if (curElement->isChord()) {
+            // call Chord::addInternal() (i.e. do the same as a call to Chord.add())
+            Chord::addInternal(toChord(curElement), s);
         }
+        break;
+    }
 
-        // To be added to a note (and in case of SYMBOL also to a rest)
-        case ElementType::SYMBOL: {
-            mu::engraving::EngravingItem* curElement = currentElement();
-            if (curElement->isRest()) {
-                s->setParent(curElement);
-                _score->undoAddElement(s);
-            }
-        } // FALLTHROUGH
-        case ElementType::FINGERING:
-        case ElementType::BEND:
-        case ElementType::NOTEHEAD: {
-            mu::engraving::EngravingItem* curElement = currentElement();
-            if (curElement->isChord()) {
-                mu::engraving::Chord* chord = toChord(curElement);
-                mu::engraving::Note* note = nullptr;
-                if (chord->notes().size() > 0) {
-                    // Get first note from chord to add element
-                    note = chord->notes().front();
-                }
-                if (note) {
-                    Note::addInternal(note, s);
-                }
-            }
-            break;
+    // To be added at chord/rest level
+    case ElementType::LYRICS: {
+        mu::engraving::EngravingItem* curElement = currentElement();
+        if (curElement->isChordRest()) {
+            s->setParent(curElement);
+            m_score->undoAddElement(s);
         }
+        break;
+    }
 
-        // To be added to a segment (clef subtype)
-        case ElementType::CLEF:
-        case ElementType::AMBITUS: {
-            mu::engraving::EngravingItem* parent = nullptr;
-            // Find backwards first measure containing a clef
-            for (mu::engraving::Measure* m = _segment->measure(); m != 0; m = m->prevMeasure()) {
-                mu::engraving::Segment* seg = m->findSegment(SegmentType::Clef | SegmentType::HeaderClef, m->tick());
-                if (seg != 0) {
-                    parent = m->undoGetSegmentR(s->isAmbitus() ? SegmentType::Ambitus : seg->segmentType(), Fraction(0,
-                                                                                                                     1));
-                    break;
-                }
-            }
-            if (parent && parent->isSegment()) {
-                if (s->isClef()) {
-                    mu::engraving::Clef* clef = toClef(s);
-                    if (clef->clefType() == mu::engraving::ClefType::INVALID) {
-                        clef->setClefType(mu::engraving::ClefType::G);
-                    }
-                }
-                s->setParent(parent);
-                s->setTrack(_track);
-                _score->undoAddElement(s);
-            }
-            break;
+    // To be added to a note (and in case of SYMBOL also to a rest)
+    case ElementType::SYMBOL: {
+        mu::engraving::EngravingItem* curElement = currentElement();
+        if (curElement->isRest()) {
+            s->setParent(curElement);
+            m_score->undoAddElement(s);
         }
+        [[fallthrough]];
+    }
+    case ElementType::FINGERING:
+    case ElementType::BEND:
+    case ElementType::NOTEHEAD: {
+        mu::engraving::EngravingItem* curElement = currentElement();
+        if (curElement->isChord()) {
+            mu::engraving::Chord* chord = toChord(curElement);
+            if (!chord->notes().empty()) {
+                // Get first note from chord to add element
+                mu::engraving::Note* note = chord->notes().front();
+                Note::addInternal(note, s);
+            }
+        }
+        break;
+    }
 
-        default:           // All others will be added to the current segment
-            _score->undoAddElement(s);
+    // To be added to a segment (clef subtype)
+    case ElementType::CLEF: {
+        mu::engraving::Clef* clef = toClef(s);
+        if (clef->clefType() == mu::engraving::ClefType::INVALID) {
+            clef->setClefType(mu::engraving::ClefType::G);
+        }
+        SegmentType st = SegmentType::Clef;
+        mu::engraving::Measure* measure = _segment->measure();
+        mu::engraving::Fraction rt = _segment->rtick();
+        if (rt == Fraction(0, 1)) {
+            mu::engraving::Measure* prevMeasure = measure->prevMeasure();
+            if (prevMeasure && !prevMeasure->sectionBreak()) {
+                measure = measure->prevMeasure();
+                rt      = measure->ticks();
+            } else {
+                st = SegmentType::HeaderClef;
+            }
+        }
+        mu::engraving::Segment* destSeg = measure->undoGetSegmentR(st, rt);
+        clef->setParent(destSeg);
+        clef->setTrack(m_track);
+        clef->setIsHeader(st == SegmentType::HeaderClef);
+        m_score->undoAddElement(clef);
+        break;
+    }
+    // To be added to a segment (clef subtype)
+    case ElementType::AMBITUS: {
+        // Find backwards first measure containing a clef
+        mu::engraving::Segment* parent = nullptr;
+        for (mu::engraving::Measure* m = _segment->measure(); m; m = m->prevMeasure()) {
+            mu::engraving::Segment* seg = m->findSegment(SegmentType::Clef | SegmentType::HeaderClef, m->tick());
+            if (!seg) {
+                continue;
+            }
+            parent = m->undoGetSegmentR(SegmentType::Ambitus, Fraction(0, 1));
+            s->setParent(parent);
+            s->setTrack(m_track);
+            m_score->undoAddElement(s);
             break;
         }
+        break;
+    }
+
+    default:           // All others will be added to the current segment
+        m_score->undoAddElement(s);
+        break;
     }
 }
 
@@ -396,7 +455,7 @@ void Cursor::addNote(int pitch, bool addToChord)
         setDuration(1, 4);
     }
     NoteVal nval(pitch);
-    _score->addPitch(nval, addToChord, is.get());
+    m_score->addPitch(nval, addToChord, is.get());
 }
 
 //---------------------------------------------------------
@@ -416,7 +475,7 @@ void Cursor::addRest()
     if (!inputState().duration().isValid()) {
         setDuration(1, 4);
     }
-    _score->enterRest(inputState().duration(), is.get());
+    m_score->enterRest(inputState().duration(), is.get());
 }
 
 //---------------------------------------------------------
@@ -488,13 +547,13 @@ void Cursor::addTuplet(FractionWrapper* ratio, FractionWrapper* duration)
         return;
     }
 
-    _score->expandVoice(inputState().segment(), inputState().track());
+    m_score->expandVoice(inputState().segment(), inputState().track());
     mu::engraving::ChordRest* cr = inputState().cr();
     if (!cr) { // shouldn't happen?
         return;
     }
 
-    _score->changeCRlen(cr, fDuration);
+    m_score->changeCRlen(cr, fDuration);
 
     mu::engraving::Tuplet* tuplet = new mu::engraving::Tuplet(tupletMeasure);
     tuplet->setParent(tupletMeasure);
@@ -504,7 +563,7 @@ void Cursor::addTuplet(FractionWrapper* ratio, FractionWrapper* duration)
     tuplet->setTicks(fDuration);
     tuplet->setBaseLen(baseLen);
 
-    _score->cmdCreateTuplet(cr, tuplet);
+    m_score->cmdCreateTuplet(cr, tuplet);
 
     inputState().setSegment(tupletMeasure->tick2segment(tupletTick));
     inputState().setDuration(baseLen);
@@ -538,13 +597,31 @@ int Cursor::tick()
     return seg ? seg->tick().ticks() : 0;
 }
 
+int Cursor::utick()
+{
+    return m_score->repeatList(true).tick2utick(tick());
+}
+
+mu::engraving::Fraction Cursor::fraction() const
+{
+    const mu::engraving::Segment* seg = segment();
+    return seg ? seg->tick() : Fraction(0, 1);
+}
+
+FractionWrapper* Cursor::qmlFraction() const
+{
+    return wrap(fraction());
+}
+
 //---------------------------------------------------------
 //   time
+/// Time at tick position, read only
+/// \param includeRepeats since 4.6
 //---------------------------------------------------------
 
-double Cursor::time()
+double Cursor::time(bool includeRepeats)
 {
-    return _score->utick2utime(tick()) * 1000;
+    return m_score->utick2utime(includeRepeats ? utick() : tick()) * 1000;
 }
 
 //---------------------------------------------------------
@@ -553,7 +630,7 @@ double Cursor::time()
 
 qreal Cursor::tempo()
 {
-    return _score->tempo(Fraction::fromTicks(tick())).val;
+    return m_score->tempo(fraction()).val;
 }
 
 //---------------------------------------------------------
@@ -613,31 +690,22 @@ int Cursor::track() const
 //   setTrack
 //---------------------------------------------------------
 
-void Cursor::setTrack(int _track)
+void Cursor::setTrack(int track)
 {
-    int tracks = static_cast<int>(_score->nstaves() * VOICES);
-    if (_track < 0) {
-        _track = 0;
-    } else if (_track >= tracks) {
-        _track = tracks - 1;
+    if (m_score->ntracks() == 0) {
+        return;
     }
-    inputState().setTrack(_track);
+    int m_track = std::clamp(track, 0, int(m_score->ntracks() - 1));
+    inputState().setTrack(static_cast<track_idx_t>(m_track));
 }
 
 //---------------------------------------------------------
-//   setStaffIdx
+//   voice
 //---------------------------------------------------------
 
-void Cursor::setStaffIdx(int v)
+int Cursor::voice() const
 {
-    track_idx_t _track = v * VOICES + track() % VOICES;
-    size_t tracks = _score->nstaves() * VOICES;
-    if (_track == muse::nidx) {
-        _track = 0;
-    } else if (_track >= tracks) {
-        _track = tracks - 1;
-    }
-    inputState().setTrack(_track);
+    return track() % VOICES;
 }
 
 //---------------------------------------------------------
@@ -646,14 +714,58 @@ void Cursor::setStaffIdx(int v)
 
 void Cursor::setVoice(int v)
 {
-    track_idx_t _track = (track() / VOICES) * VOICES + v;
-    size_t tracks = _score->nstaves() * VOICES;
-    if (_track == muse::nidx) {
-        _track = 0;
-    } else if (_track >= tracks) {
-        _track = tracks - 1;
+    int m_track = (track() / VOICES) * VOICES + v;
+    setTrack(m_track);
+}
+
+//---------------------------------------------------------
+//   staffIdx
+//---------------------------------------------------------
+
+int Cursor::staffIdx() const
+{
+    return track() / VOICES;
+}
+
+//---------------------------------------------------------
+//   staff
+//---------------------------------------------------------
+
+Staff* Cursor::staff() const
+{
+    return wrap<Staff>(inputState().staff(), Ownership::SCORE);
+}
+
+//---------------------------------------------------------
+//   setStaffIdx
+//---------------------------------------------------------
+
+void Cursor::setStaffIdx(int v)
+{
+    int m_track = v * VOICES + track() % VOICES;
+    setTrack(m_track);
+}
+
+//---------------------------------------------------------
+//   setStaff
+///   \brief Positions the cursor on a specific \ref Staff.
+///   \details As opposed to \ref cursor.setStaffIdx, this
+///   method is used to set the position of the cursor to
+///   a specific \ref Staff object (as obtained e.g. through
+///   \ref element.staff).
+///   \since MuseScore 4.6
+///   \see \ref Staff
+//---------------------------------------------------------
+
+void Cursor::setStaff(Staff* s)
+{
+    mu::engraving::Staff* staff = s->staff();
+    if (!staff || staff == inputState().staff()) {
+        return;
     }
-    inputState().setTrack(_track);
+
+    int m_track = int(staff->idx()) + track() % VOICES;
+    setTrack(m_track);
 }
 
 //---------------------------------------------------------
@@ -674,27 +786,9 @@ void Cursor::setSegment(mu::engraving::Segment* seg)
     inputState().setSegment(seg);
 }
 
-//---------------------------------------------------------
-//   staffIdx
-//---------------------------------------------------------
-
-int Cursor::staffIdx() const
-{
-    return track() / VOICES;
-}
-
-//---------------------------------------------------------
-//   voice
-//---------------------------------------------------------
-
-int Cursor::voice() const
-{
-    return track() % VOICES;
-}
-
 //---------------------------------------------------------
 //   prevInTrack
-//    go to first segment before _segment which has notes / rests in _track
+//    go to first segment before _segment which has notes / rests in m_track
 //---------------------------------------------------------
 
 void Cursor::prevInTrack()
@@ -702,25 +796,25 @@ void Cursor::prevInTrack()
     const int t = track();
     mu::engraving::Segment* seg = segment();
     if (seg) {
-        seg = seg->prev1(_filter);
+        seg = seg->prev1(m_filter);
     }
     while (seg && !seg->element(t)) {
-        seg = seg->prev1(_filter);
+        seg = seg->prev1(m_filter);
     }
     setSegment(seg);
 }
 
 //---------------------------------------------------------
 //   nextInTrack
-//    go to first segment at or after _segment which has notes / rests in _track
+//    go to first segment at or after _segment which has notes / rests in m_track
 //---------------------------------------------------------
 
 void Cursor::nextInTrack()
 {
     const int t = track();
     mu::engraving::Segment* seg = segment();
-    while (seg && seg->element(t) == 0) {
-        seg = seg->next1(_filter);
+    while (seg && !seg->element(t)) {
+        seg = seg->next1(m_filter);
     }
     setSegment(seg);
 }
@@ -733,8 +827,7 @@ void Cursor::nextInTrack()
 
 int Cursor::qmlKeySignature()
 {
-    mu::engraving::Staff* staff = _score->staves()[staffIdx()];
-    return static_cast<int>(staff->key(Fraction::fromTicks(tick())));
+    return static_cast<int>(inputState().staff()->key(fraction()));
 }
 
 //---------------------------------------------------------
@@ -744,12 +837,12 @@ int Cursor::qmlKeySignature()
 int Cursor::inputStateString() const
 {
     const InputState& istate = inputState();
-    return _score->staff(staffIdx())->staffType(istate.tick())->visualStringToPhys(istate.string());
+    return istate.staff()->staffType(istate.tick())->visualStringToPhys(istate.string());
 }
 
 void Cursor::setInputStateString(int string)
 {
     InputState& istate = inputState();
-    const int visString = _score->staff(staffIdx())->staffType(istate.tick())->visualStringToPhys(string);
+    const int visString = istate.staff()->staffType(istate.tick())->visualStringToPhys(string);
     istate.setString(visString);
 }
