@@ -19,16 +19,12 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
-#ifndef MU_ENGRAVING_MSCXCOMPAT_H
-#define MU_ENGRAVING_MSCXCOMPAT_H
+#pragma once
 
 #include "../dom/masterscore.h"
 #include "../engravingproject.h"
 
 namespace mu::engraving::compat {
-muse::Ret mscxToMscz(const muse::io::path_t& mscxFilePath, muse::ByteArray* msczData);
 muse::Ret loadMsczOrMscx(MasterScore* score, const muse::io::path_t& path, bool ignoreVersionError = false);
 muse::Ret loadMsczOrMscx(EngravingProjectPtr project, const muse::io::path_t& path, bool ignoreVersionError = false);
 }
-
-#endif // MU_ENGRAVING_MSCXCOMPAT_H
