@@ -66,9 +66,6 @@ Articulation::Articulation(ChordRest* parent, ElementType type)
     m_ornamentStyle = OrnamentStyle::DEFAULT;
     m_playArticulation = true;
 
-    m_font.setFamily(u"FreeSans", Font::Type::Tablature);
-    m_font.setPointSizeF(7.0);
-
     initElementStyle(&articulationStyle);
     setupShowOnTabStyles();
 }
@@ -95,6 +92,24 @@ void Articulation::setTextType(ArticulationTextType textType)
     m_textType = textType;
 }
 
+void Articulation::setSelected(bool f)
+{
+    if (m_text) {
+        m_text->setSelected(f);
+    }
+
+    EngravingItem::setSelected(f);
+}
+
+void Articulation::setVisible(bool f)
+{
+    if (m_text) {
+        m_text->setVisible(f);
+    }
+
+    EngravingItem::setVisible(f);
+}
+
 //---------------------------------------------------------
 //   subtype
 //---------------------------------------------------------
@@ -236,7 +251,7 @@ bool Articulation::isHiddenOnTabStaff() const
         return false;
     }
 
-    return stType->isHiddenElementOnTab(style(), m_showOnTabStyles.first, m_showOnTabStyles.second);
+    return stType->isHiddenElementOnTab(m_showOnTabStyles.first, m_showOnTabStyles.second);
 }
 
 //---------------------------------------------------------
@@ -503,6 +518,10 @@ Sid Articulation::getPropertyStyle(Pid id) const
         return EngravingItem::getPropertyStyle(id);
 
     case Pid::ARTICULATION_ANCHOR: {
+        if (isHandbellsArticulation()) {
+            return Sid::articulationAnchorDefault;
+        }
+
         switch (anchorGroup(m_symId)) {
         case AnchorGroup::ARTICULATION:
             return Sid::articulationAnchorDefault;
@@ -586,6 +605,13 @@ void Articulation::computeCategories()
                          || m_symId == SymId::luteFingeringRHThird);
     m_categories.setFlag(ArticulationCategory::LAISSEZ_VIB,
                          m_symId == SymId::articLaissezVibrerAbove || m_symId == SymId::articLaissezVibrerBelow);
+
+    m_categories.setFlag(ArticulationCategory::HANDBELLS,
+                         (static_cast<int>(m_symId) >= static_cast<int>(SymId::handbellsBelltree)
+                          && static_cast<int>(m_symId) <= static_cast<int>(SymId::handbellsTableSingleBell))
+                         || (static_cast<int>(m_textType) >= static_cast<int>(ArticulationTextType::TD)
+                             && static_cast<int>(m_textType) <= static_cast<int>(ArticulationTextType::VIB))
+                         );
 }
 
 bool Articulation::isBasicArticulation() const
@@ -654,6 +680,10 @@ void Articulation::setupShowOnTabStyles()
 
 void Articulation::styleChanged()
 {
+    if (m_text) {
+        m_text->styleChanged();
+    }
+
     bool isGolpeThumb = m_symId == SymId::guitarGolpe && m_anchor == ArticulationAnchor::BOTTOM;
     EngravingItem::styleChanged();
     if (isGolpeThumb) {
@@ -915,4 +945,18 @@ std::set<SymId> flipArticulations(const std::set<SymId>& articulationSymbolIds,
 
     return result;
 }
+
+double Articulation::LayoutData::opticalCenter() const
+{
+    switch (symId.value()) {
+    case SymId::handbellsMartellatoLift:
+        return 0.5 * m_item->symWidth(SymId::handbellsMartellato);
+    case SymId::handbellsMalletLft:
+        return 0.5 * m_item->symWidth(SymId::handbellsMalletBellOnTable);
+    case SymId::handbellsPluckLift:
+        return 0.5 * m_item->symWidth(SymId::articStaccatoAbove);
+    default:
+        return 0.5 * bbox().width();
+    }
+}
 }
