@@ -24,11 +24,15 @@
 
 #include "translation.h"
 
+#include "io/file.h"
+
+#include "anchors.h"
 #include "chord.h"
 #include "factory.h"
 #include "harmony.h"
 #include "measure.h"
 #include "note.h"
+#include "pitchspelling.h"
 #include "score.h"
 #include "segment.h"
 #include "staff.h"
@@ -36,6 +40,9 @@
 #include "system.h"
 #include "undo.h"
 
+#include "rw/harmonytodiagramreader.h"
+#include "rw/read460/tread.h"
+
 #include "log.h"
 
 using namespace mu;
@@ -66,9 +73,100 @@ static const ElementStyle fretStyle {
 //   FretDiagram
 //---------------------------------------------------------
 
+struct HarmonyMapKey
+{
+    HarmonyMapKey() {}
+    HarmonyMapKey(int k, int r, int b)
+        : keys(k), rootTpc(r), bassTpc(b) {}
+
+    int keys = 0;
+    int rootTpc = Tpc::TPC_INVALID;
+    int bassTpc = Tpc::TPC_INVALID;
+
+    bool operator<(const HarmonyMapKey& other) const
+    {
+        return std::tie(keys, rootTpc, bassTpc) < std::tie(other.keys, other.rootTpc, other.bassTpc);
+    }
+
+    bool operator==(const HarmonyMapKey& other) const
+    {
+        return std::tie(keys, rootTpc, bassTpc) == std::tie(other.keys, other.rootTpc, other.bassTpc);
+    }
+};
+
+struct DiagramInfo {
+    String harmonyName;
+    String diagramXml;
+    String diagramPattern;
+};
+
+static std::map<HarmonyMapKey /*key*/, std::vector<DiagramInfo> > s_harmonyToDiagramMap;
+static std::unordered_map<String /*pattern*/, std::vector<String /*harmonyName*/> > s_diagramPatternToHarmoniesMap;
+
+static const muse::io::path_t HARMONY_TO_DIAGRAM_FILE_PATH("://data/harmony_to_diagram.xml");
+
+static HarmonyMapKey createHarmonyMapKey(const String& harmony, const NoteSpellingType& spellingType, const ChordList* cl)
+{
+    String s = harmony;
+    NoteCaseType noteCase;
+    size_t idx;
+    int rootTpc = convertNote(s, spellingType, noteCase, idx);
+
+    int bassTpc = Tpc::TPC_INVALID;
+    size_t slash = s.lastIndexOf(u'/');
+    if (slash != muse::nidx) {
+        String bs = s.mid(slash + 1).simplified();
+        s = s.mid(idx, slash - idx).simplified();
+        size_t idx2 = 0;
+        bassTpc = convertNote(bs, spellingType, noteCase, idx2);
+
+        if (!tpcIsValid(bassTpc)) {
+            // if what follows after slash is not (just) a TPC
+            // then reassemble chord and try to parse with the slash
+            s = s + u"/" + bs;
+        }
+    } else {
+        s = s.mid(idx);
+    }
+
+    ParsedChord chord;
+    if (!chord.parse(s, cl)) {
+        LOGE() << "Error parse " << harmony;
+    }
+
+    int keys = chord.keys();
+
+    return HarmonyMapKey(keys, rootTpc, bassTpc);
+}
+
+static DiagramInfo resolveDiagram(const std::vector<DiagramInfo>& diagrams)
+{
+    auto isSimpler = [](const DiagramInfo& a, const DiagramInfo& b) {
+        const String& harmonyNameA = a.harmonyName;
+        const String& harmonyNameB = b.harmonyName;
+
+        if (harmonyNameA.size() != harmonyNameB.size()) {
+            return harmonyNameA.size() < harmonyNameB.size();
+        }
+
+        return harmonyNameA < harmonyNameB;
+    };
+
+    DiagramInfo result = diagrams.front();
+    for (size_t i = 1; i < diagrams.size(); ++i) {
+        if (!isSimpler(result, diagrams[i])) {
+            result = diagrams[i];
+        }
+    }
+
+    return result;
+}
+
 FretDiagram::FretDiagram(Segment* parent)
     : EngravingItem(ElementType::FRET_DIAGRAM, parent, ElementFlag::MOVABLE | ElementFlag::ON_STAFF)
 {
+    initDefaultValues();
+
     initElementStyle(&fretStyle);
 }
 
@@ -80,7 +178,6 @@ FretDiagram::FretDiagram(const FretDiagram& f)
     m_fretOffset = f.m_fretOffset;
     m_maxFrets   = f.m_maxFrets;
     m_userMag    = f.m_userMag;
-    m_numPos     = f.m_numPos;
     m_dots       = f.m_dots;
     m_markers    = f.m_markers;
     m_barres     = f.m_barres;
@@ -95,6 +192,21 @@ FretDiagram::FretDiagram(const FretDiagram& f)
     }
 }
 
+void FretDiagram::initDefaultValues()
+{
+    m_strings = 6;
+    m_frets = 4;
+    m_fretOffset = 0;
+    m_maxFrets = 24;
+    m_showNut = true;
+    m_orientation = Orientation::VERTICAL;
+
+    m_userMag = 1.0;
+
+    m_showFingering = false;
+    m_fingering = std::vector<int>(m_strings, 0);
+}
+
 FretDiagram::~FretDiagram()
 {
     if (m_harmony) {
@@ -118,57 +230,46 @@ EngravingItem* FretDiagram::linkedClone()
     return e;
 }
 
-//---------------------------------------------------------
-//   fromString
-///   Create diagram from string like "XO-123"
-///   Always assume barre on the first visible fret
-//---------------------------------------------------------
-
-std::shared_ptr<FretDiagram> FretDiagram::createFromString(Score* score, const String& s)
+Segment* FretDiagram::segment() const
 {
-    auto fd = Factory::makeFretDiagram(score->dummy()->segment());
-    int strings = static_cast<int>(s.size());
-
-    fd->setStrings(strings);
-    fd->setFrets(4);
-    fd->setPropertyFlags(Pid::FRET_STRINGS, PropertyFlags::UNSTYLED);
-    fd->setPropertyFlags(Pid::FRET_FRETS,   PropertyFlags::UNSTYLED);
-    int offset = 0;
-    int barreString = -1;
-    std::vector<std::pair<int, int> > dotsToAdd;
-
-    for (int i = 0; i < strings; i++) {
-        Char c = s.at(i);
-        if (c == 'X' || c == 'O') {
-            FretMarkerType mt = (c == 'X' ? FretMarkerType::CROSS : FretMarkerType::CIRCLE);
-            fd->setMarker(i, mt);
-        } else if (c == '-' && barreString == -1) {
-            barreString = i;
-        } else {
-            int fret = c.digitValue();
-            if (fret != -1) {
-                dotsToAdd.push_back(std::make_pair(i, fret));
-                if (fret - 3 > 0 && offset < fret - 3) {
-                    offset = fret - 3;
-                }
-            }
-        }
+    EngravingObject* parent = explicitParent();
+    if (!parent || !parent->isSegment()) {
+        return nullptr;
     }
 
-    if (offset > 0) {
-        fd->setFretOffset(offset);
+    return toSegment(explicitParent());
+}
+
+void FretDiagram::updateDiagram(const String& harmonyName)
+{
+    if (s_harmonyToDiagramMap.empty()) {
+        readHarmonyToDiagramFile(HARMONY_TO_DIAGRAM_FILE_PATH);
     }
 
-    for (const std::pair<int, int>& d : dotsToAdd) {
-        fd->setDot(d.first, d.second - offset, true);
+    String _harmonyName = harmonyName;
+
+    NoteSpellingType spellingType = style().styleV(Sid::chordSymbolSpelling).value<NoteSpellingType>();
+    HarmonyMapKey key = createHarmonyMapKey(_harmonyName, spellingType, score()->chordList());
+
+    std::vector<DiagramInfo> availableDiagrams = muse::value(s_harmonyToDiagramMap, key);
+    if (availableDiagrams.empty()) {
+        return;
     }
 
-    // This assumes that any barre goes to the end of the fret
-    if (barreString >= 0) {
-        fd->setBarre(barreString, -1, 1);
+    String diagramXml = resolveDiagram(availableDiagrams).diagramXml;
+
+    if (diagramXml.empty()) {
+        return;
     }
 
-    return fd;
+    clear();
+
+    read460::ReadContext ctx;
+    XmlReader reader(diagramXml.toUtf8());
+
+    read460::TRead::read(this, reader, ctx);
+
+    triggerLayout();
 }
 
 //---------------------------------------------------------
@@ -331,7 +432,7 @@ void FretDiagram::setDot(int string, int fret, bool add /*= false*/, FretDotType
 void FretDiagram::addDotForDotStyleBarre(int string, int fret)
 {
     if (m_dots[string].empty()) {
-        m_dots[string].push_back(FretItem::Dot(fret, FretDotType::NORMAL));
+        m_dots[string].push_back(FretItem::Dot(fret, FretDotType::NORMAL, /*isPartOfSlurBarre*/ true));
     }
 }
 
@@ -529,6 +630,227 @@ void FretDiagram::removeDotsMarkers(int ss, int es, int fret)
     }
 }
 
+//---------------------------------------------------------
+//   Fill diagram from string like "XO-[1-O][1-X,2-O,3-S][3-T];B1[0-5];B..."
+//   - Each character or bracketed block represents a string, from lowest to highest.
+//   - 'X' = muted string (cross marker)
+//   - 'O' = open string (circle marker)
+//   - '-' = empty or unused string
+//   - [fret-type,...] = one or more fretted dots on that string:
+//       • fret is absolute (already includes offset)
+//       • type is:
+//           - O = circle
+//           - X = cross
+//           - S = square
+//           - T = triangle
+//   - Example: [1-X,2-O,3-S] means three dots on frets 1, 2, and 3 with different types
+//   - Barre chords follow after ';', in the format: B{fret}[{start}-{end}]
+//     e.g. B1[0-5] = barre on fret 1 from string 0 to 5
+//---------------------------------------------------------
+
+void FretDiagram::applyDiagramPattern(FretDiagram* diagram, const String& pattern)
+{
+    diagram->clear();
+
+    const std::vector<String> parts = pattern.split(';');
+    if (parts.empty()) {
+        return;
+    }
+
+    const String& mainPart = parts[0];
+    std::vector<String> stringTokens;
+
+    for (size_t i = 0; i < mainPart.size();) {
+        if (mainPart[i] == u'[') {
+            size_t rb = mainPart.indexOf(u']', i);
+            if (rb != muse::nidx) {
+                stringTokens.push_back(mainPart.mid(i, rb - i + 1));
+                i = rb + 1;
+                continue;
+            }
+        }
+        stringTokens.push_back(mainPart.mid(i, 1));
+        ++i;
+    }
+
+    const int stringCount = static_cast<int>(stringTokens.size());
+    diagram->setStrings(stringCount);
+    diagram->setFrets(4);
+    diagram->setPropertyFlags(Pid::FRET_STRINGS, PropertyFlags::UNSTYLED);
+    diagram->setPropertyFlags(Pid::FRET_FRETS,   PropertyFlags::UNSTYLED);
+
+    int offset = 0;
+
+    for (int i = 0; i < stringCount; ++i) {
+        const String& token = stringTokens[i];
+
+        if (token == u"-") {
+            continue;
+        }
+
+        if (token.startsWith(u"[") && token.endsWith(u"]")) {
+            // Example: [1-O,2-X,3-S]
+            String inner = token.mid(1, token.size() - 2);
+            std::vector<String> pairs = inner.split(u',');
+
+            for (const String& p : pairs) {
+                size_t dash = p.indexOf(u'-');
+                if (dash > 0) {
+                    int fret = p.left(dash).toInt();
+                    Char typeChar = p.mid(dash + 1, 1).at(0);
+                    FretDotType dt;
+
+                    switch (typeChar.unicode()) {
+                    case 'X': dt = FretDotType::CROSS;
+                        break;
+                    case 'O': dt = FretDotType::NORMAL;
+                        break;
+                    case 'S': dt = FretDotType::SQUARE;
+                        break;
+                    case 'T': dt = FretDotType::TRIANGLE;
+                        break;
+                    default:  dt = FretDotType::NORMAL;
+                        break;
+                    }
+
+                    diagram->setDot(i, fret - offset, true, dt);
+                    if (fret - 3 > 0 && offset < fret - 3) {
+                        offset = fret - 3;
+                    }
+                }
+            }
+        } else if (token == u"X") {
+            diagram->setMarker(i, FretMarkerType::CROSS);
+        } else if (token == u"O") {
+            diagram->setMarker(i, FretMarkerType::CIRCLE);
+        }
+    }
+
+    if (offset > 0) {
+        diagram->setFretOffset(offset);
+    }
+
+    for (size_t i = 1; i < parts.size(); ++i) {
+        const String& barrePart = parts[i];
+        if (!barrePart.startsWith(u'B')) {
+            continue;
+        }
+
+        size_t lb = barrePart.indexOf(u'[');
+        size_t rb = barrePart.indexOf(u']');
+        if (lb < 2 || rb < lb) {
+            continue;
+        }
+
+        int fret = barrePart.mid(1, lb - 1).toInt();
+        if (fret <= 0) {
+            continue;
+        }
+
+        String range = barrePart.mid(lb + 1, rb - lb - 1);
+        size_t dash = range.indexOf('-');
+        if (dash > 0) {
+            int start = range.left(dash).toInt();
+            int end = range.mid(dash + 1).toInt();
+            if (start >= 0 && end >= start && end < stringCount) {
+                diagram->setBarre(start, end, fret - offset);
+            }
+        }
+    }
+}
+
+String FretDiagram::patternFromDiagram(const FretDiagram* diagram)
+{
+    const int strings = diagram->strings();
+    const int offset = diagram->fretOffset();
+
+    StringList patternParts;
+    const DotMap& dotsMap = diagram->dots();
+
+    for (int i = 0; i < strings; ++i) {
+        const FretItem::Marker marker = diagram->marker(i);
+
+        if (marker.mtype == FretMarkerType::CROSS) {
+            patternParts.push_back(u"X");
+            continue;
+        } else if (marker.mtype == FretMarkerType::CIRCLE) {
+            patternParts.push_back(u"O");
+            continue;
+        }
+
+        const auto it = dotsMap.find(i);
+        std::vector<FretItem::Dot> dotList;
+        if (it != dotsMap.end()) {
+            const std::vector<FretItem::Dot>& dots = it->second;
+            for (const FretItem::Dot& dot : dots) {
+                if (!dot.isPartOfSlurBarre) { // Don't write dot if part of slur barré
+                    dotList.push_back(dot);
+                }
+            }
+        }
+
+        if (!dotList.empty()) {
+            const auto& dotList2 = it->second;
+            StringList dotDescriptions;
+            for (const auto& dot : dotList2) {
+                if (dot.isPartOfSlurBarre) {
+                    continue;
+                }
+
+                int actualFret = dot.fret + offset;
+                Char typeChar = u'O';
+
+                switch (dot.dtype) {
+                case FretDotType::NORMAL:   typeChar = u'O';
+                    break;
+                case FretDotType::CROSS:    typeChar = u'X';
+                    break;
+                case FretDotType::SQUARE:   typeChar = u'S';
+                    break;
+                case FretDotType::TRIANGLE: typeChar = u'T';
+                    break;
+                default: break;
+                }
+
+                dotDescriptions.push_back(String::number(actualFret) + u'-' + typeChar);
+            }
+
+            patternParts.push_back(u'[' + dotDescriptions.join(u",") + u']');
+        } else {
+            patternParts.push_back(u"-");
+        }
+    }
+
+    String pattern = patternParts.join(u"");
+
+    const BarreMap& barres = diagram->barres();
+    StringList barreParts;
+    for (const auto& [fret, b] : barres) {
+        if (!b.exists()) {
+            continue;
+        }
+
+        int adjustedFret = fret + offset;
+        int start = b.startString;
+        int end = (b.endString != -1) ? b.endString : strings - 1;
+
+        barreParts.push_back(u'B' + String::number(adjustedFret)
+                             + u'[' + String::number(start)
+                             + u'-' + String::number(end) + u']');
+    }
+
+    if (!barreParts.empty()) {
+        pattern += u';' + barreParts.join(u";");
+    }
+
+    return pattern;
+}
+
+std::vector<String> FretDiagram::patternHarmonies(const String& pattern)
+{
+    return muse::value(s_diagramPatternToHarmoniesMap, pattern);
+}
+
 //---------------------------------------------------------
 //   clear
 //---------------------------------------------------------
@@ -538,6 +860,12 @@ void FretDiagram::clear()
     m_barres.clear();
     m_dots.clear();
     m_markers.clear();
+    m_fretOffset = 0;
+}
+
+bool FretDiagram::isClear() const
+{
+    return m_barres.empty() && m_dots.empty() && m_markers.empty();
 }
 
 //---------------------------------------------------------
@@ -552,6 +880,11 @@ void FretDiagram::undoFretClear()
     }
 }
 
+int FretDiagram::numPos() const
+{
+    return style().styleI(Sid::fretNumPos);
+}
+
 //---------------------------------------------------------
 //   dot
 //    take fret value of zero to mean all dots
@@ -623,6 +956,11 @@ Font FretDiagram::fingeringFont() const
     return f;
 }
 
+String FretDiagram::harmonyText() const
+{
+    return m_harmony ? m_harmony->plainText() : String();
+}
+
 //---------------------------------------------------------
 //   setHarmony
 ///   if this is being done by the user, use undoSetHarmony instead
@@ -649,13 +987,30 @@ void FretDiagram::add(EngravingItem* e)
     e->setParent(this);
     if (e->isHarmony()) {
         m_harmony = toHarmony(e);
+
         m_harmony->setTrack(track());
-        if (m_harmony->propertyFlags(Pid::OFFSET) == PropertyFlags::STYLED) {
-            m_harmony->resetProperty(Pid::OFFSET);
+
+        //! on the same lavel as diagram
+        m_harmony->setZ(z());
+
+        if (m_harmony->harmonyName().empty()) {
+            if (s_diagramPatternToHarmoniesMap.empty()) {
+                readHarmonyToDiagramFile(HARMONY_TO_DIAGRAM_FILE_PATH);
+            }
+
+            String pattern = patternFromDiagram(this);
+            if (!pattern.empty()) {
+                std::vector<String> matchedHarmonies = muse::value(s_diagramPatternToHarmoniesMap, pattern);
+                if (!matchedHarmonies.empty()) {
+                    m_harmony->setHarmony(matchedHarmonies.front());
+                }
+            }
         }
 
-        m_harmony->setProperty(Pid::ALIGN, Align(AlignH::HCENTER, AlignV::TOP));
+        m_harmony->resetProperty(Pid::OFFSET);
+        m_harmony->setProperty(Pid::ALIGN, Align(AlignH::HCENTER, AlignV::BASELINE));
         m_harmony->setPropertyFlags(Pid::ALIGN, PropertyFlags::UNSTYLED);
+
         e->added();
     } else {
         LOGW("FretDiagram: cannot add <%s>\n", e->typeName());
@@ -676,6 +1031,15 @@ void FretDiagram::remove(EngravingItem* e)
     }
 }
 
+RectF FretDiagram::drag(EditData& ed)
+{
+    RectF result = EngravingItem::drag(ed);
+
+    MoveElementAnchors::moveElementAnchorsOnDrag(this, ed);
+
+    return result;
+}
+
 //---------------------------------------------------------
 //   acceptDrop
 //---------------------------------------------------------
@@ -738,8 +1102,6 @@ PropertyValue FretDiagram::getProperty(Pid propertyId) const
         return showNut();
     case Pid::FRET_OFFSET:
         return fretOffset();
-    case Pid::FRET_NUM_POS:
-        return m_numPos;
     case Pid::ORIENTATION:
         return m_orientation;
     case Pid::FRET_SHOW_FINGERINGS:
@@ -773,9 +1135,6 @@ bool FretDiagram::setProperty(Pid propertyId, const PropertyValue& v)
     case Pid::FRET_OFFSET:
         setFretOffset(v.toInt());
         break;
-    case Pid::FRET_NUM_POS:
-        m_numPos = v.toInt();
-        break;
     case Pid::ORIENTATION:
         m_orientation = v.value<Orientation>();
         break;
@@ -785,6 +1144,29 @@ bool FretDiagram::setProperty(Pid propertyId, const PropertyValue& v)
     case Pid::FRET_FINGERING:
         setFingering(v.value<std::vector<int> >());
         break;
+    case Pid::EXCLUDE_VERTICAL_ALIGN: {
+        bool val = v.toBool();
+        setExcludeVerticalAlign(val);
+        Harmony* h = harmony();
+        if (h && h->excludeVerticalAlign() != val) {
+            h->setExcludeVerticalAlign(val);
+        }
+        Segment* parentSeg = segment();
+        if (!parentSeg) {
+            break;
+        }
+
+        for (EngravingItem* item : parentSeg->annotations()) {
+            if (!item->isFretDiagram() || !item->isHarmony() || item == this || track2staff(item->track()) != staffIdx()) {
+                continue;
+            }
+
+            if (item->excludeVerticalAlign() != val) {
+                item->setProperty(Pid::EXCLUDE_VERTICAL_ALIGN, val);
+            }
+        }
+        break;
+    }
     default:
         return EngravingItem::setProperty(propertyId, v);
     }
@@ -816,6 +1198,15 @@ PropertyValue FretDiagram::propertyDefault(Pid pid) const
     return EngravingItem::propertyDefault(pid);
 }
 
+void FretDiagram::setVisible(bool f)
+{
+    EngravingItem::setVisible(f);
+
+    if (m_harmony && m_harmony->isStyled(Pid::OFFSET)) {
+        m_harmony->resetProperty(Pid::OFFSET);
+    }
+}
+
 void FretDiagram::setTrack(track_idx_t val)
 {
     EngravingItem::setTrack(val);
@@ -825,17 +1216,6 @@ void FretDiagram::setTrack(track_idx_t val)
     }
 }
 
-//---------------------------------------------------------
-//   endEditDrag
-//---------------------------------------------------------
-
-void FretDiagram::endEditDrag(EditData& editData)
-{
-    EngravingItem::endEditDrag(editData);
-
-    triggerLayout();
-}
-
 //---------------------------------------------------------
 //   accessibleInfo
 //---------------------------------------------------------
@@ -962,6 +1342,114 @@ void FretDiagram::setFingering(std::vector<int> v)
     m_fingering = std::move(v);
 }
 
+FretDiagram* FretDiagram::makeFromHarmonyOrFretDiagram(const EngravingItem* harmonyOrFretDiagram)
+{
+    IF_ASSERT_FAILED(!harmonyOrFretDiagram || !harmonyOrFretDiagram->isHarmony() || !harmonyOrFretDiagram->isFretDiagram()) {
+        return nullptr;
+    }
+
+    FretDiagram* fretDiagram = nullptr;
+
+    if (harmonyOrFretDiagram->isHarmony() && !harmonyOrFretDiagram->parentItem()->isFretDiagram()) {
+        Harmony* harmony = toHarmony(harmonyOrFretDiagram)->clone();
+
+        fretDiagram = Factory::createFretDiagram(harmonyOrFretDiagram->score()->dummy()->segment());
+
+        fretDiagram->updateDiagram(harmony->plainText());
+
+        fretDiagram->add(harmony);
+    } else if (harmonyOrFretDiagram->isHarmony() && harmonyOrFretDiagram->parentItem()->isFretDiagram()) {
+        fretDiagram = toFretDiagram(harmonyOrFretDiagram->parentItem())->clone();
+    } else if (harmonyOrFretDiagram->isFretDiagram()) {
+        fretDiagram = toFretDiagram(harmonyOrFretDiagram)->clone();
+        fretDiagram->setOffset(PointF());
+        if (!fretDiagram->harmony()) {
+            //! generate from diagram and add harmony
+            fretDiagram->add(Factory::createHarmony(harmonyOrFretDiagram->score()->dummy()->segment()));
+        }
+    }
+
+    return fretDiagram;
+}
+
+bool FretDiagram::isInFretBox() const
+{
+    EngravingObject* parent = explicitParent();
+    return parent ? parent->isFBox() : false;
+}
+
+bool FretDiagram::isCustom(const String& harmonyNameForCompare) const
+{
+    if (harmonyNameForCompare.empty()) {
+        return false;
+    }
+
+    static const std::list<Pid> props {
+        Pid::FRET_STRINGS,
+        Pid::FRET_FRETS,
+        Pid::FRET_NUT,
+        Pid::FRET_FINGERING
+    };
+
+    for (const Pid& pid : props) {
+        if (pid == Pid::FRET_FINGERING) {
+            if (custom(Pid::FRET_SHOW_FINGERINGS)) {
+                return true;
+            }
+        } else if (custom(pid)) {
+            return true;
+        }
+    }
+
+    if (s_harmonyToDiagramMap.empty()) {
+        readHarmonyToDiagramFile(HARMONY_TO_DIAGRAM_FILE_PATH);
+    }
+
+    NoteSpellingType spellingType = style().styleV(Sid::chordSymbolSpelling).value<NoteSpellingType>();
+    HarmonyMapKey key = createHarmonyMapKey(harmonyNameForCompare, spellingType, score()->chordList());
+
+    std::vector<DiagramInfo> availableDiagrams = muse::value(s_harmonyToDiagramMap, key);
+    if (availableDiagrams.empty()) {
+        return true;
+    }
+
+    String currentPattern = patternFromDiagram(this);
+
+    for (const DiagramInfo& diagram : availableDiagrams) {
+        if (diagram.diagramPattern == currentPattern) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
+void FretDiagram::readHarmonyToDiagramFile(const muse::io::path_t& filePath) const
+{
+    TRACEFUNC;
+
+    muse::io::File file(filePath);
+    if (!file.open()) {
+        LOGE() << file.errorString();
+        return;
+    }
+
+    XmlReader reader(&file);
+
+    std::unordered_map<String, rw::HarmonyToDiagramReader::FretDiagramInfo> harmonyToDiagramMap
+        = rw::HarmonyToDiagramReader::read(reader);
+
+    const ChordList* chordList = score()->chordList();
+    const NoteSpellingType spellingType = NoteSpellingType::STANDARD;
+
+    for (auto& [key, value] : harmonyToDiagramMap) {
+        HarmonyMapKey mapKey = createHarmonyMapKey(key, spellingType, chordList);
+
+        s_harmonyToDiagramMap[mapKey].push_back({ key, value.xml, value.pattern });
+        s_diagramPatternToHarmoniesMap[value.pattern].push_back(key);
+    }
+}
+
 //---------------------------------------------------------
 //   markerToChar
 //---------------------------------------------------------
@@ -1065,6 +1553,17 @@ FretUndoData::FretUndoData(FretDiagram* fd)
     m_dots = m_diagram->m_dots;
     m_markers = m_diagram->m_markers;
     m_barres = m_diagram->m_barres;
+
+    m_strings = m_diagram->m_strings;
+    m_frets = m_diagram->m_frets;
+    m_fretOffset = m_diagram->m_fretOffset;
+    m_maxFrets = m_diagram->m_maxFrets;
+    m_showNut = m_diagram->m_showNut;
+    m_orientation = m_diagram->m_orientation;
+
+    m_userMag = m_diagram->m_userMag;
+
+    m_showFingering = m_diagram->m_showFingering;
 }
 
 //---------------------------------------------------------
@@ -1083,5 +1582,16 @@ void FretUndoData::updateDiagram()
     m_diagram->m_barres = m_barres;
     m_diagram->m_markers = m_markers;
     m_diagram->m_dots = m_dots;
+
+    m_diagram->m_strings = m_strings;
+    m_diagram->m_frets = m_frets;
+    m_diagram->m_fretOffset = m_fretOffset;
+    m_diagram->m_maxFrets = m_maxFrets;
+    m_diagram->m_showNut = m_showNut;
+    m_diagram->m_orientation = m_orientation;
+
+    m_diagram->m_userMag = m_userMag;
+
+    m_diagram->m_showFingering = m_showFingering;
 }
 }
