@@ -72,7 +72,6 @@ struct HarmonyRenderItem {
 
     virtual double leftPadding() const = 0;
     virtual double rightPadding() const = 0;
-    virtual double bboxBaseLine() const = 0;
 
     virtual HarmonyRenderItemType type() const = 0;
 
@@ -101,8 +100,6 @@ struct ChordSymbolParen : HarmonyRenderItem {
     double leftPadding() const override { return parenItem->direction() == DirectionH::LEFT ? OUTER_PADDING : INNER_PADDING; }
     double rightPadding() const override { return parenItem->direction() == DirectionH::LEFT ? INNER_PADDING : OUTER_PADDING; }
 
-    double bboxBaseLine() const override { return bottom; }
-
     HarmonyRenderItemType type() const override { return HarmonyRenderItemType::PAREN; }
 
 private:
@@ -117,7 +114,6 @@ private:
 struct TextSegment : HarmonyRenderItem {
     double width() const;
     double capHeight() const;
-    double bboxBaseLine() const override;
     RectF boundingRect() const override;
     RectF tightBoundingRect() const override;
 
