@@ -37,6 +37,7 @@
 #include "chordrest.h"
 #include "clef.h"
 #include "engravingitem.h"
+#include "hammeronpulloff.h"
 #include "harppedaldiagram.h"
 #include "hook.h"
 #include "instrchange.h"
@@ -323,28 +324,6 @@ Segment* Segment::next1(SegmentType types) const
     return 0;
 }
 
-Segment* Segment::next1ChordRestOrTimeTick() const
-{
-    Segment* nextSeg = next1(CHORD_REST_OR_TIME_TICK_TYPE);
-    while (nextSeg && nextSeg->tick() == tick()) {
-        nextSeg = nextSeg->next1(CHORD_REST_OR_TIME_TICK_TYPE);
-    }
-    if (!nextSeg) {
-        return nullptr;
-    }
-
-    Segment* nextNextSeg = nextSeg->next1(CHORD_REST_OR_TIME_TICK_TYPE);
-    if (!nextNextSeg) {
-        return nextSeg;
-    }
-
-    if (nextSeg->tick() == nextNextSeg->tick()) {
-        return nextSeg->isChordRestType() ? nextSeg : nextNextSeg;
-    }
-
-    return nextSeg;
-}
-
 Segment* Segment::next1WithElemsOnStaff(staff_idx_t staffIdx, SegmentType segType) const
 {
     Segment* next = next1(segType);
@@ -358,6 +337,17 @@ Segment* Segment::next1WithElemsOnStaff(staff_idx_t staffIdx, SegmentType segTyp
     return next;
 }
 
+Segment* Segment::next1WithElemsOnTrack(track_idx_t trackIdx, SegmentType segType) const
+{
+    Segment* next = next1(segType);
+
+    while (next && !next->hasElements(trackIdx, trackIdx)) {
+        next = next->next1(segType);
+    }
+
+    return next;
+}
+
 Segment* Segment::next1MM(SegmentType types) const
 {
     for (Segment* s = next1MM(); s; s = s->next1MM()) {
@@ -423,6 +413,17 @@ Segment* Segment::prev(SegmentType types) const
     return 0;
 }
 
+Segment* Segment::prevWithElementsOnTrack(track_idx_t trackIdx, SegmentType segType) const
+{
+    Segment* prevSeg = prev(segType);
+
+    while (prevSeg && !prevSeg->element(trackIdx)) {
+        prevSeg = prevSeg->prev(segType);
+    }
+
+    return prevSeg;
+}
+
 //---------------------------------------------------------
 //   prev1
 ///   return previous \a Segment, don’t stop searching at
@@ -438,28 +439,6 @@ Segment* Segment::prev1() const
     return m ? m->last() : 0;
 }
 
-Segment* Segment::prev1ChordRestOrTimeTick() const
-{
-    Segment* prevSeg = prev1(CHORD_REST_OR_TIME_TICK_TYPE);
-    while (prevSeg && prevSeg->tick() == tick()) {
-        prevSeg = prevSeg->prev1(CHORD_REST_OR_TIME_TICK_TYPE);
-    }
-    if (!prevSeg) {
-        return nullptr;
-    }
-
-    Segment* prevPrevSeg = prevSeg->prev1(CHORD_REST_OR_TIME_TICK_TYPE);
-    if (!prevPrevSeg) {
-        return prevSeg;
-    }
-
-    if (prevSeg->tick() == prevPrevSeg->tick()) {
-        return prevSeg->isChordRestType() ? prevSeg : prevPrevSeg;
-    }
-
-    return prevSeg;
-}
-
 Segment* Segment::prev1WithElemsOnStaff(staff_idx_t staffIdx, SegmentType segType) const
 {
     Segment* prev = prev1(segType);
@@ -710,12 +689,14 @@ void Segment::add(EngravingItem* el)
         setEmpty(false);
         break;
 
+    case ElementType::HARMONY:
+    case ElementType::FRET_DIAGRAM:
+        score()->rebuildFretBox();
+    //fallthrough
     case ElementType::TEMPO_TEXT:
     case ElementType::DYNAMIC:
     case ElementType::EXPRESSION:
-    case ElementType::HARMONY:
     case ElementType::SYMBOL:
-    case ElementType::FRET_DIAGRAM:
     case ElementType::STAFF_TEXT:
     case ElementType::SYSTEM_TEXT:
     case ElementType::TRIPLET_FEEL:
@@ -730,7 +711,6 @@ void Segment::add(EngravingItem* el)
     case ElementType::FIGURED_BASS:
     case ElementType::FERMATA:
     case ElementType::STICKING:
-    case ElementType::PARENTHESIS:
         m_annotations.push_back(el);
         break;
 
@@ -740,6 +720,11 @@ void Segment::add(EngravingItem* el)
         break;
     }
 
+    case ElementType::PLAY_COUNT_TEXT:
+        assert(isType(SegmentType::BarLineType));
+        m_annotations.push_back(el);
+        break;
+
     case ElementType::STAFF_STATE:
         if (toStaffState(el)->staffStateType() == StaffStateType::INSTRUMENT) {
             StaffState* ss = toStaffState(el);
@@ -903,11 +888,13 @@ void Segment::remove(EngravingItem* el)
         m_elist[track] = 0;
         break;
 
+    case ElementType::HARMONY:
+    case ElementType::FRET_DIAGRAM:
+        score()->rebuildFretBox();
+    //fallthrough
     case ElementType::DYNAMIC:
     case ElementType::EXPRESSION:
     case ElementType::FIGURED_BASS:
-    case ElementType::FRET_DIAGRAM:
-    case ElementType::HARMONY:
     case ElementType::IMAGE:
     case ElementType::MARKER:
     case ElementType::REHEARSAL_MARK:
@@ -923,7 +910,7 @@ void Segment::remove(EngravingItem* el)
     case ElementType::TREMOLOBAR:
     case ElementType::FERMATA:
     case ElementType::STICKING:
-    case ElementType::PARENTHESIS:
+    case ElementType::PLAY_COUNT_TEXT:
         removeAnnotation(el);
         break;
 
@@ -1900,7 +1887,16 @@ EngravingItem* Segment::lastElementOfSegment(staff_idx_t activeStaff) const
 
                 const std::vector<Articulation*>& articulations = chord->articulations();
                 if (!articulations.empty()) {
-                    return articulations.back();
+                    Articulation* lastArtic = articulations.back();
+                    if (lastArtic->isTapping()) {
+                        Tapping* tap = toTapping(lastArtic);
+                        if (tap->halfSlurBelow()) {
+                            return tap->halfSlurBelow()->frontSegment();
+                        } else if (tap->halfSlurAbove()) {
+                            return tap->halfSlurAbove()->frontSegment();
+                        }
+                    }
+                    return lastArtic;
                 }
                 return chord->upNote();
             }
@@ -2010,7 +2006,6 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
     case ElementType::REHEARSAL_MARK:
     case ElementType::MARKER:
     case ElementType::IMAGE:
-    case ElementType::PARENTHESIS:
     case ElementType::TEXT:
     case ElementType::TREMOLOBAR:
     case ElementType::TAB_DURATION_SYMBOL:
@@ -2023,6 +2018,13 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
         EngravingItem* next = nullptr;
         if (e->explicitParent() == this) {
             next = nextAnnotation(e);
+
+            if (next && next->isFretDiagram()) {
+                FretDiagram* fretDiagram = toFretDiagram(next);
+                if (fretDiagram->harmony()) {
+                    next = fretDiagram->harmony();
+                }
+            }
         }
         if (next) {
             return next;
@@ -2038,6 +2040,18 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
             if (nextEl) {
                 return nextEl;
             }
+
+            if (EngravingItem* annotation = nextSegment->firstAnnotation(activeStaff)) {
+                if (annotation && annotation->isFretDiagram()) {
+                    FretDiagram* fretDiagram = toFretDiagram(annotation);
+                    if (fretDiagram->harmony()) {
+                        annotation = fretDiagram->harmony();
+                    }
+                }
+
+                return annotation;
+            }
+
             nextSegment = nextSegment->next1MMenabled();
         }
         break;
@@ -2046,6 +2060,12 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
         if (!m_annotations.empty()) {
             EngravingItem* next = firstAnnotation(activeStaff);
             if (next) {
+                if (next->isFretDiagram()) {
+                    if (Harmony* harmony = toFretDiagram(next)->harmony()) {
+                        return harmony;
+                    }
+                }
+
                 return next;
             }
         }
@@ -2056,6 +2076,10 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
 
         Segment* nextSegment = this->next1MMenabled();
         while (nextSegment) {
+            if (EngravingItem* annotation = nextSegment->firstAnnotation(activeStaff)) {
+                return annotation;
+            }
+
             EngravingItem* nextEl = nextSegment->firstElementOfSegment(activeStaff);
             if (nextEl) {
                 return nextEl;
@@ -2094,6 +2118,11 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
                         return soundFlag;
                     }
                 }
+                if (next->isFretDiagram()) {
+                    if (Harmony* harmony = toFretDiagram(next)->harmony()) {
+                        return harmony;
+                    }
+                }
 
                 return next;
             }
@@ -2102,6 +2131,17 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
         if (s) {
             return s->spannerSegments().front();
         }
+
+        for (SpannerSegment* spannerSeg : system()->spannerSegments()) {
+            if (spannerSeg->staffIdx() == activeStaff && spannerSeg->isHammerOnPullOffSegment()) {
+                for (HammerOnPullOffText* hopoText : toHammerOnPullOffSegment(spannerSeg)->hopoText()) {
+                    if (hopoText->startChord() && hopoText->startChord()->segment() == this) {
+                        return hopoText;
+                    }
+                }
+            }
+        }
+
         Segment* nextSegment = seg->next1MMenabled();
         for (; nextSegment && nextSegment->isTimeTickType(); nextSegment = nextSegment->next1MMenabled()) {
             if (EngravingItem* annotation = nextSegment->firstAnnotation(activeStaff)) {
@@ -2133,7 +2173,7 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
                 if (lockInd) {
                     return lockInd;
                 }
-            }
+            } //TODO: StaffVisibilityIndicator handling
         }
 
         while (nextSegment) {
@@ -2141,6 +2181,11 @@ EngravingItem* Segment::nextElement(staff_idx_t activeStaff)
             if (nextEl) {
                 return nextEl;
             }
+
+            if (EngravingItem* annotation = nextSegment->firstAnnotation(activeStaff)) {
+                return annotation;
+            }
+
             nextSegment = nextSegment->next1MMenabled();
         }
     }
@@ -2179,7 +2224,6 @@ EngravingItem* Segment::prevElement(staff_idx_t activeStaff)
     case ElementType::STRING_TUNINGS:
     case ElementType::REHEARSAL_MARK:
     case ElementType::MARKER:
-    case ElementType::PARENTHESIS:
     case ElementType::IMAGE:
     case ElementType::TEXT:
     case ElementType::TREMOLOBAR:
@@ -2195,13 +2239,19 @@ EngravingItem* Segment::prevElement(staff_idx_t activeStaff)
             }
         }
 
-        EngravingItem* prev = nullptr;
+        EngravingItem* prevAnnotation = nullptr;
         if (e->explicitParent() == this) {
-            prev = prevAnnotation(e);
+            prevAnnotation = this->prevAnnotation(e);
         }
-        if (prev) {
-            return prev;
+        if (prevAnnotation) {
+            return prevAnnotation;
+        } else if (const Segment* prevSeg = prev1()) {
+            const Spanner* s = prevSeg->lastSpanner(activeStaff);
+            if (s) {
+                return s->spannerSegments().back();
+            }
         }
+
         if (notChordRestType()) {
             EngravingItem* lastEl = lastElementOfSegment(activeStaff);
             if (lastEl) {
@@ -2216,7 +2266,14 @@ EngravingItem* Segment::prevElement(staff_idx_t activeStaff)
         Segment* s = this;
         EngravingItem* el = s->element(track);
         while (track > 0 && (!el || el->staffIdx() != activeStaff)) {
+            if (s != this) {
+                if (EngravingItem* annotation = s->lastAnnotation(activeStaff)) {
+                    return annotation;
+                }
+            }
+
             el = s->element(--track);
+
             if (track == 0) {
                 track = score()->nstaves() * VOICES - 1;
                 s = s->prev1MMenabled();
@@ -2288,7 +2345,19 @@ EngravingItem* Segment::prevElement(staff_idx_t activeStaff)
             } else {
                 return prev;
             }
+        } else {
+            System* system = seg->system();
+            for (SpannerSegment* spannerSeg : system->spannerSegments()) {
+                if (spannerSeg->staffIdx() == activeStaff && spannerSeg->isHammerOnPullOffSegment()) {
+                    for (HammerOnPullOffText* hopoText : toHammerOnPullOffSegment(spannerSeg)->hopoText()) {
+                        if (hopoText->endChord() && hopoText->endChord()->segment() == seg) {
+                            return hopoText;
+                        }
+                    }
+                }
+            }
         }
+
         Segment* prevSeg = seg->prev1MMenabled();
         for (; prevSeg && prevSeg->isTimeTickType(); prevSeg = prevSeg->prev1MMenabled()) {
             if (Spanner* spanner = prevSeg->lastSpanner(activeStaff)) {
@@ -2320,13 +2389,24 @@ EngravingItem* Segment::prevElement(staff_idx_t activeStaff)
                 }
             } else if (psm != pmb) {
                 return pmb;
-            }
+            } //TODO: StaffVisibilityIndicator handling
+        }
+
+        if (EngravingItem* annotation = prevSeg->lastAnnotation(activeStaff)) {
+            return annotation;
+        }
+        if (!prev) {
+            prev = prevSeg->lastElementOfSegment(activeStaff);
         }
 
-        prev = prevSeg->lastElementOfSegment(activeStaff);
         while (!prev && prevSeg) {
             prevSeg = prevSeg->prev1MMenabled();
-            prev = prevSeg->lastElementOfSegment(activeStaff);
+
+            if (EngravingItem* annotation = prevSeg->lastAnnotation(activeStaff)) {
+                prev = annotation;
+            } else {
+                prev = prevSeg->lastElementOfSegment(activeStaff);
+            }
         }
         if (!prevSeg) {
             return score()->firstElement();
@@ -2510,7 +2590,6 @@ void Segment::createShapes()
     for (size_t staffIdx = 0; staffIdx < score()->nstaves(); ++staffIdx) {
         createShape(staffIdx);
     }
-    addPreAppendedToShape();
 }
 
 //---------------------------------------------------------
@@ -2530,17 +2609,6 @@ void Segment::createShape(staff_idx_t staffIdx)
         }
     }
 
-    if (segmentType() & (SegmentType::BarLine | SegmentType::EndBarLine | SegmentType::StartRepeatBarLine | SegmentType::BeginBarLine)) {
-        setVisible(true);
-        BarLine* bl = toBarLine(element(staffIdx * VOICES));
-        if (bl) {
-            rendering::score::LayoutContext lctx(score());
-            RectF r = rendering::score::TLayout::layoutRect(bl, lctx);
-            s.add(r.translated(bl->pos() + bl->staffOffset()), bl);
-        }
-        return;
-    }
-
     if (!score()->staff(staffIdx)->show()) {
         return;
     }
@@ -2554,23 +2622,19 @@ void Segment::createShape(staff_idx_t staffIdx)
         track_idx_t effectiveTrack = e->vStaffIdx() * VOICES + e->voice();
         if (effectiveTrack >= strack && effectiveTrack < etrack) {
             setVisible(true);
-            if (e->isRest() && toRest(e)->ticks() >= measure()->ticks() && measure()->hasVoices(e->staffIdx())) {
+            if (e->isRest() && toRest(e)->isFullMeasureRest() && measure()->hasVoices(e->staffIdx())) {
                 // A full measure rest in a measure with multiple voices must be ignored
                 continue;
             }
             if (e->isMMRest() || (e->isMeasureRepeat() && toMeasureRepeat(e)->numMeasures() > 1)) {
                 continue;
             }
-            if (e->addToSkyline()) {
+            if (e->addToSkyline() || (e->isChordRest() && toChordRest(e)->lyrics().size() > 0)) {
                 s.add(e->shape().translate((e->isClef() ? e->ldata()->pos() : e->pos()) + e->staffOffset()));
             }
             // Non-standard trills display a cue note that we must add to shape here
             if (e->isChord()) {
-                Ornament* orn = toChord(e)->findOrnament();
-                Chord* cueNoteChord = orn ? orn->cueNoteChord() : nullptr;
-                if (cueNoteChord && cueNoteChord->upNote()->visible()) {
-                    s.add(cueNoteChord->shape().translate(cueNoteChord->pos() + cueNoteChord->staffOffset()));
-                }
+                addArticulationsToShape(toChord(e), s);
             }
         }
     }
@@ -2582,17 +2646,39 @@ void Segment::createShape(staff_idx_t staffIdx)
 
         setVisible(true);
 
+        // Still include harmony attached to fret diagram
+        if (e->isFretDiagram() && !e->addToSkyline() && toFretDiagram(e)->harmony()) {
+            e = toHarmony(toFretDiagram(e)->harmony());
+        }
+
         if (!e->addToSkyline()) {
             continue;
         }
 
         if (e->isHarmony() || e->isFretDiagram()) {
             // TODO: eliminate once and for all this addHorizontalSpace hack [M.S.]
-            RectF bbox = e->ldata()->bbox().translated(e->pos());
+            PointF pos = e->pos();
+            // Harmony attached to invisible fret diagram
+            if (e->isHarmony() && e->parent()->isFretDiagram()) {
+                pos += e->parentItem()->pos();
+            }
+            RectF bbox = e->ldata()->bbox().translated(pos);
+            if (Parenthesis* p = e->leftParen()) {
+                bbox.unite(p->ldata()->bbox().translated(p->pos() + pos));
+            }
+            if (Parenthesis* p = e->rightParen()) {
+                bbox.unite(p->ldata()->bbox().translated(p->pos() + pos));
+            }
             s.addHorizontalSpacing(e, bbox.left(), bbox.right());
             if (e->isFretDiagram()) {
                 if (Harmony* harmony = toFretDiagram(e)->harmony()) {
                     RectF harmBbox = harmony->ldata()->bbox().translated(harmony->pos() + e->pos());
+                    if (Parenthesis* p = harmony->leftParen()) {
+                        harmBbox.unite(p->ldata()->bbox().translated(p->pos() + harmony->pos() + e->pos()));
+                    }
+                    if (Parenthesis* p = harmony->rightParen()) {
+                        harmBbox.unite(p->ldata()->bbox().translated(p->pos() + harmony->pos() + e->pos()));
+                    }
                     s.addHorizontalSpacing(harmony, harmBbox.left(), harmBbox.right());
                 }
             }
@@ -2609,23 +2695,18 @@ void Segment::createShape(staff_idx_t staffIdx)
                    && !e->isTripletFeel()
                    && !e->isInstrumentChange()
                    && !e->isArticulationFamily()
-                   && !e->isFermata()
                    && !e->isStaffText()
                    && !e->isHarpPedalDiagram()
                    && !e->isPlayTechAnnotation()
                    && !e->isCapo()
-                   && !e->isStringTunings()) {
+                   && !e->isStringTunings()
+                   && !e->isPlayCountText()) {
             // annotations added here are candidates for collision detection
-            // lyrics, ...
             s.add(e->shape().translate(e->pos() + e->staffOffset()));
         }
     }
-}
 
-void Segment::addPreAppendedToShape()
-{
-    track_idx_t tracks = score()->ntracks();
-    for (unsigned track = 0; track < tracks; ++track) {
+    for (track_idx_t track = strack; track < etrack; ++track) {
         if (!m_preAppendedItems[track]) {
             continue;
         }
@@ -2639,6 +2720,42 @@ void Segment::addPreAppendedToShape()
     }
 }
 
+void Segment::addArticulationsToShape(const Chord* chord, Shape& shape)
+{
+    auto addTappingHalfSlurToShape = [&] (TappingHalfSlur* slur) {
+        IF_ASSERT_FAILED(!slur->segmentsEmpty()) {
+            return;
+        }
+        TappingHalfSlurSegment* slurSeg = toTappingHalfSlurSegment(slur->frontSegment());
+        Shape slurSegShape = slurSeg->shape();
+        // Semi-hack: we don't know the exact position at this stage, but we know that it
+        // must end approx on the center of the notehead
+        Note* note = slur->up() ? chord->upNote() : chord->downNote();
+        double approxPosX = chord->x() + note->x() + 0.5 * note->headWidth() - slurSegShape.right();
+        slurSegShape.translateX(approxPosX);
+        shape.add(slurSegShape);
+    };
+
+    for (Articulation* art : chord->articulations()) {
+        if (art->isOrnament()) {
+            Chord* cueNoteChord = toOrnament(art)->cueNoteChord();
+            if (cueNoteChord && cueNoteChord->upNote()->visible()) {
+                shape.add(cueNoteChord->shape().translate(cueNoteChord->pos() + cueNoteChord->staffOffset()));
+            }
+        } else if (art->addToSkyline()) {
+            shape.add(art->shape().translated(art->pos() + chord->pos()));
+            if (art->isTapping()) {
+                if (TappingHalfSlur* halfSlur = toTapping(art)->halfSlurAbove()) {
+                    addTappingHalfSlurToShape(halfSlur);
+                }
+                if (TappingHalfSlur* halfSlur = toTapping(art)->halfSlurBelow()) {
+                    addTappingHalfSlurToShape(halfSlur);
+                }
+            }
+        }
+    }
+}
+
 //---------------------------------------------------------
 //   minRight
 //    calculate minimum distance needed to the right
@@ -2647,7 +2764,8 @@ void Segment::addPreAppendedToShape()
 double Segment::minRight() const
 {
     double distance = 0.0;
-    for (const Shape& sh : shapes()) {
+    for (Shape sh : shapes()) {
+        sh.remove_if([](ShapeElement& el) { return el.item() && el.item()->isArticulationOrFermata(); });
         distance = std::max(distance, sh.right());
     }
     return distance;
@@ -2656,7 +2774,8 @@ double Segment::minRight() const
 double Segment::minLeft() const
 {
     double distance = -DBL_MAX;
-    for (const Shape& sh : shapes()) {
+    for (Shape sh : shapes()) {
+        sh.remove_if([](ShapeElement& el) { return el.item() && el.item()->isArticulationOrFermata(); });
         double l = sh.left();
         if (l > distance) {
             distance = l;
