@@ -21,10 +21,6 @@
  */
 #include "slur.h"
 
-#include <cmath>
-
-#include "draw/types/transform.h"
-
 #include "arpeggio.h"
 #include "beam.h"
 #include "chord.h"
@@ -35,7 +31,6 @@
 #include "score.h"
 #include "stem.h"
 #include "system.h"
-#include "tie.h"
 #include "tremolotwochord.h"
 #include "undo.h"
 
@@ -46,8 +41,8 @@ using namespace mu::engraving;
 using namespace muse::draw;
 
 namespace mu::engraving {
-SlurSegment::SlurSegment(System* parent)
-    : SlurTieSegment(ElementType::SLUR_SEGMENT, parent)
+SlurSegment::SlurSegment(System* parent, ElementType type)
+    : SlurTieSegment(type, parent)
 {
 }
 
@@ -251,39 +246,7 @@ void SlurSegment::changeAnchor(EditData& ed, EngravingItem* element)
     }
 
     // update start/end elements (which could be grace notes)
-    for (EngravingObject* lsp : spanner()->linkList()) {
-        Spanner* sp = static_cast<Spanner*>(lsp);
-        if (sp == spanner()) {
-            score()->undo(new ChangeSpannerElements(sp, scr, ecr));
-        } else {
-            EngravingItem* se = 0;
-            EngravingItem* ee = 0;
-            if (scr) {
-                std::list<EngravingObject*> sel = scr->linkList();
-                for (EngravingObject* lcr : sel) {
-                    EngravingItem* le = toEngravingItem(lcr);
-                    if (le->score() == sp->score() && le->track() == sp->track()) {
-                        se = le;
-                        break;
-                    }
-                }
-            }
-            if (ecr) {
-                std::list<EngravingObject*> sel = ecr->linkList();
-                for (EngravingObject* lcr : sel) {
-                    EngravingItem* le = toEngravingItem(lcr);
-                    if (le->score() == sp->score() && le->track() == sp->track2()) {
-                        ee = le;
-                        break;
-                    }
-                }
-            }
-            if (se && ee) {
-                score()->undo(new ChangeStartEndSpanner(sp, se, ee));
-                renderer()->layoutItem(sp);
-            }
-        }
-    }
+    slur()->undoChangeStartEndElements(scr, ecr);
 
     const size_t segments  = spanner()->spannerSegments().size();
     ups(ed.curGrip).off = PointF();
@@ -300,17 +263,11 @@ void SlurSegment::changeAnchor(EditData& ed, EngravingItem* element)
 void SlurSegment::editDrag(EditData& ed)
 {
     Grip g = ed.curGrip;
-    if (g == Grip::NO_GRIP) {
-        return;
-    }
-
-    ups(g).off += ed.delta;
-
-    PointF delta;
 
     switch (g) {
     case Grip::START:
     case Grip::END:
+        ups(g).off += ed.delta;
         //
         // move anchor for slurs/ties
         //
@@ -334,25 +291,25 @@ void SlurSegment::editDrag(EditData& ed)
                 ed.view()->setDropTarget(0);
             }
         }
+        renderer()->computeBezier(this);
         break;
     case Grip::BEZIER1:
-        break;
     case Grip::BEZIER2:
+        ups(g).off += ed.delta;
+        renderer()->computeBezier(this);
         break;
     case Grip::SHOULDER:
         ups(g).off = PointF();
-        delta = ed.delta;
+        renderer()->computeBezier(this, ed.delta);
         break;
     case Grip::DRAG:
-        ups(g).off = PointF();
-        setOffset(offset() + ed.delta);
-        break;
-    case Grip::NO_GRIP:
-    case Grip::GRIPS:
+        ups(Grip::DRAG).off = PointF();
+        roffset() += ed.delta;
         break;
+    default:
+        return;
     }
 
-    renderer()->computeBezier(this, delta);
     triggerLayout();
 }
 
@@ -390,10 +347,14 @@ double SlurSegment::dottedWidth() const
     return style().styleMM(Sid::slurDottedWidth);
 }
 
+Color SlurSegment::curColor() const
+{
+    return EngravingItem::curColor(getProperty(Pid::VISIBLE).toBool(), getProperty(Pid::COLOR).value<Color>());
+}
+
 Slur::Slur(const Slur& s)
     : SlurTie(s)
 {
-    _sourceStemArrangement = s._sourceStemArrangement;
     _connectedElement = s._connectedElement;
     _partialSpannerDirection = s._partialSpannerDirection;
 }
@@ -402,22 +363,12 @@ Slur::Slur(const Slur& s)
 //   Slur
 //---------------------------------------------------------
 
-Slur::Slur(EngravingItem* parent)
-    : SlurTie(ElementType::SLUR, parent)
+Slur::Slur(EngravingItem* parent, ElementType type)
+    : SlurTie(type, parent)
 {
     setAnchor(Anchor::CHORD);
 }
 
-//---------------------------------------------------------
-//   calcStemArrangement
-//---------------------------------------------------------
-
-int Slur::calcStemArrangement(EngravingItem* start, EngravingItem* end)
-{
-    return (start && start->isChord() && toChord(start)->stem() && toChord(start)->stem()->up() ? 2 : 0)
-           + (end && end->isChord() && toChord(end)->stem() && toChord(end)->stem()->up() ? 4 : 0);
-}
-
 double Slur::scalingFactor() const
 {
     Chord* startC = startElement() && startElement()->isChord() ? toChord(startElement()) : nullptr;
@@ -520,6 +471,43 @@ bool Slur::isOutgoing() const
     return _partialSpannerDirection == PartialSpannerDirection::BOTH || _partialSpannerDirection == PartialSpannerDirection::OUTGOING;
 }
 
+void Slur::undoChangeStartEndElements(ChordRest* scr, ChordRest* ecr)
+{
+    for (EngravingObject* lsp : linkList()) {
+        Spanner* sp = static_cast<Spanner*>(lsp);
+        if (sp == this) {
+            score()->undo(new ChangeSpannerElements(this, scr, ecr));
+        } else {
+            EngravingItem* se = 0;
+            EngravingItem* ee = 0;
+            if (scr) {
+                std::list<EngravingObject*> sel = scr->linkList();
+                for (EngravingObject* lcr : sel) {
+                    EngravingItem* le = toEngravingItem(lcr);
+                    if (le->score() == sp->score() && le->track() == sp->track()) {
+                        se = le;
+                        break;
+                    }
+                }
+            }
+            if (ecr) {
+                std::list<EngravingObject*> sel = ecr->linkList();
+                for (EngravingObject* lcr : sel) {
+                    EngravingItem* le = toEngravingItem(lcr);
+                    if (le->score() == sp->score() && le->track() == sp->track2()) {
+                        ee = le;
+                        break;
+                    }
+                }
+            }
+            if (se && ee) {
+                score()->undo(new ChangeStartEndSpanner(sp, se, ee));
+                renderer()->layoutItem(sp);
+            }
+        }
+    }
+}
+
 //---------------------------------------------------------
 //   setTrack
 //---------------------------------------------------------
@@ -539,6 +527,11 @@ bool Slur::isCrossStaff()
                || startCR()->vStaffIdx() != endCR()->vStaffIdx());
 }
 
+bool Slur::hasCrossBeams()
+{
+    return (startCR() && startCR()->beam() && startCR()->beam()->cross()) || (endCR() && endCR()->beam() && endCR()->beam()->cross());
+}
+
 PropertyValue Slur::getProperty(Pid propertyId) const
 {
     switch (propertyId) {
