@@ -106,7 +106,7 @@ staff_idx_t Staff::idx() const
 
 void Staff::triggerLayout() const
 {
-    score()->setLayoutAll(idx());
+    score()->setLayoutAll(idx(), this);
 }
 
 void Staff::triggerLayout(const Fraction& tick)
@@ -218,6 +218,51 @@ bool Staff::trackHasLinksInVoiceZero(track_idx_t track)
     return false;
 }
 
+void Staff::undoSetShowMeasureNumbers(bool show)
+{
+    bool isTopStave = score()->staves().front() == this;
+    if (show) {
+        undoChangeProperty(Pid::SHOW_MEASURE_NUMBERS, isTopStave ? AutoOnOff::AUTO : AutoOnOff::ON);
+    } else {
+        undoChangeProperty(Pid::SHOW_MEASURE_NUMBERS, isTopStave ? AutoOnOff::OFF : AutoOnOff::AUTO);
+    }
+}
+
+bool Staff::shouldShowMeasureNumbers() const
+{
+    MeasureNumberPlacement placementMode = style().styleV(Sid::measureNumberPlacementMode).value<MeasureNumberPlacement>();
+    switch (placementMode) {
+    case MeasureNumberPlacement::ABOVE_SYSTEM:
+        return score()->staves().front() == this;
+    case MeasureNumberPlacement::BELOW_SYSTEM:
+        return score()->staves().back() == this;
+    case MeasureNumberPlacement::ON_SYSTEM_OBJECT_STAVES:
+    {
+        bool isTopStave = score()->staves().front() == this;
+        return (isTopStave && m_showMeasureNumbers != AutoOnOff::OFF) || (isSystemObjectStaff() && m_showMeasureNumbers == AutoOnOff::ON);
+    }
+    case MeasureNumberPlacement::ON_ALL_STAVES:
+        return show();
+    }
+
+    return false;
+}
+
+bool Staff::isLastOfScore() const
+{
+    return score()->staves().empty() ? false : score()->staves().back() == this;
+}
+
+bool Staff::isSystemObjectStaff() const
+{
+    return score() && muse::contains(score()->systemObjectStaves(), const_cast<Staff*>(this));
+}
+
+bool Staff::hasSystemObjectsBelowBottomStaff() const
+{
+    return isSystemObjectStaff() && isLastOfScore() && style().styleB(Sid::systemObjectsBelowBottomStaff);
+}
+
 //---------------------------------------------------------
 //   fillBrackets
 //    make sure index idx is valid
@@ -1194,7 +1239,9 @@ void Staff::staffTypeListChanged(const Fraction& tick)
 
 StaffType* Staff::setStaffType(const Fraction& tick, const StaffType& nst)
 {
-    return m_staffTypeList.setStaffType(tick, nst);
+    StaffType* stt = m_staffTypeList.setStaffType(tick, nst);
+    stt->setScore(score());
+    return stt;
 }
 
 //---------------------------------------------------------
@@ -1243,13 +1290,20 @@ void Staff::init(const InstrumentTemplate* t, const StaffType* staffType, int ci
 void Staff::init(const Staff* s)
 {
     m_id                = s->m_id;
-    m_staffTypeList     = s->m_staffTypeList;
+
+    setStaffType(Fraction(0, 1), s->m_staffTypeList.staffType(Fraction(0, 1)));
+    for (const auto& stPair : s->m_staffTypeList.staffTypeChanges()) {
+        const StaffType& st = stPair.second;
+        StaffType newStaffType(st);
+        setStaffType(Fraction::fromTicks(stPair.first), newStaffType);
+    }
+
     setDefaultClefType(s->defaultClefType());
     m_barLineFrom       = s->m_barLineFrom;
     m_barLineTo         = s->m_barLineTo;
     m_hideWhenEmpty     = s->m_hideWhenEmpty;
     m_cutaway           = s->m_cutaway;
-    m_showIfEmpty       = s->m_showIfEmpty;
+    m_showIfEntireSystemEmpty = s->m_showIfEntireSystemEmpty;
     m_hideSystemBarLine = s->m_hideSystemBarLine;
     m_mergeMatchingRests = s->m_mergeMatchingRests;
     m_color             = s->m_color;
@@ -1291,15 +1345,6 @@ void Staff::initFromStaffType(const StaffType* staffType)
     setStaffType(Fraction(0, 1), *staffType);
 }
 
-//---------------------------------------------------------
-//   spatiumChanged
-//---------------------------------------------------------
-
-void Staff::spatiumChanged(double oldValue, double newValue)
-{
-    m_userDist = (m_userDist / oldValue) * newValue;
-}
-
 //---------------------------------------------------------
 //   show
 //---------------------------------------------------------
@@ -1398,8 +1443,12 @@ void Staff::insertTime(const Fraction& tick, const Fraction& len)
     for (auto i = m_keys.lower_bound(tick.ticks()); i != m_keys.end();) {
         KeySigEvent kse = i->second;
         Fraction t = Fraction::fromTicks(i->first);
-        m_keys.erase(i++);
-        kl2[(t + len).ticks()] = kse;
+        if (t + len < score()->endTick()) {
+            m_keys.erase(i++);
+            kl2[(t + len).ticks()] = kse;
+        } else {
+            ++i;
+        }
     }
     m_keys.insert(kl2.begin(), kl2.end());
 
@@ -1511,6 +1560,8 @@ PropertyValue Staff::getProperty(Pid id) const
         return staffType(Fraction(0, 1))->userMag();
     case Pid::STAFF_INVISIBLE:
         return staffType(Fraction(0, 1))->invisible();
+    case Pid::HIDE_WHEN_EMPTY:
+        return m_hideWhenEmpty;
     case Pid::STAFF_COLOR:
         return PropertyValue::fromValue(staffType(Fraction(0, 1))->color());
     case Pid::PLAYBACK_VOICE1:
@@ -1531,6 +1582,10 @@ PropertyValue Staff::getProperty(Pid id) const
         return userDist();
     case Pid::GENERATED:
         return false;
+    case Pid::SHOW_MEASURE_NUMBERS:
+        return m_showMeasureNumbers;
+    case Pid::SHOW_IF_ENTIRE_SYSTEM_EMPTY:
+        return m_showIfEntireSystemEmpty;
     default:
         LOGD("unhandled id <%s>", propertyName(id));
         return PropertyValue();
@@ -1556,6 +1611,9 @@ bool Staff::setProperty(Pid id, const PropertyValue& v)
         setLocalSpatium(_spatium, spatium(Fraction(0, 1)), Fraction(0, 1));
     }
     break;
+    case Pid::HIDE_WHEN_EMPTY:
+        setHideWhenEmpty(v.value<AutoOnOff>());
+        break;
     case Pid::STAFF_COLOR:
         setColor(Fraction(0, 1), v.value<Color>());
         break;
@@ -1602,7 +1660,13 @@ bool Staff::setProperty(Pid id, const PropertyValue& v)
         setBarLineTo(v.toInt());
         break;
     case Pid::STAFF_USERDIST:
-        setUserDist(v.value<Millimetre>());
+        setUserDist(v.value<Spatium>());
+        break;
+    case Pid::SHOW_MEASURE_NUMBERS:
+        m_showMeasureNumbers = v.value<AutoOnOff>();
+        break;
+    case Pid::SHOW_IF_ENTIRE_SYSTEM_EMPTY:
+        m_showIfEntireSystemEmpty = v.toBool();
         break;
     default:
         LOGD("unhandled id <%s>", propertyName(id));
@@ -1623,6 +1687,8 @@ PropertyValue Staff::propertyDefault(Pid id) const
         return false;
     case Pid::MAG:
         return 1.0;
+    case Pid::HIDE_WHEN_EMPTY:
+        return AutoOnOff::AUTO;
     case Pid::STAFF_COLOR:
         return PropertyValue::fromValue(configuration()->defaultColor());
     case Pid::PLAYBACK_VOICE1:
@@ -1636,7 +1702,11 @@ PropertyValue Staff::propertyDefault(Pid id) const
     case Pid::STAFF_BARLINE_SPAN_TO:
         return 0;
     case Pid::STAFF_USERDIST:
-        return Millimetre(0.0);
+        return Spatium(0.0);
+    case Pid::SHOW_MEASURE_NUMBERS:
+        return AutoOnOff::AUTO;
+    case Pid::SHOW_IF_ENTIRE_SYSTEM_EMPTY:
+        return false;
     default:
         LOGD("unhandled id <%s>", propertyName(id));
         return PropertyValue();
