@@ -54,6 +54,9 @@ void GuitarBendLayout::updateSegmentsAndLayout(SLine* item, LayoutContext& ctx)
         return;
     }
 
+    item->setTick(startNote->tick());
+    item->setTick2(endNote->tick());
+
     unsigned int segmentsNeeded = system1 == system2 ? 1 : 2;
     size_t segmentCount = item->spannerSegments().size();
     if (segmentCount != segmentsNeeded) {
@@ -99,8 +102,8 @@ void GuitarBendLayout::layoutAngularBend(GuitarBendSegment* item, LayoutContext&
         return;
     }
 
-    if (bend->type() == GuitarBendType::PRE_BEND && !startNote->headHasParentheses()) {
-        startNote->setHeadHasParentheses(true, /* addToLinked= */ false, /* generated= */ true);
+    if (bend->type() == GuitarBendType::PRE_BEND && !startNote->bothParentheses()) {
+        startNote->setParenthesesMode(ParenthesesMode::BOTH, /* addToLinked= */ false, /* generated= */ true);
         startNote->mutldata()->reset();
         TLayout::layoutChord(startNote->chord(), ctx);
     }
@@ -308,10 +311,10 @@ void GuitarBendLayout::adjustX(GuitarBendSegment* item, PointF& startPos, PointF
     Chord* startChord = startNote->chord();
     Chord* endChord = endNote->chord();
 
-    bool adjustStart = item->isSingleBeginType() && ((startChord->stem() && startChord->up() == up)
+    bool adjustStart = item->isSingleBeginType() && ((startChord->stem() && startChord->up() && up)
                                                      || (up && startNote != startChord->upNote())
                                                      || (!up && startNote != startChord->downNote()));
-    bool adjustEnd = item->isSingleEndType() && ((endChord->stem() && endChord->up() == up)
+    bool adjustEnd = item->isSingleEndType() && ((endChord->stem() && !endChord->up() && !up)
                                                  || (up && endNote != endChord->upNote())
                                                  || (!up && endNote != endChord->downNote()));
 
@@ -431,6 +434,7 @@ void GuitarBendLayout::layoutTabStaff(GuitarBendSegment* item, LayoutContext& ct
     GuitarBendSegment* prevBendSeg = prevBend && !prevBend->segmentsEmpty() && !prevBend->isFullRelease()
                                      ? toGuitarBendSegment(prevBend->backSegment()) : nullptr;
     GuitarBendHoldSegment* prevHoldLine = prevBend && prevBend->holdLine() && !prevBend->holdLine()->segmentsEmpty()
+                                          && prevBend->endNote() != bend->startNote()
                                           ? toGuitarBendHoldSegment(prevBend->holdLine()->backSegment()) : nullptr;
 
     PointF prevEndPoint = PointF(0.0, 0.0);
@@ -451,6 +455,7 @@ void GuitarBendLayout::layoutTabStaff(GuitarBendSegment* item, LayoutContext& ct
     }
 
     endPos = computeEndPos(item, endNote, distAboveTab, verticalPad, arrowHeight, arrowWidth, startPos, prevEndPoint);
+    endPos.setX(std::max(startPos.x(), endPos.x())); // Edge case for ultra-ultra-narrow situations: ensure bend doesn't end before its start
 
     vertex = bend->type() == GuitarBendType::PRE_BEND && !bend->angledPreBend()
              ? 0.5 * (startPos + endPos) : PointF(endPos.x(), startPos.y());
