@@ -202,6 +202,9 @@ void HarmonyLayout::layoutModifierParentheses(const Harmony* item)
     std::vector<ChordSymbolParen*> openingParenStack;
     double lastTextSegHeight = 0.0;
     double lastTextSegTop = 0.0;
+    double rootRefHeight = 0.0;
+    double rootCapHeight = 0.0;
+    double parenExtension = 0.1 * spatium * (item->size() / 10.0);
     for (HarmonyRenderItem* renderItem : itemList) {
         if (ChordSymbolParen* curParen = dynamic_cast<ChordSymbolParen*>(renderItem)) {
             if (curParen->parenItem->direction() == DirectionH::LEFT) {
@@ -218,28 +221,36 @@ void HarmonyLayout::layoutModifierParentheses(const Harmony* item)
                 curParen->bottom = openingParen->bottom;
 
                 // Layout parenthesis pair
-                double extension = 0.1 * spatium * (item->size() / 10.0);
-                double startY = openingParen->top - extension;
-                double height = (openingParen->bottom - openingParen->top) + 2 * extension;
+                double startY = openingParen->top - parenExtension;
+                double height = (openingParen->bottom - openingParen->top) + 2 * parenExtension;
                 if (std::isinf(height)) {
                     height = lastTextSegHeight;
                 }
                 if (muse::RealIsEqual(DBL_MAX, startY)) {
                     startY = lastTextSegTop;
                 }
-                double midPointThickness = height / 30 * openingParen->parenItem->ldata()->mag();
-                double endPointThickness = 0.05;
+
+                const double mag = openingParen->parenItem->ldata()->mag();
+                const double scale = (height - 2 * parenExtension) / rootCapHeight;
+                static constexpr double HEIGHT_TO_WIDTH_RATIO = 20;
+                const double midPointThickness = height / HEIGHT_TO_WIDTH_RATIO * mag * 1 / std::sqrt(scale);
+                const double endPointThickness = 0.03;
+                const double shoulder = 0.2 * height * std::pow(mag, 0.1) * 1 / std::sqrt(scale);
+
                 openingParen->parenItem->mutldata()->startY = startY;
                 openingParen->sety(startY);
                 openingParen->parenItem->mutldata()->height = height;
                 openingParen->parenItem->mutldata()->midPointThickness.set_value(midPointThickness);
                 openingParen->parenItem->mutldata()->endPointThickness.set_value(endPointThickness);
+                openingParen->parenItem->mutldata()->shoulderWidth = shoulder;
 
                 curParen->parenItem->mutldata()->startY = startY;
                 curParen->sety(startY);
                 curParen->parenItem->mutldata()->height = height;
                 curParen->parenItem->mutldata()->midPointThickness.set_value(midPointThickness);
                 curParen->parenItem->mutldata()->endPointThickness.set_value(endPointThickness);
+                curParen->parenItem->mutldata()->shoulderWidth = shoulder;
+
                 double closingPos = openingParen->closingParenPos;
                 if (muse::RealIsEqual(-DBL_MAX, closingPos)) {
                     closingPos = openingParen->x() + openingParen->boundingRect().width();
@@ -259,10 +270,19 @@ void HarmonyLayout::layoutModifierParentheses(const Harmony* item)
             // Set top paren height
             lastTextSegHeight = textSeg->height();
             lastTextSegTop = textSeg->tightBoundingRect().translated(textSeg->pos()).y();
+            if (muse::RealIsNull(rootRefHeight)) {
+                rootRefHeight = textSeg->height();
+                double top = textSeg->tightBoundingRect().translated(textSeg->pos()).y();
+                double bottom =  textSeg->pos().y();
+                double height = (bottom - top) + 2 * parenExtension;
+
+                rootRefHeight = height;
+                rootCapHeight = textSeg->capHeight();
+            }
             if (!openingParenStack.empty() && textSeg->font().type() != Font::Type::MusicSymbolText) {
                 ChordSymbolParen* topParen = openingParenStack.back();
                 topParen->top = std::min(topParen->top, textSeg->tightBoundingRect().translated(textSeg->pos()).y());
-                topParen->bottom = std::max(topParen->bottom, textSeg->bboxBaseLine() + textSeg->pos().y());
+                topParen->bottom = std::max(topParen->bottom, textSeg->pos().y());
                 topParen->closingParenPos = std::max(topParen->closingParenPos, textSeg->x() + textSeg->width());
                 continue;
             }
@@ -290,22 +310,32 @@ void HarmonyLayout::layoutModifierParentheses(const Harmony* item)
     // TODO - STACKED C7((#11)#13)
     // C7(#11(#13))
     double additionalSpace = 0.0;
+    bool prevParen = false;
+
     for (size_t i = 0; i < itemList.size(); i++) {
         HarmonyRenderItem* renderItem = itemList.at(i);
         double padding = i != 0 ? computePadding(itemList.at(i - 1), renderItem) : 0.0;
 
         if (ChordSymbolParen* paren = dynamic_cast<ChordSymbolParen*>(renderItem)) {
             if (paren->parenItem->direction() == DirectionH::LEFT) {
-                additionalSpace += paren->parenItem->width() + padding;
+                double heightToRefRatio = std::max(paren->height(), rootRefHeight) / rootRefHeight;
+                bool scaleParenWidth = !prevParen && !muse::RealIsEqual(heightToRefRatio, 1.0);
+                const double PAREN_PADDING_SCALER = 0.6;
+                double scaledParenWidth = paren->parenItem->width() * PAREN_PADDING_SCALER / heightToRefRatio;
+
+                double parenWidth = scaleParenWidth ? scaledParenWidth : paren->parenItem->width();
+                additionalSpace += parenWidth + padding;
                 paren->movex(additionalSpace);
             }
             if (paren->parenItem->direction() == DirectionH::RIGHT) {
                 paren->movex(additionalSpace + padding);
                 additionalSpace += paren->parenItem->width() + padding;
             }
+            prevParen = true;
         } else if (TextSegment* ts = dynamic_cast<TextSegment*>(renderItem)) {
             additionalSpace += padding;
             ts->movex(additionalSpace);
+            prevParen = false;
         }
     }
 }
@@ -615,7 +645,7 @@ void HarmonyLayout::renderAction(Harmony* item, Harmony::LayoutData* ldata, cons
 {
     switch (a->actionType()) {
     case RenderAction::RenderActionType::SET:
-        renderActionSet(item, ldata, std::static_pointer_cast<RenderActionSet>(a), harmonyCtx);
+        renderActionSet(item, ldata, std::static_pointer_cast<RenderActionSet>(a), harmonyCtx, ctx);
         break;
     case RenderAction::RenderActionType::MOVE:
         renderActionMove(item, std::static_pointer_cast<RenderActionMove>(a), harmonyCtx);
@@ -780,12 +810,14 @@ void HarmonyLayout::renderActionParen(Harmony* item, const RenderActionParenPtr&
     harmonyCtx.renderItemList.push_back(parenItem);
 }
 
-void HarmonyLayout::kernCharacters(const Harmony* item, const String& text, HarmonyRenderCtx& harmonyCtx)
+void HarmonyLayout::kernCharacters(const Harmony* item, const String& text, HarmonyRenderCtx& harmonyCtx, const LayoutContext& ctx)
 {
     if (harmonyCtx.renderItemList.empty()) {
         return;
     }
     // Character pair and distance to move the second
+    // Blank strings will apply the kerning no matter the following character
+    // TODO - move this information to the XML file
     static const std::map<std::pair<String, String>, double> KERNED_CHARACTERS {
         { { u"A", u"\uE870" }, -0.4 },  // dim
         { { u"A", u"\uE871" }, -0.3 },  // half-dim
@@ -794,9 +826,15 @@ void HarmonyLayout::kernCharacters(const Harmony* item, const String& text, Harm
         { { u"A", u"/" }, 0.1 },
 
         { { u"A", u"\uE18E" }, -0.15 },  // dim JAZZ
-        { { u"A", u"\uE18F" }, -0.15 },  // hal-dim JAZZ
+        { { u"A", u"\uE18F" }, -0.15 },  // half-dim JAZZ
         { { u"\uE18A", u"\uE18E" }, -0.15 },  // triangle - dim JAZZ
         { { u"\uE18A", u"\uE18F" }, -0.15 },  // triangle - half-dim JAZZ
+
+        { { u"\u266D", u"" }, -0.15 },  // b JAZZ
+        { { u"\u266E", u"" }, -0.15 },  // natural JAZZ
+        { { u"\u266F", u"" }, -0.15 },  // # JAZZ
+        { { u"\u1D12A", u"" }, -0.15 }, // ## JAZZ
+        { { u"\u1D12B", u"" }, -0.15 }, // bb JAZZ
     };
 
     HarmonyRenderItem* prevSeg = harmonyCtx.renderItemList.back();
@@ -807,7 +845,16 @@ void HarmonyLayout::kernCharacters(const Harmony* item, const String& text, Harm
 
     for (auto& kernInfo : KERNED_CHARACTERS) {
         const std::pair<String, String> kernPair = kernInfo.first;
-        if (ts->text().endsWith(kernPair.first) && text.startsWith(kernPair.second)) {
+        bool endChar = ts->text().endsWith(kernPair.first);
+        bool startChar = text.startsWith(kernPair.second);
+        // VERY DIRTY HACK ALERT
+        // "Match any" should only be applied to the Jazz preset currently.
+        // Remove the jazz condition when these kern values are moved to the XML files
+        if (kernPair.second.isEmpty() && ctx.conf().styleV(Sid::chordStyle).value<ChordStylePreset>() != ChordStylePreset::JAZZ) {
+            continue;
+        }
+        bool startMatchAny = kernPair.second.isEmpty();
+        if ((endChar && startChar) || (endChar && startMatchAny)) {
             const FontMetrics fm = FontMetrics(item->font());
             const double scale = harmonyCtx.scale * item->mag();
             harmonyCtx.pos = harmonyCtx.pos + PointF(kernInfo.second, 0.0) * FontMetrics::capHeight(item->font()) * scale;
@@ -816,7 +863,8 @@ void HarmonyLayout::kernCharacters(const Harmony* item, const String& text, Harm
     }
 }
 
-void HarmonyLayout::renderActionSet(Harmony* item, Harmony::LayoutData* ldata, const RenderActionSetPtr& a, HarmonyRenderCtx& harmonyCtx)
+void HarmonyLayout::renderActionSet(Harmony* item, Harmony::LayoutData* ldata, const RenderActionSetPtr& a, HarmonyRenderCtx& harmonyCtx,
+                                    const LayoutContext& ctx)
 {
     const ChordList* chordList = item->score()->chordList();
     const ChordSymbol cs = chordList->symbol(a->text());
@@ -828,7 +876,7 @@ void HarmonyLayout::renderActionSet(Harmony* item, Harmony::LayoutData* ldata, c
         font.setPointSizeF(font.pointSizeF() * nmag);
     }
 
-    kernCharacters(item, text, harmonyCtx);
+    kernCharacters(item, text, harmonyCtx, ctx);
 
     TextSegment* ts = new TextSegment(text, font, harmonyCtx.x(), harmonyCtx.y(), harmonyCtx.hAlign);
     harmonyCtx.movex(ts->width());
