@@ -43,6 +43,7 @@
 #include "dom/spacer.h"
 #include "dom/spannermap.h"
 #include "dom/staff.h"
+#include "dom/staffvisibilityindicator.h"
 #include "dom/system.h"
 #include "dom/systemdivider.h"
 #include "dom/tremolosinglechord.h"
@@ -178,11 +179,11 @@ void PageLayout::collectPage(LayoutContext& ctx)
                         Spacer* sp = m->vspacerUp(0);
                         if (sp) {
                             if (sp->spacerType() == SpacerType::FIXED) {
-                                distance = sp->gap();
+                                distance = sp->absoluteGap();
                                 fixedDistance = true;
                                 break;
                             } else {
-                                distance = std::max(distance, sp->gap().val());
+                                distance = std::max(distance, sp->absoluteGap());
                             }
                         }
                     }
@@ -305,19 +306,9 @@ void PageLayout::collectPage(LayoutContext& ctx)
                             continue;
                         }
                         ChordRest* cr = toChordRest(e2);
-                        if (BeamLayout::notTopBeam(cr)) {                           // layout cross staff beams
+                        if (BeamLayout::isStartOfCrossBeam(cr)) {                           // layout cross staff beams
                             TLayout::layoutBeam(cr->beam(), ctx);
                             BeamLayout::checkCrossPosAndStemConsistency(cr->beam(), ctx);
-                            for (EngravingItem* item : cr->beam()->elements()) {
-                                if (!item || !item->isRest()) {
-                                    continue;
-                                }
-                                Rest* rest = toRest(item);
-                                Beam* beam = rest->beam();
-                                if (beam && beam->fullCross() && rest->staffMove()) {
-                                    BeamLayout::verticalAdjustBeamedRests(rest, beam, ctx);
-                                }
-                            }
                         }
                         if (TupletLayout::notTopTuplet(cr)) {
                             // fix layout of tuplets
@@ -365,6 +356,10 @@ void PageLayout::collectPage(LayoutContext& ctx)
             MeasureLayout::layout2(m, ctx);
         }
         SystemLayout::layoutSystemLockIndicators(s, ctx);
+
+        if (StaffVisibilityIndicator* visibilityIndicator = s->staffVisibilityIndicator()) {
+            TLayout::layoutIndicatorIcon(visibilityIndicator, visibilityIndicator->mutldata());
+        }
     }
 
     // If this is the last page we layout, we must also relayout the first barlines of the
@@ -443,8 +438,18 @@ void PageLayout::layoutCrossStaffSlurs(LayoutContext& ctx, System* system)
         if (!sp->isSlur() || sp->tick() == system->endTick()) {
             continue;
         }
-        if (toSlur(sp)->isCrossStaff()) {
-            SlurTieLayout::layoutSystem(toSlur(sp), system, ctx);
+        Slur* slur = toSlur(sp);
+        if (slur->isCrossStaff() || slur->hasCrossBeams()) {
+            SlurTieLayout::layoutSystem(slur, system, ctx);
+
+            ChordRest* scr = toChordRest(slur->startElement());
+            ChordRest* ecr = toChordRest(slur->endElement());
+            if (scr && scr->isChord()) {
+                ChordLayout::layoutArticulations3(toChord(scr), slur, ctx);
+            }
+            if (ecr && ecr->isChord()) {
+                ChordLayout::layoutArticulations3(toChord(ecr), slur, ctx);
+            }
         }
     }
 }
@@ -746,7 +751,7 @@ void PageLayout::distributeStaves(LayoutContext& ctx, Page* page, double footerP
     double spaceRemaining{ std::min(page->height() - combinedBottomMargin - yBottom, page->height() - marginToStaff - prevYBottom) };
 
     if (nextSpacer) {
-        spaceRemaining -= std::max(0.0, nextSpacer->gap() - spacerOffset - staffLowerBorder);
+        spaceRemaining -= std::max(0.0, nextSpacer->absoluteGap() - spacerOffset - staffLowerBorder);
     }
     if (spaceRemaining <= 0.0) {
         return;
