@@ -183,7 +183,7 @@ void ScoreHorizontalViewLayout::layoutLinear(LayoutContext& ctx)
                         continue;
                     }
                     ChordRest* cr = toChordRest(e);
-                    if (BeamLayout::notTopBeam(cr)) {                           // layout cross staff beams
+                    if (BeamLayout::isStartOfCrossBeam(cr)) {                           // layout cross staff beams
                         TLayout::layoutBeam(cr->beam(), ctx);
                     }
                     if (TupletLayout::notTopTuplet(cr)) {
@@ -233,7 +233,7 @@ void ScoreHorizontalViewLayout::layoutLinear(LayoutContext& ctx)
     auto spanners = ctx.dom().spannerMap().findOverlapping(system->tick().ticks(), system->endTick().ticks());
     for (auto interval : spanners) {
         Spanner* sp = interval.value;
-        if (sp->isSlur() && toSlur(sp)->isCrossStaff()) {
+        if (sp->isSlur() && (toSlur(sp)->isCrossStaff() || toSlur(sp)->hasCrossBeams())) {
             SlurTieLayout::layoutSystem(toSlur(sp), system, ctx);
         }
     }
@@ -254,6 +254,8 @@ void ScoreHorizontalViewLayout::layoutLinear(LayoutContext& ctx)
 
 void ScoreHorizontalViewLayout::layoutSystemLockIndicators(System* system)
 {
+    // TODO: layout StaffVisibilityIndicator here
+
     system->deleteLockIndicators();
 
     std::vector<const SystemLock*> systemLocks = system->score()->systemLocks()->allLocks();
@@ -261,7 +263,7 @@ void ScoreHorizontalViewLayout::layoutSystemLockIndicators(System* system)
         SystemLockIndicator* lockIndicator = Factory::createSystemLockIndicator(system, lock);
         lockIndicator->setParent(system);
         system->addLockIndicator(lockIndicator);
-        TLayout::layoutSystemLockIndicator(lockIndicator, lockIndicator->mutldata());
+        TLayout::layoutIndicatorIcon(lockIndicator, lockIndicator->mutldata());
     }
 }
 
@@ -294,7 +296,7 @@ void ScoreHorizontalViewLayout::collectLinearSystem(LayoutContext& ctx)
     std::set<Measure*> measuresToLayout;
 
     while (ctx.state().curMeasure()) {
-        if (ctx.state().curMeasure()->isVBox() || ctx.state().curMeasure()->isTBox()) {
+        if (ctx.state().curMeasure()->isVBox() || ctx.state().curMeasure()->isTBox() || ctx.state().curMeasure()->isFBox()) {
             ctx.mutState().curMeasure()->resetExplicitParent();
             MeasureLayout::getNextMeasure(ctx);
             continue;
@@ -327,8 +329,6 @@ void ScoreHorizontalViewLayout::collectLinearSystem(LayoutContext& ctx)
                 MeasureLayout::removeSystemTrailer(m);
             }
 
-            MeasureLayout::updateGraceNotes(m, ctx);
-
             if (m->tick() >= ctx.state().startTick() && m->tick() <= ctx.state().endTick()) {
                 // for measures in range, do full layout
                 if (ctx.conf().isMode(LayoutMode::HORIZONTAL_FIXED)) {
@@ -342,6 +342,7 @@ void ScoreHorizontalViewLayout::collectLinearSystem(LayoutContext& ctx)
                     MeasureLayout::computePreSpacingItems(m, ctx);
                     MeasureLayout::createEndBarLines(m, false, ctx);
                     MeasureLayout::setRepeatCourtesiesAndParens(m, ctx);
+                    MeasureLayout::updateGraceNotes(m, ctx);
                     curSystemWidth = HorizontalSpacing::updateSpacingForLastAddedMeasure(system, firstMeasureInLayout);
                     measuresToLayout.insert(m);
                     if (firstMeasureInLayout) {
