@@ -80,6 +80,8 @@
 #include "../../dom/note.h"
 #include "../../dom/notedot.h"
 
+#include "../../dom/parenthesis.h"
+#include "../../dom/playcounttext.h"
 #include "../../dom/playtechannotation.h"
 
 #include "../../dom/rehearsalmark.h"
@@ -95,6 +97,7 @@
 #include "../../dom/systemtext.h"
 #include "../../dom/soundflag.h"
 
+#include "../../dom/tapping.h"
 #include "../../dom/textbase.h"
 #include "../../dom/tempotext.h"
 #include "../../dom/text.h"
@@ -122,6 +125,9 @@ class GradualTempoChange;
 
 class HairpinSegment;
 class Hairpin;
+class HammerOnPullOff;
+class HammerOnPullOffSegment;
+class HammerOnPullOffText;
 class HarmonicMarkSegment;
 
 class LedgerLine;
@@ -152,7 +158,6 @@ class Slur;
 class Spacer;
 class SpannerSegment;
 class StaffLines;
-class StretchedBend;
 class StringTunings;
 
 class BSymbol;
@@ -163,6 +168,7 @@ class SystemDivider;
 class SystemText;
 
 class TabDurationSymbol;
+class Tapping;
 class TempoText;
 class Text;
 class TextLine;
@@ -200,7 +206,7 @@ public:
     static void layoutAmbitus(const Ambitus* item, Ambitus::LayoutData* ldata, const LayoutContext& ctx);
     static void layoutArpeggio(const Arpeggio* item, Arpeggio::LayoutData* ldata, const LayoutConfiguration& conf,
                                bool includeCrossStaffHeight = false);
-    static void layoutArticulation(const Articulation* item, Articulation::LayoutData* ldata);
+    static void layoutArticulation(Articulation* item, Articulation::LayoutData* ldata);
     static void fillArticulationShape(const Articulation* item, Articulation::LayoutData* ldata);
 
     static void layoutBarLine(const BarLine* item, BarLine::LayoutData* ldata, const LayoutContext& ctx);
@@ -215,7 +221,7 @@ public:
     static void layoutHBox2(HBox* item, const LayoutContext& ctx);
     static void layoutVBox(const VBox* item, VBox::LayoutData* ldata, const LayoutContext& ctx);
     static void layoutFBox(const FBox* item, FBox::LayoutData* ldata, const LayoutContext& ctx);
-    static void layoutTBox(const TBox* item, FBox::LayoutData* ldata, const LayoutContext& ctx);
+    static void layoutTBox(const TBox* item, TBox::LayoutData* ldata, const LayoutContext& ctx);
 
     static void layoutBracket(const Bracket* item, Bracket::LayoutData* ldata, const LayoutConfiguration& conf);
     static void layoutBreath(const Breath* item, Breath::LayoutData* ldata, const LayoutConfiguration& conf);
@@ -230,7 +236,7 @@ public:
 
     static void layoutExpression(const Expression* item, Expression::LayoutData* ldata);
 
-    static void layoutFermata(const Fermata* item, Fermata::LayoutData* ldata, const LayoutConfiguration& conf);
+    static void layoutFermata(const Fermata* item, Fermata::LayoutData* ldata);
     static void layoutFiguredBass(const FiguredBass* item, FiguredBass::LayoutData* ldata, const LayoutContext& ctx);
     static void layoutFingering(const Fingering* item, Fingering::LayoutData* ldata);
     static void layoutFretDiagram(const FretDiagram* item, FretDiagram::LayoutData* ldata, const LayoutContext& ctx);
@@ -247,12 +253,17 @@ public:
 
     static void layoutHairpinSegment(HairpinSegment* item, LayoutContext& ctx);
     static void layoutHairpin(Hairpin* item, LayoutContext& ctx);
+    static void layoutHammerOnPullOff(HammerOnPullOff* item, LayoutContext& ctx);
+    static void layoutHammerOnPullOffSegment(HammerOnPullOffSegment* item, LayoutContext& ctx);
+    static void layoutHammerOnPullOffText(HammerOnPullOffText* item, LayoutContext& ctx);
     static void fillHairpinSegmentShape(const HairpinSegment* item, HairpinSegment::LayoutData* ldata);
     static void layoutHarpPedalDiagram(const HarpPedalDiagram* item, HarpPedalDiagram::LayoutData* ldata);
     static void layoutHarmonicMarkSegment(HarmonicMarkSegment* item, LayoutContext& ctx);
-    static void layoutHarmony(const Harmony* item, Harmony::LayoutData* ldata, const LayoutContext& ctx);
+    static void layoutHarmony(Harmony* item, Harmony::LayoutData* ldata, const LayoutContext& ctx);
     static void layoutHook(const Hook* item, Hook::LayoutData* ldata);
 
+    static void layoutIndicatorIcon(const IndicatorIcon* item, IndicatorIcon::LayoutData* ldata);
+
     static void layoutImage(const Image* item, Image::LayoutData* ldata);
     static void layoutInstrumentChange(const InstrumentChange* item, InstrumentChange::LayoutData* ldata);
     static void layoutInstrumentName(const InstrumentName* item, InstrumentName::LayoutData* ldata);
@@ -263,7 +274,6 @@ public:
 
     static void layoutLaissezVib(LaissezVib* item);
     static void layoutLayoutBreak(const LayoutBreak* item, LayoutBreak::LayoutData* ldata);
-    static void layoutSystemLockIndicator(const SystemLockIndicator* item, SystemLockIndicator::LayoutData* ldata);
     static void layoutLedgerLine(LedgerLine* item, LayoutContext& ctx);
     static void layoutLetRing(LetRing* item, LayoutContext& ctx);
     static void layoutLetRingSegment(LetRingSegment* item, LayoutContext& ctx);
@@ -272,15 +282,14 @@ public:
     static void layoutLyricsLine(LyricsLine* item, LayoutContext& ctx);
     static void layoutLyricsLineSegment(LyricsLineSegment* item, LayoutContext& ctx);
 
-    static void layoutMarker(const Marker* item, Marker::LayoutData* ldata);
+    static void layoutMarker(const Marker* item, Marker::LayoutData* ldata, LayoutContext& ctx);
     static void layoutMeasureBase(MeasureBase* item, LayoutContext& ctx); // factory
     static void layoutBaseMeasureBase(const MeasureBase* item, MeasureBase::LayoutData* ldata, const LayoutContext& ctx); // base class
-    static void layoutMeasureNumber(const MeasureNumber* item, MeasureNumber::LayoutData* ldata, const LayoutContext& ctx);
-    static void layoutMeasureNumberBase(const MeasureNumberBase* item, MeasureNumberBase::LayoutData* ldata, const LayoutContext& ctx); // base class
+    static void layoutMeasureNumber(MeasureNumber* item, MeasureNumber::LayoutData* ldata, const LayoutContext& ctx);
     static void layoutMeasureRepeat(const MeasureRepeat* item, MeasureRepeat::LayoutData* ldata, const LayoutContext& ctx);
     static void layoutMeasureRepeatExtender(const MeasureRepeat* item, MeasureRepeat::LayoutData* ldata, const LayoutContext& ctx);
     static void layoutMMRest(const MMRest* item, MMRest::LayoutData* ldata, const LayoutContext& ctx);
-    static void layoutMMRestRange(const MMRestRange* item, MMRestRange::LayoutData* ldata, const LayoutContext& ctx);
+    static void layoutMMRestRange(MMRestRange* item, MMRestRange::LayoutData* ldata, const LayoutContext& ctx);
 
     static void layoutNote(const Note* item, Note::LayoutData* ldata);
     static void fillNoteShape(const Note* item, Note::LayoutData* ldata);
@@ -297,10 +306,11 @@ public:
 
     static void layoutPalmMute(PalmMute* item, LayoutContext& ctx);
     static void layoutPalmMuteSegment(PalmMuteSegment* item, LayoutContext& ctx);
-    static void layoutParenthesis(Parenthesis* item, LayoutContext& ctx);
+    static void layoutParenthesis(Parenthesis* item, Parenthesis::LayoutData* ldata, const LayoutContext& ctx);
     static void layoutPedal(Pedal* item, LayoutContext& ctx);
     static void layoutPedalSegment(PedalSegment* item, LayoutContext& ctx);
     static void layoutPickScrapeSegment(PickScrapeSegment* item, LayoutContext& ctx);
+    static void layoutPlayCountText(PlayCountText* item, PlayCountText::LayoutData* ldata);
     static void layoutPlayTechAnnotation(const PlayTechAnnotation* item, PlayTechAnnotation::LayoutData* ldata);
 
     static void layoutRasgueadoSegment(RasgueadoSegment* item, LayoutContext& ctx);
@@ -320,8 +330,6 @@ public:
     static void layoutStem(const Stem* item, Stem::LayoutData* ldata, const LayoutConfiguration& conf);
     static void layoutStemSlash(const StemSlash* item, StemSlash::LayoutData* ldata, const LayoutConfiguration& conf);
     static void layoutSticking(const Sticking* item, Sticking::LayoutData* ldata);
-    static void layoutStretchedBend(StretchedBend* item, LayoutContext& ctx);
-    static void layoutStretched(StretchedBend* item, LayoutContext& ctx);
     static void layoutStringTunings(StringTunings* item, LayoutContext& ctx);
     static void layoutSoundFlag(const SoundFlag* item, SoundFlag::LayoutData* ldata);
 
@@ -332,6 +340,8 @@ public:
     static void layoutSystemText(const SystemText* item, SystemText::LayoutData* ldata);
 
     static void layoutTabDurationSymbol(const TabDurationSymbol* item, TabDurationSymbol::LayoutData* ldata);
+    static void layoutTapping(Tapping* item, Tapping::LayoutData* ldata, LayoutContext& ctx);
+    static void layoutTappingHalfSlur(TappingHalfSlur* item);
     static void layoutTempoText(const TempoText* item, TempoText::LayoutData* ldata);
 
     static void layoutTextBase(TextBase* item, LayoutContext& ctx);                 // factory
@@ -367,7 +377,7 @@ public:
 
     static void layoutWhammyBarSegment(WhammyBarSegment* item, LayoutContext& ctx);
 
-    static RectF layoutRect(const BarLine* item, LayoutContext& ctx);
+    static void updateBarlineShape(const BarLine* item, BarLine::LayoutData* ldata, const LayoutContext& ctx);
 
     // layoutSystem;
     static SpannerSegment* layoutSystem(Spanner* item, System* system, LayoutContext& ctx); // factory
@@ -383,15 +393,18 @@ private:
     static void layoutFiguredBassItem(const FiguredBassItem* item, FiguredBassItem::LayoutData* ldata, const LayoutContext& ctx);
 
     static SpannerSegment* layoutSystemSLine(SLine* line, System* system, LayoutContext& ctx);
+    static double voltaMidEndSegmentStartX(Volta* volta, System* system, LayoutContext& ctx);
     static SpannerSegment* getNextLayoutSystemSegment(Spanner* spanner, System* system,
                                                       std::function<SpannerSegment* (System* parent)> createSegment);
 
     static Shape textLineBaseSegmentShape(const TextLineBaseSegment* item);
-    static void layoutDynamicToEndOfPrevious(const Dynamic* item, Dynamic::LayoutData* ldata, const LayoutConfiguration& conf);
 
     static void manageHairpinSnapping(HairpinSegment* item, LayoutContext& ctx);
 
     static void checkRehearsalMarkVSBigTimeSig(const RehearsalMark* item, RehearsalMark::LayoutData* ldata);
+
+    static void manageTempoChangeSnapping(GradualTempoChangeSegment* item, LayoutContext& ctx);
+    static void doLayoutGradualTempoChangeSegment(GradualTempoChangeSegment* item, LayoutContext& ctx);
 };
 }
 
