@@ -71,7 +71,6 @@ void MeasureRead::readMeasure(Measure* measure, XmlReader& e, ReadContext& ctx,
         return;
     }
 
-    double _spatium = measure->spatium();
     ctx.setCurrentMeasure(measure);
     int nextTrack = staffIdx * VOICES;
     ctx.setTrack(nextTrack);
@@ -91,8 +90,8 @@ void MeasureRead::readMeasure(Measure* measure, XmlReader& e, ReadContext& ctx,
         bool ok = true;
         measure->m_len = Fraction::fromString(e.attribute("len"), &ok);
         if (!ok || measure->m_len < Fraction(1, 128)) {
-            e.raiseError(muse::mtrc("engraving", "MSCX error at line %1: invalid measure length: %2")
-                         .arg(e.lineNumber()).arg(e.attribute("len")));
+            e.raiseError(muse::mtrc("engraving", "MSCX error at byte offset %1: invalid measure length: %2")
+                         .arg(e.byteOffset()).arg(e.attribute("len")));
             return;
         }
         irregular = true;
@@ -147,7 +146,7 @@ void MeasureRead::readMeasure(Measure* measure, XmlReader& e, ReadContext& ctx,
                 spacer->setTrack(staffIdx * VOICES);
                 measure->add(spacer);
             }
-            measure->m_mstaves[staffIdx]->vspacerDown()->setGap(Millimetre(e.readDouble() * _spatium));
+            measure->m_mstaves[staffIdx]->vspacerDown()->setGap(Spatium(e.readDouble()));
         } else if (tag == "vspacerFixed") {
             if (!measure->m_mstaves[staffIdx]->vspacerDown()) {
                 Spacer* spacer = Factory::createSpacer(measure);
@@ -155,7 +154,7 @@ void MeasureRead::readMeasure(Measure* measure, XmlReader& e, ReadContext& ctx,
                 spacer->setTrack(staffIdx * VOICES);
                 measure->add(spacer);
             }
-            measure->m_mstaves[staffIdx]->vspacerDown()->setGap(Millimetre(e.readDouble() * _spatium));
+            measure->m_mstaves[staffIdx]->vspacerDown()->setGap(Spatium(e.readDouble()));
         } else if (tag == "vspacerUp") {
             if (!measure->m_mstaves[staffIdx]->vspacerUp()) {
                 Spacer* spacer = Factory::createSpacer(measure);
@@ -163,7 +162,7 @@ void MeasureRead::readMeasure(Measure* measure, XmlReader& e, ReadContext& ctx,
                 spacer->setTrack(staffIdx * VOICES);
                 measure->add(spacer);
             }
-            measure->m_mstaves[staffIdx]->vspacerUp()->setGap(Millimetre(e.readDouble() * _spatium));
+            measure->m_mstaves[staffIdx]->vspacerUp()->setGap(Spatium(e.readDouble()));
         } else if (tag == "visible") {
             measure->m_mstaves[staffIdx]->setVisible(e.readInt());
         } else if ((tag == "slashStyle") || (tag == "stemless")) {
