@@ -5,7 +5,7 @@
  * MuseScore
  * Music Composition & Notation
  *
- * Copyright (C) 2021 MuseScore BVBA and others
+ * Copyright (C) 2025 MuseScore BVBA and others
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -19,8 +19,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
-#ifndef MU_AUDIO_AUDIOSTUBMODULE_H
-#define MU_AUDIO_AUDIOSTUBMODULE_H
+
+#pragma once
 
 #include "modularity/imodulesetup.h"
 #include "async/asyncable.h"
@@ -36,5 +36,3 @@ public:
     void registerUiTypes() override;
 };
 }
-
-#endif // MU_AUDIO_AUDIOSTUBMODULE_H
