@@ -25,11 +25,13 @@
 
 #include <QObject>
 
+#include "global/async/asyncable.h"
+
 #include "modularity/ioc.h"
 #include "iinteractive.h"
 
 namespace muse::uicomponents {
-class ColorPickerModel : public QObject, public muse::Injectable
+class ColorPickerModel : public QObject, public muse::Injectable, public async::Asyncable
 {
     Q_OBJECT
 
@@ -38,7 +40,11 @@ class ColorPickerModel : public QObject, public muse::Injectable
 public:
     explicit ColorPickerModel(QObject* parent = nullptr);
 
-    Q_INVOKABLE QColor selectColor(const QColor& currentColor);
+    Q_INVOKABLE void selectColor(const QColor& currentColor);
+
+signals:
+    void colorSelected(QColor color);
+    void selectRejected();
 };
 }
 
