@@ -183,8 +183,8 @@ void PopupView::initCloseController()
 
     m_closeController->setParentItem(parentItem());
     m_closeController->setWindow(window());
-    m_closeController->setPopupHasFocus(!(m_openPolicies & OpenPolicy::NoActivateFocus));
     m_closeController->setIsCloseOnPressOutsideParent(m_closePolicies & ClosePolicy::CloseOnPressOutsideParent);
+    m_closeController->setCanClosed(!m_closePolicies.testFlag(ClosePolicy::NoAutoClose));
 
     m_closeController->closeNotification().onNotify(this, [this]() {
         close(true);
@@ -238,6 +238,8 @@ void PopupView::doOpen()
 
     beforeOpen();
 
+    resolveParentWindow();
+
     updateGeometry();
 
     if (!isDialog()) {
@@ -310,6 +312,7 @@ void PopupView::close(bool force)
     }
 
     if (m_closeController) {
+        m_closeController->setCanClosed(true);
         m_closeController->setActive(false);
     }
 
@@ -330,11 +333,6 @@ void PopupView::toggleOpened()
     }
 }
 
-void PopupView::setParentWindow(QWindow* window)
-{
-    m_window->setParentWindow(window);
-}
-
 bool PopupView::isOpened() const
 {
     return m_window ? m_window->isVisible() : false;
@@ -350,9 +348,9 @@ PopupView::ClosePolicies PopupView::closePolicies() const
     return m_closePolicies;
 }
 
-PopupView::Placement PopupView::placement() const
+PopupView::PlacementPolicies PopupView::placementPolicies() const
 {
-    return m_placement;
+    return m_placementPolicies;
 }
 
 bool PopupView::activateParentOnClose() const
@@ -476,11 +474,6 @@ void PopupView::setOpenPolicies(PopupView::OpenPolicies openPolicies)
     }
 
     m_openPolicies = openPolicies;
-
-    if (m_closeController) {
-        m_closeController->setPopupHasFocus(!(m_openPolicies & OpenPolicy::NoActivateFocus));
-    }
-
     emit openPoliciesChanged(m_openPolicies);
 }
 
@@ -510,14 +503,14 @@ void PopupView::setClosePolicies(ClosePolicies closePolicies)
     emit closePoliciesChanged(closePolicies);
 }
 
-void PopupView::setPlacement(Placement placement)
+void PopupView::setPlacementPolicies(muse::uicomponents::PopupView::PlacementPolicies placementPolicies)
 {
-    if (m_placement == placement) {
+    if (m_placementPolicies == placementPolicies) {
         return;
     }
 
-    m_placement = placement;
-    emit placementChanged(placement);
+    m_placementPolicies = placementPolicies;
+    emit placementPoliciesChanged(placementPolicies);
 }
 
 void PopupView::setObjectId(QString objectId)
@@ -627,16 +620,6 @@ void PopupView::setRet(QVariantMap ret)
     emit retChanged(m_ret);
 }
 
-void PopupView::setOpensUpward(bool opensUpward)
-{
-    if (m_opensUpward == opensUpward) {
-        return;
-    }
-
-    m_opensUpward = opensUpward;
-    emit opensUpwardChanged(m_opensUpward);
-}
-
 void PopupView::setArrowX(int arrowX)
 {
     if (m_arrowX == arrowX) {
@@ -647,6 +630,26 @@ void PopupView::setArrowX(int arrowX)
     emit arrowXChanged(m_arrowX);
 }
 
+void PopupView::setArrowY(int arrowY)
+{
+    if (m_arrowY == arrowY) {
+        return;
+    }
+
+    m_arrowY = arrowY;
+    emit arrowYChanged(m_arrowY);
+}
+
+void PopupView::setPopupPosition(PopupPosition::Type position)
+{
+    if (m_popupPosition == position) {
+        return;
+    }
+
+    m_popupPosition = position;
+    emit popupPositionChanged(m_popupPosition);
+}
+
 void PopupView::setPadding(int padding)
 {
     if (m_padding == padding) {
@@ -702,9 +705,9 @@ QVariantMap PopupView::ret() const
     return m_ret;
 }
 
-bool PopupView::opensUpward() const
+PopupPosition::Type PopupView::popupPosition() const
 {
-    return m_opensUpward;
+    return m_popupPosition;
 }
 
 int PopupView::arrowX() const
@@ -712,6 +715,11 @@ int PopupView::arrowX() const
     return m_arrowX;
 }
 
+int PopupView::arrowY() const
+{
+    return m_arrowY;
+}
+
 int PopupView::padding() const
 {
     return m_padding;
@@ -734,10 +742,30 @@ void PopupView::setErrCode(Ret::Code code)
     setRet(ret);
 }
 
+QWindow* PopupView::parentWindow() const
+{
+    return m_window->parentWindow();
+}
+
+void PopupView::setParentWindow(QWindow* window)
+{
+    m_window->setParentWindow(window);
+}
+
+void PopupView::resolveParentWindow()
+{
+    if (QQuickItem* parent = parentItem()) {
+        if (QWindow* window = parent->window()) {
+            setParentWindow(window);
+            return;
+        }
+    }
+    setParentWindow(mainWindow()->qWindow());
+}
+
 QScreen* PopupView::resolveScreen() const
 {
-    const QQuickItem* parent = parentItem();
-    const QWindow* parentWindow = parent ? parent->window() : nullptr;
+    const QWindow* parentWindow = this->parentWindow();
     QScreen* screen = parentWindow ? parentWindow->screen() : nullptr;
 
     if (!screen) {
@@ -776,23 +804,58 @@ void PopupView::updateGeometry()
         viewRect.moveTopLeft(m_globalPos);
     };
 
-    bool canFitAbove = viewRect.height() < parentTopLeft.y();
-    bool canFitBelow = viewRect.bottom() < anchorRect.bottom();
+    bool ignoreFit = m_placementPolicies.testFlag(PlacementPolicy::IgnoreFit);
+    bool canFitAbove = !ignoreFit ? viewRect.height() < parentTopLeft.y() : true;
+    bool canFitBelow = !ignoreFit ? viewRect.bottom() < anchorRect.bottom() : true;
+    bool canFitLeft = !ignoreFit ? viewRect.width() < parentTopLeft.x() : true;
+    bool canFitRight = !ignoreFit ? viewRect.right() < anchorRect.right() : true;
 
-    if (placement() == Placement::PreferAbove && canFitAbove) {
-        movePos(m_globalPos.x(), parentTopLeft.y() - viewRect.height());
-        setOpensUpward(true);
-    } else if (placement() == Placement::PreferBelow && canFitBelow) {
+    auto moveBelow = [&]() {
         movePos(m_globalPos.x(), parentTopLeft.y() + parent->height());
-    } else if (!canFitBelow) {
-        if (canFitAbove) {
-            // move to the top of the parent
-            movePos(m_globalPos.x(), parentTopLeft.y() - viewRect.height());
-            setOpensUpward(true);
-        } else {
-            // move to the right of the parent and move to top to an area that doesn't fit
-            movePos(parentTopLeft.x() + parent->width(), m_globalPos.y() - (viewRect.bottom() - anchorRect.bottom()) + padding());
-        }
+        setPopupPosition(PopupPosition::Bottom);
+    };
+
+    auto moveAbove = [&]() {
+        movePos(m_globalPos.x(), parentTopLeft.y() - viewRect.height());
+        setPopupPosition(PopupPosition::Top);
+    };
+
+    auto moveLeft = [&]() {
+        movePos(parentTopLeft.x() - viewRect.width(), m_globalPos.y());
+        setPopupPosition(PopupPosition::Left);
+    };
+
+    auto moveRight = [&]() {
+        movePos(parentTopLeft.x() + parent->width(), m_globalPos.y());
+        setPopupPosition(PopupPosition::Right);
+    };
+
+    bool placementDefault = m_placementPolicies.testFlag(PlacementPolicy::Default);
+    bool preferBelow = m_placementPolicies.testFlag(PlacementPolicy::PreferBelow);
+    bool preferAbove = m_placementPolicies.testFlag(PlacementPolicy::PreferAbove);
+    bool preferLeft = m_placementPolicies.testFlag(PlacementPolicy::PreferLeft);
+    bool preferRight = m_placementPolicies.testFlag(PlacementPolicy::PreferRight);
+
+    if ((preferBelow || placementDefault) && canFitBelow) {
+        moveBelow();
+    } else if ((preferAbove || placementDefault) && canFitAbove) {
+        moveAbove();
+    } else if (preferLeft && canFitLeft) {
+        moveLeft();
+    } else if (preferRight && canFitRight) {
+        moveRight();
+    } else if (!canFitBelow && canFitAbove && (preferBelow || placementDefault)) {
+        moveAbove();
+    } else if (!canFitAbove && canFitBelow && (preferAbove || placementDefault)) {
+        moveBelow();
+    } else if (!canFitLeft && canFitRight && preferLeft) {
+        moveRight();
+    } else if (!canFitRight && canFitLeft && preferRight) {
+        moveLeft();
+    } else {
+        // move to the right of the parent and move to top to an area that doesn't fit
+        movePos(parentTopLeft.x() + parent->width(), m_globalPos.y() - (viewRect.bottom() - anchorRect.bottom()) + padding());
+        setPopupPosition(PopupPosition::Right);
     }
 
     if (viewRect.left() < anchorRect.left()) {
@@ -806,7 +869,11 @@ void PopupView::updateGeometry()
     }
 
     if (!showArrow()) {
-        movePos(m_globalPos.x() - padding(), m_globalPos.y());
+        if (popupPosition() == PopupPosition::Bottom || popupPosition() == PopupPosition::Top) {
+            movePos(m_globalPos.x() - padding(), m_globalPos.y());
+        } else if (popupPosition() == PopupPosition::Left || popupPosition() == PopupPosition::Right) {
+            movePos(m_globalPos.x(), m_globalPos.y() - padding());
+        }
     }
 }
 
@@ -829,6 +896,12 @@ void PopupView::updateContentPosition()
         } else {
             setArrowX(parentTopLeft.x() + (parent->width() / 2) - m_globalPos.x());
         }
+
+        if (parentTopLeft.y() < viewTopLeft.y() || parentTopLeft.y() > viewGeometry.bottom()) {
+            setArrowY(viewGeometry.height() / 2);
+        } else {
+            setArrowY(parentTopLeft.y() + (parent->height() / 2) - m_globalPos.y());
+        }
     }
 }
 
@@ -860,8 +933,6 @@ void PopupView::resolveNavigationParentControl()
         connect(qmlCtrl, &QObject::destroyed, this, [this]() {
             setNavigationParentControl(nullptr);
         });
-
-        setParentWindow(ctrl->window());
     }
 }
 
