@@ -19,10 +19,9 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
-#ifndef IMPORTMIDI_INSTRUMENT_H
-#define IMPORTMIDI_INSTRUMENT_H
+#pragma once
 
-#include "../midishared/midifile.h"
+#include "internal/midishared/generalmidi.h"
 
 class QString;
 
@@ -34,7 +33,7 @@ namespace mu::iex::midi {
 class MTrack;
 
 namespace MidiInstr {
-QString instrumentName(MidiType type, int program, bool isDrumTrack);
+QString instrumentName(GM1Program, bool isDrumTrack);
 QString msInstrName(int trackIndex);
 QString concatenateWithComma(const QString& left, const QString& right);
 bool isGrandStaff(const MTrack& t1, const MTrack& t2);
@@ -45,5 +44,3 @@ void createInstruments(engraving::Score* score, QList<MTrack>& tracks);
 extern void instrumentTemplatesChanged();
 } // namespace MidiInstr
 } // namespace mu::iex::midi
-
-#endif // IMPORTMIDI_INSTRUMENT_H
