@@ -43,6 +43,7 @@ using namespace mu::notation;
 using namespace muse;
 
 static const String TEST_SCORE_PATH(u"data/test.mscx");
+static constexpr bool FLUSH_SOUND = true;
 
 class NotationViewInputControllerTests : public ::testing::Test
 {
@@ -90,6 +91,7 @@ public:
     std::shared_ptr<NotationSelectionMock> m_selection;
     std::shared_ptr<NotationSelectionRangeMock> m_selectionRange;
     std::shared_ptr<playback::PlaybackControllerMock > m_playbackController;
+    playback::IPlaybackController::PlayParams m_playParams;
 
     mutable QList<QInputEvent*> m_events;
 
@@ -120,7 +122,7 @@ public:
         Qt::KeyboardModifiers modifiers = Qt::NoModifier,
         QPointF pos = QPointF(100, 100)) const
     {
-        QMouseEvent* ev = new QMouseEvent(QMouseEvent::Type::MouseButtonPress, pos, button, {}, modifiers);
+        QMouseEvent* ev = new QMouseEvent(QMouseEvent::Type::MouseButtonPress, pos, pos, button, {}, modifiers);
 
         m_events << ev;
 
@@ -194,6 +196,13 @@ public:
     }
 };
 
+namespace mu::playback {
+inline bool operator==(const IPlaybackController::PlayParams& p1, const IPlaybackController::PlayParams& p2)
+{
+    return p1.duration == p2.duration && p1.flushSound == p2.flushSound;
+}
+}
+
 /**
  * @brief WheelEvent_ScrollVertical
  * @details Received wheel event, without modifiers
@@ -278,9 +287,6 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_Range_Start_Drag_From_Selec
     //! [GIVEN] There is a test score
     engraving::MasterScore* score = engraving::ScoreRW::readScore(TEST_SCORE_PATH);
 
-    //! [GIVEN] Previous selected note
-    INotationInteraction::HitElementContext oldContext = hitContext(score, { ElementType::NOTE, true /*first note*/ });
-
     //! [GIVEN] User selected new note that was already selected
     INotationInteraction::HitElementContext newContext = hitContext(score, { ElementType::NOTE, false /*last note*/ });
     newContext.element->setSelected(true);
@@ -300,8 +306,7 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_Range_Start_Drag_From_Selec
     .Times(1);
 
     EXPECT_CALL(*m_interaction, hitElementContext())
-    .Times(2)
-    .WillOnce(ReturnRef(oldContext))
+    .Times(1)
     .WillOnce(ReturnRef(newContext));
 
     //! [GIVEN] There is a range selection
@@ -319,11 +324,11 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_Range_Start_Drag_From_Selec
     .WillByDefault(Return(false));
 
     //! [THEN] We will seek and play selected note, but no select again
-    EXPECT_CALL(*m_playbackController, seekElement(newContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(newContext.element, FLUSH_SOUND))
     .Times(1);
 
     std::vector<const EngravingItem*> elements = { newContext.element };
-    EXPECT_CALL(*m_playbackController, playElements(elements, false))
+    EXPECT_CALL(*m_playbackController, playElements(elements, m_playParams, false))
     .Times(1);
 
     std::vector<EngravingItem*> selectElements = { newContext.element };
@@ -331,7 +336,7 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_Range_Start_Drag_From_Selec
     .Times(0);
 
     //! [WHEN] User pressed left mouse button
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPointF(100, 100)));
 }
 
 /**
@@ -339,7 +344,7 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_Range_Start_Drag_From_Selec
  * @details User pressed left mouse button on already selected text element
  *          We should change text cursor position
  */
-TEST_F(NotationViewInputControllerTests, DISABLED_Mouse_Press_On_Selected_Text_Element)
+TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Selected_Text_Element)
 {
     //! [GIVEN] There is a test score
     engraving::MasterScore* score = engraving::ScoreRW::readScore(TEST_SCORE_PATH);
@@ -365,11 +370,6 @@ TEST_F(NotationViewInputControllerTests, DISABLED_Mouse_Press_On_Selected_Text_E
     EXPECT_CALL(*m_interaction, setHitElementContext(newContext))
     .Times(1);
 
-    EXPECT_CALL(*m_interaction, hitElementContext())
-    .Times(2)
-    .WillOnce(ReturnRef(oldContext))
-    .WillOnce(ReturnRef(newContext));
-
     //! [GIVEN] There isn't a range selection
     ON_CALL(*m_selection, isRange())
     .WillByDefault(Return(false));
@@ -381,15 +381,15 @@ TEST_F(NotationViewInputControllerTests, DISABLED_Mouse_Press_On_Selected_Text_E
     EXPECT_CALL(m_view, isNoteEnterMode())
     .WillOnce(Return(false));
 
-    EXPECT_CALL(*m_playbackController, isPlaying())
-    .WillOnce(Return(false));
+    ON_CALL(*m_playbackController, isPlaying())
+    .WillByDefault(Return(false));
 
     //! [THEN] We will seek and play selected dynamic, but no select again
-    EXPECT_CALL(*m_playbackController, seekElement(newContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(newContext.element, FLUSH_SOUND))
     .Times(1);
 
     std::vector<const EngravingItem*> elements = { newContext.element };
-    EXPECT_CALL(*m_playbackController, playElements(elements, false))
+    EXPECT_CALL(*m_playbackController, playElements(elements, m_playParams, false))
     .Times(0);
 
     std::vector<EngravingItem*> selectElements = { newContext.element };
@@ -407,7 +407,7 @@ TEST_F(NotationViewInputControllerTests, DISABLED_Mouse_Press_On_Selected_Text_E
     .Times(1);
 
     //! [WHEN] User pressed left mouse button
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPointF(100, 100)));
 }
 
 /**
@@ -441,11 +441,6 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Selected_Non_Text_Elemen
     EXPECT_CALL(*m_interaction, setHitElementContext(newContext))
     .Times(1);
 
-    EXPECT_CALL(*m_interaction, hitElementContext())
-    .Times(2)
-    .WillOnce(ReturnRef(oldContext))
-    .WillOnce(ReturnRef(newContext));
-
     //! [GIVEN] There isn't a range selection
     ON_CALL(*m_selection, isRange())
     .WillByDefault(Return(false));
@@ -461,11 +456,11 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Selected_Non_Text_Elemen
     .WillByDefault(Return(false));
 
     //! [THEN] We will seek and play selected hairpin, but no select again
-    EXPECT_CALL(*m_playbackController, seekElement(newContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(newContext.element, FLUSH_SOUND))
     .Times(1);
 
     std::vector<const EngravingItem*> elements = { newContext.element };
-    EXPECT_CALL(*m_playbackController, playElements(elements, false))
+    EXPECT_CALL(*m_playbackController, playElements(elements, m_playParams, false))
     .Times(0);
 
     std::vector<EngravingItem*> selectElements = { newContext.element };
@@ -481,11 +476,11 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Selected_Non_Text_Elemen
     .Times(0);
     EXPECT_CALL(*m_interaction, startEditGrip(newContext.element, _))
     .Times(0);
-    EXPECT_CALL(*m_interaction, startEditElement(newContext.element, _))
+    EXPECT_CALL(*m_interaction, startEditElement(newContext.element))
     .Times(0);
 
     //! [WHEN] User pressed left mouse button
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPointF(100, 100)));
 }
 
 /**
@@ -516,8 +511,7 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_Range_Start_Play_From_First
     .Times(1);
 
     EXPECT_CALL(*m_interaction, hitElementContext())
-    .Times(2)
-    .WillOnce(ReturnRef(oldContext))
+    .Times(1)
     .WillOnce(ReturnRef(newContext));
 
     //! [GIVEN] No note enter mode, no playing
@@ -530,13 +524,14 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_Range_Start_Play_From_First
     //! [THEN] We will select and play selected note, but no seek
     std::vector<EngravingItem*> selectElements = { newContext.element };
     EXPECT_CALL(*m_interaction, select(selectElements, _, _))
-    .Times(1);
+    .Times(1)
+    .WillOnce([newContext] { newContext.element->setSelected(true); });
 
     std::vector<const EngravingItem*> playElements = { newContext.element };
-    EXPECT_CALL(*m_playbackController, playElements(playElements, false))
+    EXPECT_CALL(*m_playbackController, playElements(playElements, m_playParams, false))
     .Times(1);
 
-    EXPECT_CALL(*m_playbackController, seekElement(newContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(newContext.element, FLUSH_SOUND))
     .Times(0);
 
     //! [GIVEN] There is a range selection with two notes
@@ -548,11 +543,11 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_Range_Start_Play_From_First
     .WillOnce(ReturnRef(selectElements));
 
     //! [THEN] We will seek first note in the range
-    EXPECT_CALL(*m_playbackController, seekElement(oldContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(oldContext.element, FLUSH_SOUND))
     .Times(1);
 
     //! [WHEN] User pressed left mouse button with ShiftModifier on the new note
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPointF(100, 100)));
 }
 
 /**
@@ -565,43 +560,35 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Already_Selected_Range)
     //! [GIVEN] There is a test score
     engraving::MasterScore* score = engraving::ScoreRW::readScore(TEST_SCORE_PATH);
 
-    //! [GIVEN] Previous selected measure
-    INotationInteraction::HitElementContext oldContext = hitContext(score, { ElementType::MEASURE });
-
-    //! [GIVEN] User selected measure that is already selected
-    INotationInteraction::HitElementContext newContext = oldContext;
-    newContext.element->setSelected(true);
-
-    std::vector<EngravingItem*> selectedElements {
-        newContext.element
-    };
+    //! [GIVEN] User selects a measure
+    INotationInteraction::HitElementContext context = hitMeasureContext(score, 0 /*first measure*/);
 
     EXPECT_CALL(*m_interaction, hitElement(_, _))
-    .WillOnce(Return(newContext.element));
+    .WillOnce(Return(context.element));
 
     EXPECT_CALL(*m_interaction, hitStaff(_))
-    .WillOnce(Return(newContext.element->staff()));
+    .WillOnce(Return(context.staff));
 
     //! [GIVEN] The new hit element context with new measure will be set
-    EXPECT_CALL(*m_interaction, setHitElementContext(newContext))
+    EXPECT_CALL(*m_interaction, setHitElementContext(context))
     .Times(1);
 
     EXPECT_CALL(*m_interaction, hitElementContext())
-    .Times(2)
-    .WillOnce(ReturnRef(oldContext))
-    .WillOnce(ReturnRef(oldContext));
+    .Times(1)
+    .WillOnce(ReturnRef(context));
 
     //! [GIVEN] There is a range selection
     ON_CALL(*m_selection, isRange())
     .WillByDefault(Return(true));
-    ON_CALL(*m_selectionRange, containsPoint(_))
+    ON_CALL(*m_selectionRange, containsItem(context.element, _))
     .WillByDefault(Return(true));
 
+    std::vector<EngravingItem*> selectElements = { context.element };
     EXPECT_CALL(*m_selection, elements())
-    .WillOnce(ReturnRef(selectedElements));
+    .WillOnce(ReturnRef(selectElements));
 
     //! [THEN] We should seek measure from the range
-    EXPECT_CALL(*m_playbackController, seekElement(newContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(context.element, FLUSH_SOUND))
     .Times(1);
 
     //! [THEN] No selection change
@@ -609,7 +596,56 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Already_Selected_Range)
     .Times(0);
 
     //! [WHEN] User pressed left mouse button
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPointF(100, 100)));
+}
+
+/**
+ * @brief Mouse_Press_Shift_On_Selected_Selected_Range
+ * @details User pressed left mouse button with Shift on already selected range
+ *          This should result in a call to `select`, to extend/diminish the selection
+ */
+TEST_F(NotationViewInputControllerTests, Mouse_Press_Shift_On_Already_Selected_Range)
+{
+    //! [GIVEN] There is a test score
+    engraving::MasterScore* score = engraving::ScoreRW::readScore(TEST_SCORE_PATH);
+
+    //! [GIVEN] User selects a measure
+    INotationInteraction::HitElementContext context = hitMeasureContext(score, 0 /*first measure*/);
+
+    EXPECT_CALL(*m_interaction, hitElement(_, _))
+    .WillOnce(Return(context.element));
+
+    EXPECT_CALL(*m_interaction, hitStaff(_))
+    .WillOnce(Return(context.staff));
+
+    //! [GIVEN] The new hit element context with new measure will be set
+    EXPECT_CALL(*m_interaction, setHitElementContext(context))
+    .Times(1);
+
+    EXPECT_CALL(*m_interaction, hitElementContext())
+    .Times(1)
+    .WillOnce(ReturnRef(context));
+
+    //! [GIVEN] There is a range selection
+    ON_CALL(*m_selection, isRange())
+    .WillByDefault(Return(true));
+    ON_CALL(*m_selectionRange, containsItem(context.element, _))
+    .WillByDefault(Return(true));
+
+    //! [THEN] We should seek measure from the range
+    EXPECT_CALL(*m_playbackController, seekElement(context.element, FLUSH_SOUND))
+    .Times(1);
+
+    //! [THEN] Selection is extended/diminished
+    std::vector<EngravingItem*> selectElements = { context.element };
+    EXPECT_CALL(*m_interaction, select(selectElements, _, _))
+    .Times(1);
+
+    EXPECT_CALL(*m_selection, elements())
+    .WillOnce(ReturnRef(selectElements));
+
+    //! [WHEN] User pressed left mouse button
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPointF(100, 100)));
 }
 
 /**
@@ -638,10 +674,9 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Already_Selected_Element
     EXPECT_CALL(*m_interaction, setHitElementContext(newContext))
     .Times(1);
 
-    EXPECT_CALL(*m_interaction, hitElementContext())
-    .Times(2)
-    .WillOnce(ReturnRef(oldContext))
-    .WillOnce(ReturnRef(newContext));
+    std::vector<EngravingItem*> selectedElements = { oldContext.element };
+    ON_CALL(*m_selection, elements())
+    .WillByDefault(ReturnRef(selectedElements));
 
     //! [GIVEN] No note enter mode, no playing
     EXPECT_CALL(m_view, isNoteEnterMode())
@@ -651,15 +686,14 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Already_Selected_Element
     .WillByDefault(Return(false));
 
     //! [THEN] We will no select already selected note, but play and seek
-    std::vector<EngravingItem*> selectElements = { newContext.element };
-    EXPECT_CALL(*m_interaction, select(selectElements, _, _))
+    EXPECT_CALL(*m_interaction, select(_, _, _))
     .Times(0);
 
     std::vector<const EngravingItem*> playElements = { newContext.element };
-    EXPECT_CALL(*m_playbackController, playElements(playElements, false))
+    EXPECT_CALL(*m_playbackController, playElements(playElements, m_playParams, false))
     .Times(1);
 
-    EXPECT_CALL(*m_playbackController, seekElement(newContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(newContext.element, FLUSH_SOUND))
     .Times(1);
 
     //! [GIVEN] There is no a range selection
@@ -667,7 +701,7 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Already_Selected_Element
     .WillByDefault(Return(false));
 
     //! [WHEN] User pressed left mouse button with NoModifier on the new note
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::NoModifier, QPointF(100, 100)));
 }
 
 /**
@@ -696,8 +730,7 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range)
     .Times(1);
 
     EXPECT_CALL(*m_interaction, hitElementContext())
-    .Times(2)
-    .WillOnce(ReturnRef(oldContext))
+    .Times(1)
     .WillOnce(ReturnRef(newContext));
 
     //! [GIVEN] No note enter mode, no playing
@@ -710,7 +743,8 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range)
     //! [THEN] We will select new measure
     std::vector<EngravingItem*> selectElements = { newContext.element };
     EXPECT_CALL(*m_interaction, select(selectElements, _, _))
-    .Times(1);
+    .Times(1)
+    .WillOnce([newContext] { newContext.element->setSelected(true); });
 
     //! [GIVEN] There is a range selection with two measures
     ON_CALL(*m_selection, isRange())
@@ -718,11 +752,11 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range)
 
     //! [THEN] No play measure
     std::vector<const EngravingItem*> playElements = { newContext.element };
-    EXPECT_CALL(*m_playbackController, playElements(playElements, false))
+    EXPECT_CALL(*m_playbackController, playElements(playElements, m_playParams, false))
     .Times(0);
 
     //! [THEN] Because it's a range selection, we should start playing from first measure in the range
-    EXPECT_CALL(*m_playbackController, seekElement(oldContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(oldContext.element, FLUSH_SOUND))
     .Times(1);
 
     selectElements.push_back(oldContext.element);
@@ -730,7 +764,7 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range)
     .WillOnce(ReturnRef(selectElements));
 
     //! [WHEN] User pressed left mouse button with ShiftModifier on the new note
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPointF(100, 100)));
 }
 
 /**
@@ -743,8 +777,6 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range_Context_Menu)
     //! [GIVEN] There is a test score
     engraving::MasterScore* score = engraving::ScoreRW::readScore(TEST_SCORE_PATH);
 
-    INotationInteraction::HitElementContext oldContext;
-
     //! [GIVEN] User selected a measure
     INotationInteraction::HitElementContext selectMeasureContext = hitMeasureContext(score, 0 /*first measure*/);
 
@@ -768,12 +800,13 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range_Context_Menu)
     .WillOnce(Return());
 
     EXPECT_CALL(*m_interaction, hitElementContext())
-    .WillOnce(ReturnRef(oldContext))
     .WillOnce(ReturnRef(selectMeasureContext))
     //! right button click
-    .WillOnce(ReturnRef(selectMeasureContext))
     .WillOnce(ReturnRef(contextMenuOnMeasureContext))
-    .WillOnce(ReturnRef(contextMenuOnMeasureContext)); // for context menu
+#if QT_VERSION < QT_VERSION_CHECK(6, 9, 0)
+    .WillOnce(ReturnRef(contextMenuOnMeasureContext)) // for context menu
+#endif
+    ;
 
     //! [GIVEN] No note enter mode, no playing
     EXPECT_CALL(m_view, isNoteEnterMode())
@@ -785,23 +818,20 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range_Context_Menu)
     //! [THEN] We will select new measure only one time
     std::vector<EngravingItem*> selectElements = { selectMeasureContext.element };
     EXPECT_CALL(*m_interaction, select(selectElements, _, _))
-    .Times(1)
-    .WillOnce(Return());
+    .Times(1);
 
     EXPECT_CALL(*m_selection, isRange())
     .WillRepeatedly(Return(true));
 
-    //! [THEN] New element is initially not in selected range,
-    //!        but is in selected range after performing selection
+    //! [GIVEN] Element is in selected range at the moment that it is selected for the second time
     EXPECT_CALL(*m_selectionRange, containsItem(contextMenuOnMeasureContext.element, _))
-    .WillOnce(Return(false))
     .WillOnce(Return(true));
 
     std::vector<const EngravingItem*> playElements = { selectMeasureContext.element };
-    EXPECT_CALL(*m_playbackController, playElements(playElements, false))
+    EXPECT_CALL(*m_playbackController, playElements(playElements, m_playParams, false))
     .Times(0);
 
-    EXPECT_CALL(*m_playbackController, seekElement(selectMeasureContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(selectMeasureContext.element, FLUSH_SOUND))
     .WillOnce(Return())
     //! right button click
     .WillOnce(Return());
@@ -810,15 +840,19 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range_Context_Menu)
     EXPECT_CALL(*m_selection, elements())
     .WillRepeatedly(ReturnRef(selectElements));
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 9, 0)
+    //! Note: the context menu itself is shown by AbstractNotationPaintView::event
+#else
     //! [THEN] Show context menu for measure
     EXPECT_CALL(m_view, showContextMenu(contextMenuOnMeasureContext.element->type(), _))
     .Times(1);
+#endif
 
     //! [WHEN] User pressed left mouse button with ShiftModifier on the new measure
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPointF(100, 100)));
 
     //! [WHEN] User pressed right mouse button with NoModifier on the selected measure
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::RightButton, Qt::NoModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::RightButton, Qt::NoModifier, QPointF(100, 100)));
 }
 
 /**
@@ -831,8 +865,6 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range_Context_Menu_New_S
     //! [GIVEN] There is a test score
     engraving::MasterScore* score = engraving::ScoreRW::readScore(TEST_SCORE_PATH);
 
-    INotationInteraction::HitElementContext oldContext;
-
     //! [GIVEN] User selected a measure
     INotationInteraction::HitElementContext selectMeasureContext = hitMeasureContext(score, 0 /*first measure*/);
 
@@ -855,12 +887,13 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range_Context_Menu_New_S
     .WillOnce(Return());
 
     EXPECT_CALL(*m_interaction, hitElementContext())
-    .WillOnce(ReturnRef(oldContext))
     .WillOnce(ReturnRef(selectMeasureContext))
     //! right button click
-    .WillOnce(ReturnRef(selectMeasureContext))
     .WillOnce(ReturnRef(contextMenuOnMeasureContext))
-    .WillOnce(ReturnRef(contextMenuOnMeasureContext));     // for context menu
+#if QT_VERSION < QT_VERSION_CHECK(6, 9, 0)
+    .WillOnce(ReturnRef(contextMenuOnMeasureContext)) // for context menu
+#endif
+    ;
 
     //! [GIVEN] No note enter mode, no playing
     EXPECT_CALL(m_view, isNoteEnterMode())
@@ -875,7 +908,7 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range_Context_Menu_New_S
     //! [THEN] The selection should be changed
     std::vector<EngravingItem*> selectElements = { selectMeasureContext.element };
     EXPECT_CALL(*m_interaction, select(selectElements, _, _))
-    .WillOnce(Return());
+    .Times(1);
 
     std::vector<EngravingItem*> contextMenuSelectElements = { contextMenuOnMeasureContext.element };
     EXPECT_CALL(*m_interaction, select(contextMenuSelectElements, _, _))
@@ -885,31 +918,31 @@ TEST_F(NotationViewInputControllerTests, Mouse_Press_On_Range_Context_Menu_New_S
     .WillOnce(ReturnRef(selectElements))
     .WillOnce(ReturnRef(contextMenuSelectElements));
 
-    //! [THEN] Old element is not in selected range at the moment that it is selected
-    EXPECT_CALL(*m_selectionRange, containsItem(selectMeasureContext.element, _))
-    .WillOnce(Return(false));
-
-    //! [THEN] New element is not in selected range at the moment that it is selected
+    //! [GIVEN] New element is not in selected range at the moment that it is selected
     EXPECT_CALL(*m_selectionRange, containsItem(contextMenuOnMeasureContext.element, _))
     .WillOnce(Return(false));
 
-    EXPECT_CALL(*m_playbackController, playElements(_, false))
+    EXPECT_CALL(*m_playbackController, playElements(_, m_playParams, false))
     .Times(0);
 
     //! [THEN] We will seek each measures
-    EXPECT_CALL(*m_playbackController, seekElement(selectMeasureContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(selectMeasureContext.element, FLUSH_SOUND))
     .WillOnce(Return());
 
-    EXPECT_CALL(*m_playbackController, seekElement(contextMenuOnMeasureContext.element))
+    EXPECT_CALL(*m_playbackController, seekElement(contextMenuOnMeasureContext.element, FLUSH_SOUND))
     .WillOnce(Return());
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 9, 0)
+    //! Note: the context menu itself is shown by AbstractNotationPaintView::event
+#else
     //! [THEN] Show context menu for new measure
     EXPECT_CALL(m_view, showContextMenu(contextMenuOnMeasureContext.element->type(), _))
     .Times(1);
+#endif
 
     //! [WHEN] User pressed left mouse button with ShiftModifier on the new measure
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::LeftButton, Qt::ShiftModifier, QPointF(100, 100)));
 
     //! [WHEN] User pressed right mouse button with NoModifier on the selected measure
-    m_controller->mousePressEvent(make_mousePressEvent(Qt::RightButton, Qt::NoModifier, QPoint(100, 100)));
+    m_controller->mousePressEvent(make_mousePressEvent(Qt::RightButton, Qt::NoModifier, QPointF(100, 100)));
 }
