@@ -42,6 +42,8 @@
 #include "translation.h"
 #include "types/uri.h"
 
+#include "app_config.h"
+
 using namespace muse;
 using namespace mu::palette;
 using namespace mu::engraving;
@@ -338,27 +340,27 @@ bool UserPaletteController::move(const QModelIndex& sourceParent, int sourceRow,
 }
 
 void UserPaletteController::showHideOrDeleteDialog(const std::string& question,
-                                                   std::function<void(AbstractPaletteController::RemoveAction)> resultHandler)
-const
+                                                   std::function<void(AbstractPaletteController::RemoveAction)> resultHandler) const
 {
     int hideButton = int(IInteractive::Button::CustomButton) + 1;
     int deleteButton = hideButton + 1;
 
-    IInteractive::Result result = interactive()->question(std::string(), question, {
+    auto result = interactive()->question(std::string(), question, {
         IInteractive::ButtonData(hideButton, muse::trc("palette", "Hide")),
         IInteractive::ButtonData(deleteButton, muse::trc("palette", "Delete permanently")),
         interactive()->buttonData(IInteractive::Button::Cancel)
     });
 
-    RemoveAction action = RemoveAction::NoAction;
+    result.onResolve(this, [deleteButton, hideButton, resultHandler](const IInteractive::Result& res) {
+        RemoveAction action = RemoveAction::NoAction;
+        if (res.isButton(deleteButton)) {
+            action = RemoveAction::DeletePermanently;
+        } else if (res.isButton(hideButton)) {
+            action = RemoveAction::Hide;
+        }
 
-    if (result.button() == deleteButton) {
-        action = RemoveAction::DeletePermanently;
-    } else if (result.button() == hideButton) {
-        action = RemoveAction::Hide;
-    }
-
-    resultHandler(action);
+        resultHandler(action);
+    });
 }
 
 void UserPaletteController::queryRemove(const QModelIndexList& removeIndices, int customCount)
@@ -393,16 +395,15 @@ void UserPaletteController::queryRemove(const QModelIndexList& removeIndices, in
                                    ? muse::trc("palette", "Do you want to permanently delete this custom palette cell?")
                                    : muse::trc("palette", "Do you want to permanently delete these custom palette cells?");
 
-            IInteractive::Result result = interactive()->question(std::string(), question, {
+            interactive()->question(std::string(), question, {
                 IInteractive::Button::Yes,
                 IInteractive::Button::No
+            })
+            .onResolve(this, [this, removeIndices](const IInteractive::Result& res) {
+                if (res.isButton(IInteractive::Button::Yes)) {
+                    remove(removeIndices, RemoveAction::DeletePermanently);
+                }
             });
-
-            if (result.standardButton() == IInteractive::Button::Yes) {
-                remove(removeIndices, RemoveAction::DeletePermanently);
-            }
-
-            return;
         }
     } else {
         if (visible) {
@@ -415,9 +416,9 @@ void UserPaletteController::queryRemove(const QModelIndexList& removeIndices, in
         } else {
             action = RemoveAction::Hide;
         }
-    }
 
-    remove(removeIndices, action);
+        remove(removeIndices, action);
+    }
 }
 
 void UserPaletteController::remove(const QModelIndexList& unsortedRemoveIndices,
@@ -822,7 +823,13 @@ bool PaletteProvider::addPalette(const QPersistentModelIndex& index)
 
     if (index.model() == m_masterPaletteModel) {
         QMimeData* data = m_masterPaletteModel->mimeData({ QModelIndex(index) });
-        const bool success = m_userPaletteModel->dropMimeData(data, Qt::CopyAction, 0, 0, QModelIndex());
+        QModelIndex dropIndex = m_userPaletteModel->index(0, 0);
+
+        const bool success = m_userPaletteModel->dropMimeData(data, Qt::CopyAction, dropIndex.row(), dropIndex.column(), QModelIndex());
+        if (success) {
+            m_userPaletteModel->setData(dropIndex, true, PaletteTreeModel::VisibleRole);
+        }
+
         data->deleteLater();
         return success;
     }
@@ -830,48 +837,27 @@ bool PaletteProvider::addPalette(const QPersistentModelIndex& index)
     return false;
 }
 
-bool PaletteProvider::removeCustomPalette(const QPersistentModelIndex& index)
+void PaletteProvider::resetPalette(const QModelIndex& index)
 {
     if (!index.isValid()) {
-        return false;
+        return;
     }
 
-    if (index.model() == m_userPaletteModel) {
-        const bool custom = index.data(PaletteTreeModel::CustomRole).toBool();
-        if (!custom) {
-            return false;
+    std::string title = muse::trc("palette",
+                                  "Do you want to restore this palette to its default state? All changes to this palette will be lost.");
+
+    interactive()->question("", title, {
+        IInteractive::Button::No, IInteractive::Button::Yes
+    })
+    .onResolve(this, [this, index](const IInteractive::Result& res) {
+        if (res.isButton(IInteractive::Button::Yes)) {
+            doResetPalette(index);
         }
-
-        IInteractive::Result result
-            = interactive()->question("", muse::trc("palette", "Do you want to permanently delete this custom palette?"), {
-            IInteractive::Button::Yes, IInteractive::Button::No
-        });
-
-        if (result.standardButton() == IInteractive::Button::Yes) {
-            return m_userPaletteModel->removeRow(index.row(), index.parent());
-        }
-
-        return false;
-    }
-
-    return false;
+    });
 }
 
-bool PaletteProvider::resetPalette(const QModelIndex& index)
+void PaletteProvider::doResetPalette(const QModelIndex& index)
 {
-    if (!index.isValid()) {
-        return false;
-    }
-
-    IInteractive::Result result
-        = interactive()->question("", muse::trc("palette",
-                                                "Do you want to restore this palette to its default state? All changes to this palette will be lost."), {
-        IInteractive::Button::No, IInteractive::Button::Yes
-    });
-    if (result.standardButton() != IInteractive::Button::Yes) {
-        return false;
-    }
-
     Q_ASSERT(m_defaultPaletteModel != m_userPaletteModel);
 
     QAbstractItemModel* resetModel = nullptr;
@@ -897,7 +883,7 @@ bool PaletteProvider::resetPalette(const QModelIndex& index)
     const bool wasExpanded = index.data(PaletteTreeModel::PaletteExpandedRole).toBool();
 
     if (!m_userPaletteModel->removeRow(row, parent)) {
-        return false;
+        return;
     }
 
     if (resetIndex.isValid()) {
@@ -911,13 +897,11 @@ bool PaletteProvider::resetPalette(const QModelIndex& index)
     const QModelIndex newIndex = m_userPaletteModel->index(row, column, parent);
     m_userPaletteModel->setData(newIndex, wasVisible, PaletteTreeModel::VisibleRole);
     m_userPaletteModel->setData(newIndex, wasExpanded, PaletteTreeModel::PaletteExpandedRole);
-
-    return true;
 }
 
 QString PaletteProvider::getPaletteFilename(bool open, const QString& name) const
 {
-    QString title;
+    std::string title;
     std::vector<std::string> filter;
 #ifdef WIN_PORTABLE
     QString wd = QDir::cleanPath(QString("%1/../../../Data/settings").arg(QCoreApplication::applicationDirPath()));
@@ -926,10 +910,10 @@ QString PaletteProvider::getPaletteFilename(bool open, const QString& name) cons
                  .arg(QCoreApplication::applicationName());
 #endif
     if (open) {
-        title  = muse::qtrc("palette", "Load palette");
+        title  = muse::trc("palette", "Load palette");
         filter = { muse::trc("palette", "MuseScore Studio palette") + " (*.mpal)" };
     } else {
-        title  = muse::qtrc("palette", "Save palette");
+        title  = muse::trc("palette", "Save palette");
         filter = { muse::trc("palette", "MuseScore Studio palette") + " (*.mpal)" };
     }
 
@@ -946,9 +930,9 @@ QString PaletteProvider::getPaletteFilename(bool open, const QString& name) cons
 
     muse::io::path_t fn;
     if (open) {
-        fn = interactive()->selectOpeningFile(title, defaultPath, filter);
+        fn = interactive()->selectOpeningFileSync(title, defaultPath, filter);
     } else {
-        fn = interactive()->selectSavingFile(title, defaultPath, filter);
+        fn = interactive()->selectSavingFileSync(title, defaultPath, filter);
     }
     return fn.toQString();
 }
