@@ -447,8 +447,10 @@ void PlaybackController::onAudioResourceChanged(const TrackId trackId, const Ins
     if (audio::isOnlineAudioResource(newMeta)) {
         addToOnlineSounds(trackId, newMeta);
         tours()->onEvent(u"online_sounds_added");
+        notationPlayback->setSendEventsOnScoreChange(instrumentTrackId, true);
     } else if (audio::isOnlineAudioResource(oldMeta)) {
         removeFromOnlineSounds(trackId);
+        notationPlayback->setSendEventsOnScoreChange(instrumentTrackId, false);
     }
 }
 
@@ -622,6 +624,8 @@ void PlaybackController::togglePlay()
     if (isPlaying()) {
         pause();
     } else if (isPaused()) {
+        notationPlayback()->sendEventsForChangedTracks();
+
         if (currentPlayer()) {
             secs_t pos = currentPlayer()->playbackPosition();
             secs_t endSecs = playbackEndSecs();
@@ -633,6 +637,8 @@ void PlaybackController::togglePlay()
             resume();
         }
     } else {
+        notationPlayback()->sendEventsForChangedTracks();
+
         play();
     }
 }
@@ -707,6 +713,20 @@ void PlaybackController::resume()
     currentPlayer()->resume(delay);
 }
 
+void PlaybackController::onPlaybackStatusChanged()
+{
+    if (!notationPlayback()) {
+        return;
+    }
+
+    bool playing = isPlaying();
+
+    for (const auto& pair : m_instrumentTrackIdMap) {
+        bool shouldSendOnScoreChange = playing || muse::contains(m_onlineSounds, pair.second);
+        notationPlayback()->setSendEventsOnScoreChange(pair.first, shouldSendOnScoreChange);
+    }
+}
+
 secs_t PlaybackController::playbackStartSecs() const
 {
     if (!m_notation) {
@@ -1097,6 +1117,10 @@ void PlaybackController::doAddTrack(const InstrumentTrackId& instrumentTrackId,
 
         if (muse::audio::isOnlineAudioResource(appliedParams.in.resourceMeta)) {
             addToOnlineSounds(trackId, appliedParams.in.resourceMeta);
+
+            if (notationPlayback()) {
+                notationPlayback()->setSendEventsOnScoreChange(instrumentTrackId, true);
+            }
         }
     })
     .onReject(this, [instrumentTrackId, onFinished](int code, const std::string& msg) {
@@ -1552,6 +1576,7 @@ void PlaybackController::setupSequencePlayer()
 
     currentPlayer()->playbackStatusChanged().onReceive(this, [this](PlaybackStatus) {
         m_isPlayingChanged.notify();
+        onPlaybackStatusChanged();
     });
 
     currentPlayer()->setDuration(secsToMilisecs(notationPlayback()->totalPlayTime()));
